\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\-----------------------------------------------------------------------------
\\ Kevin Ryde, August 2021
\\
\\ Usage; gp <a346913-rows.gp
\\
\\ This is some PARI/GP code implementing the successor function (step to
\\ next row) of Beyer and Hedetniemi's rooted tree level sequences A346913
\\ and Knuth's corresponding vpar forests form A346914.
\\
\\ Both are efficient ways to iterate through rooted trees or rooted forests
\\ in levels form or vpar form.

default(recover,0)
default(strictargs,1)


\\-----------------------------------------------------------------------------
\\ A346913_next_levels()

\\ Global variable "levels" is a vector containing a rooted tree levels
\\ sequence which is a row from A346913.  A346913_next_levels() changes
\\ levels[] to the next row of the sequence.

levels = [1];   \\ first row, n=1

A346913_next_levels() =
{
  forstep(p=#levels,3,-1,
     levels[p]>=3 || next;         \\ find p = end-most with a grandparent
     forstep(q=p-1,2,-1,
        levels[q]<levels[p] || next;  \\ find q = parent of p

        \\ change levels[p..#levels] to copies of levels[q .. p-1],
        \\ including partial copy at the end if necessary
        while(p<=#levels,
              levels[p] = levels[q];
              p++; q++);
        return(1)));  \\ have stepped to new tree of same num vertices

  \\ no vertex with level>=3 exists, so no more of this #levels length,
  \\ go to the first of the next longer length
  levels = [1..#levels+1];
  0;  \\ new num vertices
}

{
  print("A346913 levels sequences:");
  for(n=1,18,  \\ OFFSET=1, n=1 is the first row
     printf("  row n=%2d   %s\n",n,levels);
     A346913_next_levels());
  print();
}

\\ Beyer and Hedetniemi show the work done in the successor function is
\\ constant amortized time, meaning time per tree, averaged over all the
\\ trees of a given N vertices, has a constant upper bound.
\\
\\ Of course in an interpreted language time is taken by all sorts of
\\ overheads, but this is an algorithmic complexity measure.
\\
\\ levels[] as a global variable would be undesirable in a real program,
\\ but here it's a convenient way to let A346913_next_levels() replace the
\\ vector with a new longer vector when needed, and so iterate through all
\\ rows of the sequence.
\\
\\ In a real program, a possibility would be call-by-reference (new in GP
\\ 2.13) so the subroutine can modify vector contents.  It could return a
\\ flag to indicate there's no next vector of this length (like the return
\\ 0 or 1 in A346913_next_levels()).
\\
\\ There's actually a decent chance an application is interested in trees of
\\ a given N vertices anyway, so it can be left to the caller to start a new
\\ iteration for each desired N.  The first tree of N vertices is path down,
\\
\\     levels = [1..8]        \\ first tree of 8 vertices
\\
\\ Another possibility would be List() type and call-by-reference.  That
\\ lets the subroutine modify the contents and also lengthen with listput().
\\ However, as of PARI circa version 2.13, the List() type is not accepted
\\ in a few places (eg. lex() comparisons) so is likely to be less
\\ convenient than an ordinary vector.


\\-----------------------------------------------------------------------------
\\ A346914_next_vpar()

\\ Global variable "vpar" is a vector which is the vertex parent array of a
\\ rooted forest, being a row from A346914.  A346914_next_vpar() changes
\\ vpar to the next row of A346914.

vpar = [0];    \\ first row, n=2

A346914_next_vpar() =
{
  forstep(p=#vpar,2,-1,  \\ find end-most vertex p which has a parent
     my(q=vpar[p]);      \\ q = vpar[p] = parent of p
     q || next;          \\ but 0 means no parent

     \\ replace p..#vpar with copies of block vpar[q .. p-1],
     \\ with suitable offset added for their new location in vpar,
     \\ and except each copy of vpar[q] is unchanged (stays value g)
     my(g=vpar[q],       \\ parent of q
        offset=p-q);
     while(p<=#vpar,
           vpar[p] = vpar[q] + if(vpar[q]!=g, offset);
           p++; q++);
        return(1));  \\ have stepped to new forest of same num vertices

  \\ no vertex has a parent, so no more forests of this many vertices,
  \\ go to the first of the next greater number of vertices (#vpar+1)
  vpar = [0..#vpar];
  0;  \\ new num vertices
}

{
  print("A346914 vpar rows (vertex parent arrays):");
  for(n=2,18,  \\ OFFSET=2, n=2 is first row
     printf("  row n=%2d   %s\n",n,vpar);
     A346914_next_vpar());
  print();
}

\\ A346914_next_vpar() is constant amortized time the same as
\\ A346913_next_levels().  See Knuth's exercise A346915 on the mean
\\ work done in vpar form.
\\
\\ vpar form allows q=vpar[p] to get q the parent of p directly rather than
\\ a search back through levels[].  But the vpar block copies must apply an
\\ offset for the new locations.


\\-----------------------------------------------------------------------------
\\ Tree Levels -> Forest Vpar
\\
\\ Row numbers n in A346913 and A346914 are chosen so tree n in A346913
\\ corresponds to forest n in A346914 by deleting the root of the tree to
\\ leave the rest as a forest.
\\
\\ This equivalence between levels and vpar is illustrated by the following
\\ conversion.  The direction is vpar -> levels since that's a touch easier
\\ than the converse.  At vpar vertex v, can refer back in the levels
\\ sequence so-far to get the level of the parent of v, and v is then 1
\\ level deeper.
\\
\\ The levels vector has a new root vertex v=1 added above the forest, so
\\ vpar vertices 1..N move up to levels vertices 2..N+1.

vpar_forest_to_tree_levels(vpar) =
{
  my(levels=vector(#vpar+1));
  levels[1]=1;       \\ root of tree
  for(v=1,#vpar,
     my(p=vpar[v]);  \\ parent, or 0 if a root
     levels[v+1] = levels[p+1]+1);
  levels;
}

{
  \\ Iterate for a while and check levels equivalent to vpar.
  \\ A346914_next_vpar() allows vpar=[] for an empty row n=1 and steps it
  \\ to next row n=2 vpar=[0].
  my(count=0);
  levels=[1];   \\ initial tree, row n=1
  vpar=[];      \\ initial equivalent forest, empty row n=1
  for(n=1,1000,
     levels == vpar_forest_to_tree_levels(vpar)  || error();
     count++;
     A346913_next_levels();
     A346914_next_vpar());
  print(count," levels <-> vpar checks");
}


\\-----------------------------------------------------------------------------
\\ Lexicographically Decreasing Rows

\\ Levels sequences of given N vertices are in lexicographically
\\ decreasing order.
{
  levels=[1];   \\ first levels vector
  my(count=0);
  for(n=1,1000,
     my(prev=levels);
     A346913_next_levels();
     if(#prev == #levels,
        lex(prev,levels) > 0  || error();   \\ decreasing
        count++));
  print(count," levels lex decreasing checks");
}

\\ vpar vectors of given N vertices are also in lexicographically
\\ decreasing order too.
\\
\\ In general for any pre-order tree or forest, lex comparison of levels
\\ sequences gives the same as lex comparison of the corresponding vpars.
{
  vpar=[0];   \\ initial forest
  my(count=0);
  for(n=1,1000,
     my(prev=vpar);
     A346914_next_vpar();
     if(#prev == #vpar,
        lex(prev,vpar) > 0  || error();    \\ decreasing
        count++));
  print(count," vpar lex decreasing checks");
}


\\-----------------------------------------------------------------------------
\\ Lexicographically Descending Siblings

\\ The canonical levels sequence form used by Beyer and Hedetniemi is the
\\ lexicographically greatest levels sequence among all possible orderings
\\ of sibling subtrees.
\\
\\ This is equivalent to everywhere sibling subtrees in descending order of
\\ their respective subtree levels sub-sequences.  For example, tree
\\
\\     1..................           level 1, root
\\     |   \      \       \
\\     2     6     10      14        level 2, children of root
\\     |     |\     | \     |
\\     3     7 9   11 13   15        level 3
\\     |\    |      |
\\     4 5   8     12                level 4
\\
\\ is the following levels sequence (A346913 row n=140193),
\\
\\     vertex     1  2         6        10        14
\\     levels = [ 1, 2,3,4,4,  2,3,4,3,  2,3,4,3,  2,3 ]
\\                   \-----/   \-----/   \-----/   \-/
\\                   subtree   subtree   subtree  subtree
\\                      2         6        10       14
\\
\\ Vertices 2,6,10,14 are siblings.  Subtree 2 means vertex 2 and all
\\ vertices below it.  Notice its subtree levels sequence 2,3,4,4 is
\\ lexicographically greater than subtree 6 which is only 2,3,4,3.
\\ Subtree 6 is equal to subtree 10.  Subtree 10 is lexicographically
\\ greater than subtree 14.  (Longer means greater when terms are
\\ otherwise equal, as usual.)
\\
\\ The following code checks sibling subtrees are in descending order
\\ throughout a tree.  This code is offered only as an illustration of the
\\ condition.  The actual amount of work needed to recognise good canonical
\\ levels (or good canonical vpar) is much less.

\\ levels is a vector and v is a vertex in it.
\\ Return the vertex number of the next greater sibling of v,
\\ or return 0 if v has no greater sibling.
levels_next_sibling(levels,v) =
{
  for(i=v+1,#levels,
     if(levels[i] < levels[v],  \\ something up above,
        return(0));             \\   v has no next sibling
     if(levels[i] == levels[v], \\ next of same level
        return(i)));            \\   is sibling of v
  0;  \\ v subtree extends through to the end of levels[], no next sibling
}

\\ levels is a levels sequence vector and v is a vertex in it.
\\ Extract from levels the sub-vector of levels which are the subtree
\\ comprising vertex v and all vertices below v.
levels_get_subtree(levels,v) =
{
  for(i=v+1,#levels,
     if(levels[i] <= levels[v],    \\ i is next sibling, or something higher,
        return(levels[v..i-1])));  \\ so end of subtree below v
  \\ subtree below v goes all the way to the end of levels[]
  levels[v..#levels];
}

levels_validate_siblings_are_descending(levels) =
{
  for(v=1,#levels,
     my(s=levels_next_sibling(levels,v));
     if(s==0, next);  \\ v has no next sibling

     my(v_subtree_levels=levels_get_subtree(levels,v),
        s_subtree_levels=levels_get_subtree(levels,s));
     \\ expect v >= s
     lex(v_subtree_levels, s_subtree_levels) >= 0 || error());
}
{
  \\ example tree in the comments above
  \\ vertices:  1  2         6        10        14
  my(levels = [ 1, 2,3,4,4,  2,3,4,3,  2,3,4,3,  2,3 ]);
  levels_next_sibling(levels, 2) == 6    || error();
  levels_next_sibling(levels, 6) == 10   || error();
  levels_next_sibling(levels, 10) == 14  || error();
  levels_next_sibling(levels, 14) == 0   || error();
  levels_get_subtree(levels, 2)  == [2,3,4,4]  || error();
  levels_get_subtree(levels, 6)  == [2,3,4,3]  || error();
  levels_get_subtree(levels, 10) == [2,3,4,3]  || error();
  levels_get_subtree(levels, 14) == [2,3]      || error();
  lex([2,3,4,4], [2,3,4,3]) >= 0  || error();
  lex([2,3,4,3], [2,3])     >= 0  || error();
  levels_validate_siblings_are_descending(levels);
}
{
  \\ iterate for a while and check
  levels = [1];  \\ first levels vector
  for(i=1,1000,
     levels_validate_siblings_are_descending(levels);
     count++;
     A346913_next_levels());
  print(count," sibling subtrees lex descending checks");
}


\\-----------------------------------------------------------------------------

\\ My vpar.gp code includes a vpar_unlabelled_next() with a choice of
\\ iterating vpar rooted trees or rooted forests.  A346914_next_vpar()
\\ can be adapted to iterate only trees by changing
\\
\\     q || next;
\\
\\ to restrict
\\
\\     q>1 || next;
\\
\\ This requires q is not the tree root (not q=1), so that raising p up to
\\ the same depth level as q doesn't cause p to become another root.
\\
\\ My vpar.gp also includes a vpar_unlabelled_prev() for the predecessor
\\ operation, but this is more complicated and examines more of the tree.
\\ It wants to increase the depth level of the end-most vertex p which is
\\ able to go deeper, but must be careful not to violate the canonical form
\\ by causing a subtree to become greater than its preceding sibling.
\\ Subtree checks must be made for each ancestor of a prospective deeper p.
\\ Iterating instead in lex decreasing order as per Beyer and Hedetniemi is
\\ the easier direction.

\\-----------------------------------------------------------------------------
print("end");
