\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\-----------------------------------------------------------------------------

\\ Kevin Ryde, September 2021
\\
\\ Usage; gp <a347620.gp
\\
\\ Calculate terms of A347620, being the position where Matula-Goebel tree
\\ number n is located when Matula-Goebel numbers are sorted by number of
\\ vertices then numerically.  That sort order is A061773 and is a
\\ permutation of the natural numbers.  A347620 here is its inverse

default(recover,0)
default(strictargs,1)


\\ Return a vector of length "len" where ret[n] = A000081(n) = number of
\\ rooted trees of n vertices.
num_trees_vector(len) =
{
  my(ret=vector(len,i,1),
     c=vector(len-1));      \\ c[n] = A209397(n) helper
  for(n=1,len-1,
     c[n] = sumdiv(n,d, ret[d]*d);
     ret[n+1] = sum(i=1,n, ret[i]*c[n-i+1]) / n);
  ret;
}

\\ Return a vector of length "len" where ret[n] = A087803(n-1) = the number
\\ of rooted trees of < n vertices, ie. how many trees of 1..n-1 vertices.
num_smaller_trees_vector(len) =
{
  if(len==0, [],
     my(ret=concat(0,num_trees_vector(len-1)));
     for(i=3,#ret, ret[i]+=ret[i-1]);  \\ cumulative
     ret);
}

\\ Return a vector of length "len" where ret[n] = A061775(n) = number of
\\ vertices in the tree of Matula-Goebel number n.
num_vertices_vector(len) =
{
  my(ret=vector(len),
     pi=0);               \\ number primes seen so far
  forfactored(elem=1,len,
     my(n,f); [n,f]=elem;
     ret[n] = if(matsize(f)[1]==1 && f[1,2]==1,
                 ret[pi++],                                    \\ n prime
                 vecextract(ret,f[,1])*f[,2] - vecsum(f[,2]))  \\ n composite
              + 1);
  ret;
}

\\ Return a vector of length "len" where ret[n] = A347620(n).
A347620_vector(len) =
{
  my(ret=num_vertices_vector(len),
     upto=num_smaller_trees_vector(vecmax(ret)));
  for(i=1,#ret,
     ret[i] = upto[ret[i]]++);  \\ ret becomes position
  ret;
}

{
  my(len=20);

  print1("M-G code   n = ");
  for(i=1,len, printf("%2d,",i));
  print("...");

  my(num_vertices=num_vertices_vector(len));
  print1("num vertices = ");
  for(i=1,#num_vertices, printf("%2d,",num_vertices[i]));
  print("...");

  my(pos=A347620_vector(len));
  print1("A347620(n)   = ");
  for(i=1,#pos, printf("%2d,",pos[i]));
  print("...");
}


\\ Notes:
\\
\\ num_trees_vector() is a usual implementation by successive Euler
\\ transforms, with vector c[] holding terms A209397 used multiple times.
\\
\\
\\ num_vertices_vector() uses the prime factorization of n to get the sizes
\\ of the child subtrees at the root, and their multiplicity.  If n is
\\ composite then previously calculated value ret[p] for each prime factor p
\\ gives the subtree size, but counting also a root above it (a "planted"
\\ tree).  Those roots above, with multiplicity, are excluded by vecsum()
\\ subtract.
\\
\\ There'd be a few equivalent ways to sum the quantities with multiplicity.
\\ vecextract() and subtract vecsum() used here measures a little faster
\\ than making ret[p]-1 terms.  This will only be an interpreted language
\\ thing where it's best to let builtin functions do the work.
\\
\\ If n is prime then the tree size is ret[pi]+1 for a new root above the
\\ previously calculated ret[pi], where pi = primepi(n).  This pi is made by
\\ keeping a counter pi.  (The primepi() function would do the same, but
\\ already have the information needed to just count.)
\\ 
\\
\\ A347620_vector() uses the formula shown in the sequence.  Each n has some
\\ number k vertices and then a(n) = A087803(k-1) + s, where s is how many
\\ of k seen so far, inclusive of this n.  upto[k] holds the s for each k.
\\
\\ Terms of A347620 could be output progressively if desired by running the
\\ forfactored() loop to get num_vertices, and grow the num_trees[] and in
\\ turn upto[] vectors on an as-needed basis.  This would still require
\\ keeping most of the num_vertices[] vector calculated so far, ready to get
\\ num_vertices[p] for subtrees.  A given prime p will be wanted again at n
\\ = 2*p and onwards.


\\-----------------------------------------------------------------------------
print("end");