\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\-----------------------------------------------------------------------------

\\ Kevin Ryde, September 2021
\\
\\ Usage; gp <a347966.gp
\\
\\ Calculate terms of A347966(n) = number of maximal matchings in the tree
\\ with Matula-Goebel number n.

default(recover,0)
default(strictargs,1)


\\ A maximal matching is a matching in which no pair of adjacent vertices is
\\ unmatched.  This is maximal in the sense that no pair can be added to the
\\ matching to increase its size.
\\
\\ Maximal matchings include all the maximum matchings (A347967) and there
\\ may be more which are maximal and yet not the maximum size.
\\
\\
\\ Maximal matchings in a tree can be counted by calculating 3 quantities at
\\ each vertex v, representing matchings in the subtree comprising v and
\\ everything below v (all its descendants).
\\
\\     with = how many maximal matchings where v matched.
\\
\\     without_max = how many maximal matchings where v not matched.
\\
\\     without = how many matchings where v not matched and which might or
\\       might not be maximal, but will be made maximal if v is paired to
\\       another vertex later.  This is all of "without_max" plus possible
\\       additional matchings.
\\
\\ The final result is
\\
\\     maximals = with + without_max
\\
\\ When a child subtree c is added underneath v, these quantities calculated
\\ on the c subtree can be accumulated into the quantities at v.
\\
\\ New "without_max" is the simplest.  The child c must be matched already,
\\ otherwise adjacent v and c both unmatched is not maximal.  So all
\\ combinations of v_without_max and c_with,
\\
\\     new_without_max = v_without_max * c_with
\\
\\ New "without" is similar, but the child root c can be unmatched as long
\\ as c is maximal.  Hence
\\
\\     c_max = c_with + c_without_max
\\     new_without = v_without * c_max
\\
\\ The c_without_max combinations are v and c both unmatched so are the
\\ "v must pair with something else later" type within new_without.
\\
\\ New "with" can be either v and c both already matched, or v and c both
\\ unmatched and made a new pair.  Unmatched v and c are both "without" type
\\ since they gain the pairing which "without" demands.
\\
\\     new_with =   v_with * c_max
\\                + v_without * c_without
\\
\\
\\ This type of accumulation of quantities is a generic approach.  It can be
\\ applied starting at leaves of a tree and working inwards to any desired
\\ final vertex -- which in the code here is the root.
\\
\\ The leaves as starting points are singleton vertices (nothing below) and
\\ their quantities are
\\
\\     with        = 0
\\     without_max = 1      \ the empty matching is maximal
\\     without     = 1      / and is the only possible
\\
\\
\\ The code uses recursion to get child subtree quantities.  It's expected
\\ that maximum depth in the tree, and hence maximum recursion level, will
\\ be modest for a tree in the range which factor() and primepi() are able
\\ to decode.
\\
\\ "rep" is the number of repetitions of a given child subtree and rep is
\\ applied by powering.  Repeats of "without_max *= c_with" are simply
\\
\\     without_max *= c_with^rep
\\
\\ Similarly "without *= c_max".  Working through repeated "new_with" shows
\\ it becomes
\\
\\     new_with = v_with    * c_max^rep
\\              + v_without * rep * c_max^(rep-1) * c_without
\\
\\ Powering is efficient if large rep occurs, though in practice would
\\ expect rep=1 most of the time, depending on inputs of course.


\\ Return a 3-element vector [with, without_max, without] of quantities for
\\ the tree of Matula-Goebel number n.
A347966_maximal_matchings_quantities(n) =
{
  my(with        = 0,     \\ initial values for a singleton vertex
     without_max = 1,
     without     = 1,
     f=factor(n));
  for(i=1,matsize(f)[1],  \\ each child subtree
     my(c     = A347966_maximal_matchings_quantities(primepi(f[i,1])),
        rep   = f[i,2],       \\ how many repetitions of child subtree
        c_max = c[1] + c[2],  \\ c maximals = c_with + c_without_max
        c_max_repsub1 = c_max^(rep-1),
        c_max_rep     = c_max_repsub1 * c_max);  \\ c_max^rep
     with = with * c_max_rep
          + without * rep * c_max_repsub1 * c[3];
     without_max *= c[1]^rep;
     without     *= c_max_rep);

  [with, without_max, without];
}

A347966(n) = vecsum(A347966_maximal_matchings_quantities(n)[^3]);

{
  print("A347966 number of maximal matchings in tree number n:");
  print1("  ");
  for(n=1,35,
     print1(A347966(n),","));
  print("...");
}

print("end");
