\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\-----------------------------------------------------------------------------
\\ Kevin Ryde, October 2022
\\
\\ This is some PARI/GP code to calculate terms of A600002, the total path
\\ length (sum of vertex depths) in the rooted binary tree with
\\ Colijn-Plazzotta tree number n.
\\ 
\\ Requires PARI/GP 2.13 for sqrtint() remainder.


default(recover,0);
default(strictargs,1);

{
  \\ children(n), for n>=2, returns a vector [x,y] of the child subtrees of n,
  \\ x = A002024(n-1) and y = A002260(n-1)
  my(children(n) =
    my(r, s=sqrtint((n-1)<<1,&r));
    if(r>s,  r-=s;s++, r+=s);
    [s,r>>1]);

  \\ return sum of depths of vertices, taking the root at "depth" (an integer)
  my(traverse(n,depth) =
     depth + if(n>1,
                depth++;
                vecsum([self()(c,depth) |c<-children(n)])));

a(n) = traverse(n,0);
}

{
  print1("A600002 = ");
  for(n=1,12, print1(a(n),", "));
  print(", ...");
}

\\ traverse() is a recursive traversal of the tree.  The depth parameter
\\ tells it the depth down so far when in a subtree.
\\
\\ Peak recursion depth is peak vertex depth (tree height) and that's modest
\\ since the smallest n of height h is n = A108225(h) which is very large
\\ very quickly.


\\-----------------------------------------------------------------------------
\\ Vector of Terms
\\
\\ A vector of initial terms of the sequence can be calculated by adding
\\ previous terms and number of vertices.  Each vertex is 1 level deeper so
\\ must add 1 for each vertex, as per the sequence formula.

\\ return a vector of length len
a_vector(len) =
{
  my(ret=vector(len),
     num_vertices=Vecsmall([1],len));
  if(len,
     my(n=1);       \\ last stored in ret[]
     for(x=1,oo,
        for(y=1,x,
           if(n++ > len, break(2));
           my(s = num_vertices[x] + num_vertices[y]);
           num_vertices[n] = s+1;
           ret[n] = ret[x] + ret[y] + s)));
  ret;
}

{
  my(v=a_vector(100));
  for(n=1,100,
     v[n] == a(n) || error());
}

\\ The num_vertices[] vector could be returned too if desired (and usually a
\\ full Vec ratehr than Vecsmall is more convenient in that case).
\\
\\ For just the total path lengths vector is wanted, then num_vertices[] is
\\ only needed up to the largest x, which is children(len)[1].


\\-----------------------------------------------------------------------------

print("end");
