\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\-----------------------------------------------------------------------------

\\ Kevin Ryde, July 2022
\\
\\ Usage: gp <a356121.gp
\\
\\ This is some PARI/GP code to calculate terms of A356121, the
\\ Matula-Goebel number of the binomial tree of n vertices.

default(recover,0);
default(strictargs,1);


a(n) =
{
  my(ret=1);
  if(n--,     \\ working on n-1
     my(i=0,
        T=1,   \\ a(2^i)
        max_i = if(n,logint(n,2)));
     while(if(bittest(n,i),
              ret = T*prime(ret));
           i++ <= max_i
           ,
           T *= prime(T)));
  ret;
}

{
  print1("A356121 = ");
  for(n=1,12, print1(a(n),", "));
  print("...");
}

\\ This code uses the formula,
\\
\\     a(n) = a(2^k) * prime(a(r))
\\     where n = 2^k + r and 1 <= r <= 2^k
\\
\\ except the working n is decremented to an m and the aim is a(m+1)
\\
\\     m = n-1
\\     a(m+1) = a(2^k) * prime(a(s+1))
\\     where m = 2^k + s and 0 <= s < 2^k
\\
\\ This decrement means the 2^k is bit k of m, and s is the bits below, and
\\ hence successive bittest().
\\
\\ At the top of the loop, the least significant i many bits have been
\\ considered,
\\
\\     ret = a(1 + (m mod 2^i))
\\     T   = a(2^i)
\\
\\ T "doubles" each time like A076146.
\\
\\ The effect of the recurrence in the tree can be illustrated
\\
\\     0                     n = 14 vertices
\\     | \  \     \          m = n-1 = binary 1101
\\     1  2  4     8
\\        |  | \   | \  \
\\        3  5  6  9 10 12
\\              |     |  |
\\              7    11 13
\\
\\ i=0 is singleton vertices T={12} and ret={13} put under it to {12..13}.
\\ T doubles in size too.
\\
\\ i=1 is a 0-bit in m so no change to ret but T doubles in size again.
\\
\\ i=2 is T={8..11} and ret={12..13} put under it to set ret={8..13}.
\\ T doubles again too.
\\
\\ i=3 is the final step T={0..7} and ret{8..13} put under it to make the
\\ whole tree.  Looping ends without doubling T again.
\\
\\ The little oddity in the loop control where code preceds the loop
\\ condition is so that T is not doubled on the last loop, since such a
\\ double would be unused (and the biggest prime() so far).
\\
\\
\\ Speed
\\ -----
\\
\\ Speed is limited by prime(x) on ever larger x.  PARI speed is usually
\\ good with a suitably large "primelimit".  Circa PARI 2.13, a 32-bit build
\\ can only have primelimit <= about 4 billion.  That suffices (or the
\\ default too actually) up to for n=59, but beyond that is unfeasably slow.


\\-----------------------------------------------------------------------------
\\ Vector of Terms
\\
\\ Like most recurrenceas, a vector of terms can be made iwth the recurrence
\\ taking various terms previously calculated.
\\
\\ Here, each prime(a(n)) is calculated when needed and then kept for
\\ possible re-use.  The speed considerations noted above for large prime(x)
\\ apply the same here.

a_vector(len) =
{
  my(v=vector(len,i,1),
     pr=vector(len),
     k=0,r=1,
     n=1);  \\ v[n] last written
  while(n<len,
        if(!pr[r], pr[r] = prime(v[r]));
        v[n++] = v[2^k] * pr[r];
        if(r++ > 2^k, k++;r=1));
  v;
}

{
  \\ check a_vector() against a()
  my(v=a_vector(32));
  #v == 32                 || error();
  vector(#v,n, a(n)) == v  || error();
}

\\-----------------------------------------------------------------------------
print("end");
