\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\ FIXME: pow of non-integer ?


\\ This is some code for simple interval arithmetic using 2-element vectors.
\\
\\     t = interv_add(x,y)
\\     t = interv_add_constant(t,c)
\\     t = interv_sub(x,y)
\\     t = interv_sub_constant(t,c)
\\     t = interv_neg(t)
\\     t = interv_mul(x,y)
\\     t = interv_div(x,y)
\\     t = interv_recip(t)
\\     t = interv_pow_constant(t,c)
\\
\\     bool = interv_contains(t,sub)         contains "sub" interval
\\     bool = interv_contains_constant(t,c)
\\     bool = interv_contains_0(t)
\\
\\     t = interv_union(x,y)
\\     t = interv_union_0(t)
\\     t = interv_widen_integer(t)
\\     t = interv_widen_bits(t,k)
\\
\\     vec = interv_frac_digits(t,base=10,limit='none)
\\     t = interv_frac_fromdigits(v,base=10,point=1)
\\
\\     vec = interv_contfrac(t,limit='none)
\\     t = interv_from_contfrac(vec)
\\
\\     t = interv_solve_bisection(t,func,steps)
\\
\\
\\ Intervals
\\ ---------
\\
\\ An interval is represented by a 2-element vector
\\
\\     [lo, hi]         interval lo <= value <= hi
\\
\\ All functions accept an interval either way around [lo,hi] or [hi,lo],
\\ but outputs are always [lo,hi].
\\
\\ The purpose of interval arithmetic is to make calculations with an upper
\\ and lower bound around some true value which is difficult to calculate or
\\ known only to limited precision.  Many quantities such as sum of a series
\\ or limit of a sequence can be hopelessly impractical to represent in an
\\ exact way, so have only an upper and lower bound of some target accuracy.
\\
\\ lo and hi should generally be integers or rationals.  Floating point
\\ works, but interval endpoints are then subject to floating-point rounding
\\ which defeats the purpose of certainty of upper and lower bounds.
\\
\\ Width
\\ -----
\\
\\ Width = hi - lo of an interval is how far apart the  interval endpoints.
\\ Addition and subtraction both add widths.  For example
\\
\\     [100,110] + [5,7]  =  [105, 117]     width 10 + 2 = 12
\\     [100,110] - [5,7]  =  [ 93, 105]     width 10 + 2 = 12
\\
\\ Generally widths tend to increase during a calculation, due to each
\\ operation assuming the worst case combinations.
\\
\\ Widening
\\ --------
\\
\\ Sometimes during a calculation, denominators from rationals used will
\\ become large -- much larger than relevant for the calculation.  It may be
\\ worth widening intervals to some sort of common denominator.
\\ interv_widen_bits() does this to a specified bit precision (a denominator
\\ 2^k).  This is efficient, but some other common base might be preferred,
\\ such as when targeting decimal digits.
\\
\\ Other Ways to Do It
\\ -------------------
\\
\\ The sophisticated libraries for interval arithmetic or directed rounding
\\ tend to keep precision in bits, and possibly in a floating point style.
\\
\\ An implicit scale factor could be reckond on the intervals here, with
\\ care taken to apply it at the right times, if really wanted.  But in
\\ general interv.gp is meant to stay in GP and be easy for simple
\\ calcualtions and leave more to other code.
\\
\\ Changes
\\ -------
\\
\\ Version 1 - the first version.
\\ Version 2 - ...


\\-----------------------------------------------------------------------------

interv_contains_0(t) = vecprod(apply(sign,t)) <= 0;
{
addhelp(interv_contains_0,
"bool = interv_contains_0(t)
Return 1 if interval t contains 0, or return 0 if not.
Interval endpoints are inclusive, so test lo <= 0 <= hi.");
}

interv_contains_constant(t,c) = vecmin(t) <= c && c <= vecmax(t);
{
addhelp(interv_contains_constant,
"bool = interv_contains_constant(t,c)
Return 1 if interval t contains constant c, or return 0 if not.
Interval endpoints are inclusive, so test lo <= c <= hi.");
}

interv_contains(t,sub) = \
  vecmin(t) <= vecmin(sub) && vecmax(sub) <= vecmax(t);
{
addhelp(interv_contains,
"bool = interv_contains(t,sub)
Return 1 if interval t contains all of interval \"sub\" as a sub-interval,
or return 0 if not.
Interval endpoints are inclusive, so t and sub endpoints can be equal, as
long as none of sub is strictly outside t.");
}

interv_union(x,y) = x=concat(x,y); [vecmin(x), vecmax(x)];
{
addhelp(interv_union,
"t = interv_union(x,y)
x and y are intervals.  Return an interval of their union, meaning the
smallest and largest endpoints of both.");
}

interv_union_0(t) = t=concat(0,t); [vecmin(t), vecmax(t)];
{
addhelp(interv_union_0,
"t = interv_union_0(t)
t is an interval.  Return it unioned with 0, so that the bigger or smaller
endpoint is raised or lowered to 0 if 0 is not already contained.");
}

interv_neg(t) = [-vecmax(t), -vecmin(t)];
{
addhelp(interv_neg,
"t = interv_neg(t)
t is an interval.  Return negation -t.

A plain negate \"-t\" suffices, but it leaves the endpoints [hi,lo] whereas
this function always gives [lo,hi].");
}

interv_add(x,y) = [vecmin(x)+vecmin(y), vecmax(x)+vecmax(y)];
{
addhelp(interv_add,
"t = interv_add(x,y)
x and y are intervals.  Return sum x + y.

If x and y are both in [lo,hi] order then plain vector addition \"x+y\"
suffices too.  But that's not so if the orders are opposite [lo,hi] and
[hi,lo].  interv_add() handles all combinations.");
}

interv_add_constant(t,c) = vecsort([t[1]+c, t[2]+c]);
{
addhelp(interv_add_constant,
"t = interv_add_constant(t,c)
t is an interval and c is a constant.  Return interval sum t + c.");
}

interv_sub(x,y) = [vecmin(x)-vecmax(y), vecmax(x)-vecmin(y)];
{
addhelp(interv_sub,
"t = interv_sub(x,y)
x and y are intervals.  Return subtraction x - y.");
}

interv_sub_constant(t,c) = vecsort([t[1]-c, t[2]-c]);
{
addhelp(interv_sub_constant,
"t = interv_sub_constant(t,c)
t is an interval and c is a constant.  Return interval sum t - c.");
}

\\ plot(x=-2,2,1/x)
interv_recip(t) = \
  if(interv_contains_0(t), [-1/0,1/0], vecsort([1/t[1], 1/t[2]]));
{
addhelp(interv_recip,
"t = interv_recip(t)
t is an interval.  Return the interval reciprocal 1/t.

If t contains 0 (either an endpoint or within) then a division-by-0 error
results.  (An infinite interval [-oo,oo] would be possible, but unlikely to
be useful since that is effectively no information at all about the value.)");
}

interv_mul(x,y) =
{
  \\ 4 combinations of endpoints
  \\ ENHANCE-ME: Attention to signs so only 2 multiplies ?
  x = concat(apply(e->e*y, x));
  [vecmin(x),vecmax(x)];
}
{
addhelp(interv_mul,
"t = interv_mul(x,y)
x and y are intervals.  Return product x*y.");
}

interv_div(x,y) = interv_mul(x, interv_recip(y));
{
addhelp(interv_div,
"t = interv_div(x,y)
x and y are intervals.  Return division x/y.
If y contains 0 (either an endpoint or within) then a division-by-0 error
results, like interv_recip().");
}

interv_pow_constant(t,c) =
{
  if(c%2==0 && interv_contains_0(t),
     [0, vecmax(abs(t))^c],
     vecsort([t[1]^c, t[2]^c]));
}
{
addhelp(interv_pow_constant,
"t = interv_pow_constant(t,c)
t is an interval and c is an integer.  Return power t^c.");
}


\\-----------------------------------------------------------------------------
\\ Widening

interv_widen_integer(t) = [floor(vecmin(t)), ceil(vecmax(t))];
{
addhelp(interv_widen_integer,
"t = interv_widen_integer(t)
t is an interval.  Return it widened it to the next bigger and smaller
integers.  See also interv_widen_bits().");
}

interv_widen_bits(t,k) = my(p=2^k); interv_widen_integer(t*p) / p;
{
addhelp(interv_widen_bits,
"t = interv_widen_bits(t,k)
t is an interval.
Return it widened it to fraction of the form n/2^k.

This is useful for avoiding large denominators when various terms are added
or multiplied.  Usually k will be some bit position a little beyond the
target precision for a final result.

k=0 is the same as interv_widen_integer().
k can be negative for multiples of a power of 2.");
}


\\-----------------------------------------------------------------------------
\\ Digits

interv_frac_digits(t,base=10,limit='none) =
{
  my(l=List([]), f);
  while((limit=='none || #l<limit) && (f=floor(t))[1]==f[2],
        listput(l,f[1]);
        t=(t-f)*base);
  Vec(l);
}
{
addhelp(interv_frac_digits,
"vec = interv_frac_digits(t,base=10,limit='none)
t is an interval.  Return a vector of its integer part then fractional
digits, up to the accuracy of t.

vec[1] returned is the integer part of t (floor), then vec[2..] are decimal
digits.  Optional parameter \"base\" gives digits in another base.

Optional parameter \"limit\" is a maximum length for the return vec, so stop
at vec length \"limit\" or the accuracy of t, whichever comes first.
Default symbol 'none is no limit (so all of the accuracy of t).");
}

interv_frac_fromdigits(v,base=10,point=1) =
{
  my(p=1/base^(#v-point));
  v=fromdigits(v,base)*p;
  [v, v+p];
}
{
addhelp(interv_frac_fromdigits,
"t = interv_frac_fromdigits(v,base=10,point=1)
v is a vector of digits, with the radix point immediately after v[point],
which are known to be good, but with further digits after v unknown other
than being in the range 0..base-1.  Return an interval for these digits.

The default point=1 and base=10 suits the corresponding defaults from
interv_frac_digits().  vec[1] in that case is an integer part (so
possibly bigger than \"9\").  Any digits can be outside the normal 0..9, so
long as the unknown part means the interval is v to v + 1/base^(#v-point).");
}


\\-----------------------------------------------------------------------------
\\ Continued Fractions

interv_contfrac(t,limit='none) =
{
  my(l=List([]), f);
  while((limit=='none || #l<limit)
          && (f=floor(t))[1]==f[2],
        listput(l,f[1]);
        t=interv_recip(t-f));
  Vec(l);
}
{
addhelp(interv_contfrac,
"vec = interv_contfrac(t,limit='none)
Return a vector of terms of the continued fraction expansion of interval t,
up to the accuracy of t.

Optional parameter \"limit\" can be an integer length limit for the return
vector, so stop at vec length \"limit\" or accuracy of t, whichever comes
first.  Default symbol 'none is no limit (just accuracy of t).");
}

interv_from_contfrac(vec) =
{
  \\ 1 +      1
  \\      -----------
  \\      x +   1
  \\          -------
  \\          unknown
  \\
  \\ 1/unknown part ranges from 0 to 1,
  \\ so same as vec through to concat(vec,1),
  \\ which one contfracpnqn() matrix suffices

  my(m=contfracpnqn(vec));
  vecsort([ m[1,1]        /m[2,1],
           (m[1,1]+m[1,2])/(m[2,1]+m[2,2]) ]);
}
{
addhelp(interv_from_contfrac,
"t = interv_from_contfrac(vec)
vec is a vector of continued fraction terms (positive integers),
starting from the integer part.

Return an interval for vec followed by unknown additional terms.
vec must not be empty [], as that would be any value at all.

Additional terms add an amount 1/extra which ranges 0 to 1 and so is as if
concat(vec,[1]) were the other endpoint (and is bigger or smaller according
as vec length odd or even.");
}

\\ MAYBE: Could allow for vec terms smaller than 1 after vec by an optional
\\ parameter.  But integers are usual for continued fraction terms, so
\\ hardly need it.


\\-----------------------------------------------------------------------------

interv_solve_bisection(t,func,steps) =
{
  t=vecsort(t);
  my(ylo=func(t[1]));
  if(ylo==0, t[2]=t[1],
     my(yhi=func(t[2]));
     if(yhi==0, t[1]=t[2],
        if(sign(ylo)==sign(yhi),
           error("interv_solve_bisection() func does not change sign"));
        for(i=1,steps,
           my(tmid=vecsum(t)/2, ymid=func(tmid));
           if(ymid==0, t[1]=t[2]=tmid;break,
              sign(ymid)==sign(ylo), t[1]=tmid,
                                     t[2]=tmid))));
  t;
}
{
addhelp(interv_solve_bisection,
"t = interv_solve_bisection(t,func,steps)
t is in an interval over which func changes sign,
so sign(func(lo)) != sign(func(hi)).

Narrow the interval by taking the midpoint and putting it to either lo or hi
according as sign(func(mid)).  Do this \"steps\" many times and return the
result.

If some func(xmid)==0 is found, then return that as an exact solution found
[xmid,xmid].

This bisection suits a func which is continuous at its 0 crossing and gives
a rational value for a rational operand.  The width of interval t will
narrow to width/2^steps.

func can cross 0 multiple times within t and in that case it's unwise ro
retly on wihch 0 will be narrowed to.  Normaly usage is to start t as an
interval crossing a single solution.  For polynomial roots, see polsturm()
to find how many solutions in a given range.");
}

\\-----------------------------------------------------------------------------
