\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <minimal-asymmetric.gp
\\
\\ A minimal asymmetric graph is asymmetric and has no induced subgraph
\\ which is asymmetric.  Schweitzer and Schweitzer proved a conjecture of
\\ Nesetril and Sabidussi that a certain set of 18 graphs are the only such.
\\
\\    Pascal Schweitzer and Patrick Schweitzer, "Minimal Asymmetric Graphs",
\\    Journal of Combinatorial Theory, Series B, volume 127, November 2017,
\\    pages 215-227
\\    https://www.sciencedirect.com/science/article/pii/S0095895617300539
\\    https://arxiv.org/abs/1605.01320
\\
\\ Among these graphs there is one tree (X9 in their numbering), which is
\\ the asymmetric free tree of fewest vertices.

read("../vpar.gp");


\\   1---2
\\   |           tree X9
\\   3---4       per Schweitzer and Schweitzer
\\   |
\\   5           https://hog.grinvin.org/ViewGraphInfo.action?id=934
\\   |
\\   6---7
\\
X9=[0,1,1,3,3,5,6];
print("X9 = "X9);

\\ Return 1 if vpar is minimal asymmetric as a free tree, or return 0 if not.
\\ This is by brute force forming and checking all induced subgraphs.
\\
\\ vpar_keep_vertices() acts to get the subgraph induced by the given
\\ vertices, in that it keeps edges between kept vertices.  (Its converse
\\ vpar_delete_vertices() does the same.)
\\
is_minimal_asymmetric(vpar) =
{
  vpar_is_free_asymmetric(vpar) || return(0);
  forvec(flags=vector(#vpar,i,[0,1]),
        my(set=select(s->s, flags, 1));
        if(#set<#vpar,
           my(sub=vpar_keep_vertices(vpar,set));  \\ subgraph
           if(#sub>=2 && vpar_is_free_asymmetric(sub),
              return(0))));
  1;
}

print("is_minimal_asymmetric() = "is_minimal_asymmetric(X9));
is_minimal_asymmetric(X9) || error();
print();

\\ try a search to see that X9 is indeed the only is_minimal_asymmetric(),
\\ among small trees
{
  for(n=2,12,
     print("search n="n);
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           if(is_minimal_asymmetric(vpar),
              print(vpar);
              if(vpar_is_free_isomorphic(vpar,X9),
                 print("  is X9"));
              vpar_is_free_isomorphic(vpar,X9) || error();
             )));
}
