\\ Copyright 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp -f <test-slow.gp
\\
\\ This is some tests which are either a bit slow or have special
\\ dependencies that might not run in the main test.gp.
\\
\\ Requires:
\\     - install() of Pari library functions
\\     - Map() new in gp 2.9.x
\\     - program vpar-to-graph6.c compiled
\\     - system() of more or less Unix shell syntax
\\     - various OEIS data files

allocatemem(50*10^6);     \\ 50 mbytes stack
default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

install(group_domain,    "lG", "group_domain");
install(group_export,    "GL", "group_export");
install(group_elts,      "GL", "group_elts");
install(group_order,     "lG", "group_order");
install(group_ident,     "lGG", "group_ident");
install(group_ident_trans, "lGG", "group_ident_trans");
install(group_isabelian, "lG", "group_isabelian");
install(group_isA4S4,    "lG", "group_isA4S4");
install(group_subgroups, "G", "group_subgroups");
install(group_subgroup_isnormal,    "lGG", "group_subgroup_isnormal");
install(perm_commute,    "iGG", "perm_commute");
install(galois_group,    "G",  "galois_group");
install(abelian_group,   "G",  "abelian_group");
install(group_abelianHNF,"GG", "group_abelianHNF");
install(group_abelianSNF,"GG", "group_abelianSNF");
install(group_quotient,  "GG", "group_quotient");
install(quotient_group,  "GG", "quotient_group");
install(groupelts_abelian_group, "G", "groupelts_abelian_group");
install(groupelts_exponent, "lG", "groupelts_exponent");
install(groupelts_center, "G", "groupelts_centre");

nearly_equal_epsilon = 1e-15;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) =
{
  \\ print(type(x)," ",type(y)," ",x," ",y);
  my(d=x-y); \\ ensure compatible types
  if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
  if(type(x)=="t_VEC" || type(x)=="t_COL",
       sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
     type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
          nearly_equal(real(x-y),0,epsilon)
       && nearly_equal(imag(x-y),0,epsilon),
     abs(d) < epsilon);
}

\\-----------------------------------------------------------------------------
\\ A289399  Complete Ternary Total Path Length
\\ a(n) = 3*(1 - 3^n + 2*3^n*n) / 4
\\ of height n

{
  my(a(n) = ((2*n-1)*3^n + 1)*3/4);
  OEIS_check_func("A289399",a);

  \\ num vertices
  check_equal(vector(5,n,#vpar_make_complete_tree(3,n-1)),
              vector(5,n, (3^n-1)/2));

  \\ recurrence, extra crossings of top edges
  check_equal(vector(10,n,n++; a(n)),
              vector(10,n,n++; 3*(a(n-1) + (3^n-1)/2)));
  check_equal(vector(5,n, a(n)),
              vector(5,n, vecsum(vpar_depths_vector(vpar_make_complete_tree(3,n)))));
}

\\-----------------------------------------------------------------------------
progress("n-Alkane");

\\ alkanes trees with degrees <= 4
\\
\\ n-alkane straight line filled so
\\
\\ methane n=1   ethane n=2
\\    *             *  *
\\    |             |  |
\\ *--*--*       *--*--*--*
\\    |             |  |
\\    *             *  *

\\ OEIS sequences start from n=1, so don't consider what an n=0 alkane
\\ might be
make_n_alkane(n) = \
  vpar_make_caterpillar(vector(n+2,i, if(i==1||i==n+2,0,2)));
{
  check_equal(make_n_alkane(1), [0, 1, 2, 2, 2]);
  check_equal(make_n_alkane(0), vpar_make_path(2));
  check_equal(vpar_is_free_isomorphic(make_n_alkane(1), vpar_make_star(5)), 1);

  check_equal(vpar_domsets_count(vpar_make_star(5)), 17);
  check_equal(vector(20,n,n--; vpar_domsets_count(make_n_alkane(n))),
              vector(20,n,n--; if(n==0,3, n==1,17, 81*5^(n-2))),
              "n-alkane domsets count formula");
  my(want=OEIS_data("A347501"));
  check_equal(vector(#want,n, vpar_domsets_count(make_n_alkane(n))),
              want,
              "A347501 n-alkane domsets count");
}
{
  check_equal(vector(20,n, vpar_totdomsets_count(make_n_alkane(n))),
              vector(20,n, if(n==1,15, 4^(n+1))),
              "n-alkane totdomsets count");
  my(want=OEIS_data("A347478"));
  check_equal(vector(#want,n, vpar_totdomsets_count(make_n_alkane(n))),
              want,
              "A347478 n-alkane totdomsets count");
}
{
  my(want=OEIS_data("A180148"));
  check_equal(vector(#want,n,n--; vpar_matchings_count(make_n_alkane(n))),
              want,
              "n-alkane matchings count");
}
{
  my(want=OEIS_data("A228602"));
  check_equal(vector(#want,n, vpar_indsets_count(make_n_alkane(n))),
              want,
              "n-alkane indsets count");
}

\\ Pell numbers A000129, per GP code by Michael Somos in A000129
my(c=1+quadgen(8)); /* 1+sqrt2 */ \
test_Pell(n) = imag(c^n);
{
  OEIS_check_func("A000129","test_Pell");

  check_equal(vector(100,n,n--; test_Pell(n)),
              vector(100,n,n--; 
      subst(lift(Mod('x,'x^2-2*x-1)^(n-1)),'x,2) ),
              "test_Pell() by polmod");

  my(A052542(n) = if(n==0,1, 2*test_Pell(n)));
  my(want=OEIS_data("A052542"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; A052542(n)),
              want,
              "2*test_Pell() vs A052542 samples");

  my(A163271(n) = 2*test_Pell(n-1));
  my(want=OEIS_data("A163271"));  \\ OFFSET=1
  check_equal(vector(#want,n, A163271(n)), want,
              "A163271() vs samples");

  check_equal(vector(100,n,n++; A163271(n)),  \\ for n>=2
              vector(100,n,n++; A052542(n-1)),
              "A163271() formula using A052542");
}
{

  \\ maximum matchings
  \\ A052542 or A163271
  check_equal(vector(50,n, vpar_maximal_matchings_count(make_n_alkane(n))),
              vector(50,n, 2*test_Pell(n+1)),
              "n-alkane maximal matchings = 2*Pell");
}
{
  \\ maximum matchings
  \\ A005010 9*2^n
  check_equal(vector(50,n, vpar_matchnum_and_count(make_n_alkane(n))[2]),
              vector(50,n, if(n==1,4, 9*2^(n-2))),
              "n-alkane maximum matchings");

  check_equal(vector(50,n, vpar_is_equimatchable(make_n_alkane(n))),
              vector(50,n, n==1),
              "n-alkane not equimatchable, other than at n=1");
}


\\------------------------------------------------------------------------------
\\ EGF of A020531 powers

{
  my(a(n) = n>=0 || error();
     1 + 3*4^n + 16^n);
  OEIS_check_func("A020531",a,['offset,0]);

  check_equal(test_gf_terms(-(116*x^2 - 76*x + 5)
            / ((x - 1)*(4*x - 1)*(16*x - 1)), 100),
              vector(100,n,n--; a(n)),
              "gf");

  check_equal(vector(100,n,n+=2; a(n)),
              vector(100,n,n+=2; 21*a(n-1) - 84*a(n-2) + 64*a(n-3)));

  my(want=vector(200,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
     \\ egf by Stefano Spezia in A020531
  check_equal(exp(x)*(1 + 3*exp(3*x) + exp(15*x)),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A116471  2*(n +/- 1)^2 sorted
\\   0, 2, 8, 8, 18, 18, 32, 32, 50, 50,
\\   values twice each
\\ recurrence_guess(concat(0,OEIS_data("A116471")))
\\  1/2 * n^2
\\  + vector_modulo([0, 1],n) * n
\\  + vector_modulo([0, 1/2],n)


{
  my(a(n) = n>=1 || error();
     if(n==1,0, sqr((n+1)>>1)<<1);
    );
  OEIS_check_func("A116471",a,['offset,1]);

  check_equal(test_gf_terms(-2*x^2*(x^4 - x^3 - 2*x^2 + 3*x
               + 1)/((x - 1)^3*(x + 1)^2), 100),
              vector(100,n,n--; if(n==0,0,a(n))),
              "gf");

  check_equal(vector(100,n,n+=6; a(n)),
              vector(100,n,n+=6;
    a(n-1) + 2*a(n-2) - 2*a(n-3) - a(n-4) + a(n-5)));

  \\ formula by Bruno Berselli
  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++;
    (2*n^2 + 2*n - (2*n + 1)*(-1)^n + 1)/4));

  my(want=vector(200,n,n--; if(n==0,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
     \\ egf by Stefano Spezia in A020531
  check_equal((  x*(3 + x)*cosh(x)
               + (1 + x + x^2)*sinh(x)
               - 4*x)/2,
              want_egf);
}

\\------------------------------------------------------------------------------
\\ EGF of A002492 poly


{
  my(a(n) = n>=0 || error();
     binomial(2*n+2, 3));
  OEIS_check_func("A002492",a,['offset,0]);

  my(sqrt5=quadgen(20),
     want=vector(200,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ egf by Stefano Spezia in A005744
  check_equal(2*exp(x)*x*(6 + 9*x + 2*x^2)/3,
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A008486 of 3*n except at n=0

{
  my(a(n) = n>=0 || error();
     if(n==0,1, 3*n));
  OEIS_check_func("A008486",a,['offset,0]);

  my(want=vector(200,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ egf by Stefano Spezia in A005744
  check_equal(1 + 3*exp(x)*x,
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A008579 coordinateion 3.6.3.6
\\  initial exceptions [1, 4]
\\  vector_modulo([5, 4],n) * n
\\  + vector_modulo([-2, 2],n)

{
  my(g=(1 + 2*x)*(1 + 2*x + 2*x^2 + 2*x^3 - x^4)
       / (1 - x^2)^2,
     a(n) = n>=0 || error();
     test_gf_term(g,n));
  OEIS_check_func("A008579",a,['offset,0]);
  \\ recurrence_guess(vector(100,n,n--; a(n)));quit;

  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++;
                     (9 + (-1)^n)*n/2 - 2*(-1)^n),
              "formula for n>=2"); 
  
  my(want=vector(10,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ egf by Stefano Spezia in A005744
  my(got_egf = 3 - 2*x + (4*x - 2)*cosh(x) +
               (5*x + 2)*sinh(x));
  check_equal(got_egf, want_egf,
              "coordination egf");
}


\\------------------------------------------------------------------------------
\\ EGF of A005744 poly


{
  my(a(n) = n>=0 || error();
     test_gf_term(x*(1+x-x^2)/((1-x)^4*(1+x)), n));
  OEIS_check_func("A005744",a,['offset,0]);

  my(sqrt5=quadgen(20),
     want=vector(200,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want),
     \\ egf by Stefano Spezia in A014217
     got_egf = (x*(2*x^2 + 21*x + 27)*cosh(x)
                + (2*x^3 + 21*x^2 + 27*x - 3)*sinh(x)) / 24);
  check_equal(got_egf == want_egf, 1);
}


\\------------------------------------------------------------------------------
\\ EGF of A151979  0,1 mod 19

{
  my(a(n) = n>=1 || error();
     19*(n-1)>>1 - bitnegimply(1,n)<<3);
  OEIS_check_func("A151979",a,['offset,1]);
  if(0,  \\ ~/OEIS/b151979.txt
     OEIS_check_func("A151979",a,'bfile));

  OEIS_check_predfunc("A151979",(m)->(m^2-m)%19==0);
  check_equal(vector(100,n,n--; if(n==0,0, a(n))),
              test_gf_terms(x^2*(1+18*x)/((1-x)^2*(1+x)), 100),
              "gf");

  check_equal(vector(100,n, a(n)),
              vector(100,n,
     (1/4)*(38*n - 55 - 17*(-1)^n)));

  my(want=vector(200,n,n--; if(n,a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want),
     got_egf = (19/2)*(x*(cosh(x) + sinh(x)) - sinh(x))
                - 18*(cosh(x) - 1));
  check_equal(got_egf == want_egf, 1);
}


\\------------------------------------------------------------------------------
\\ EGF of A014217  floor phi^n

{
  my(phi=quadgen(5),
     a(n) = n>=0 || error();
     floor(phi^n));
  OEIS_check_func("A014217",a,['offset,0]);
  if(0, OEIS_check_func("A014217",a,'bfile));

  my(sqrt5=quadgen(20),
     want=vector(200,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want),
     \\ egf by Stefano Spezia in A014217
     got_egf = (2*exp(x/2)*cosh(sqrt5*x/2) - cosh(x)));
  check_equal(got_egf == want_egf, 1);
}


\\------------------------------------------------------------------------------
\\ EGF of A022383  Fibonacci starting 4,14

{
  my(a(n) = n>=0 || error();
     4*fibonacci(n-1) + 14*fibonacci(n));
  OEIS_check_func("A022383",a,['offset,0]);

  my(sqrt5=quadgen(20),
     want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ egf by Stefano Spezia in A022383
  check_equal(serlaplace(4*exp(x/2)*(5*cosh(sqrt5*x/
               2) + 6*sqrt5*sinh(sqrt5*x/2))/5) ==
              sum(n=0,#want, a(n)*x^n),
              1,
              "EGF gy serlaplace()");
  check_equal(4*exp(x/2)*(5*cosh(sqrt5*x/
               2) + 6*sqrt5*sinh(sqrt5*x/2))/5 ==
              want_egf,
              1);

  my(A022113(n) = n>=0 || error();
     2*fibonacci(n-1) + 7*fibonacci(n));
  OEIS_check_func("A022113",A022113,['offset,0]);
  check_equal(vector(100,n,n--; a(n)),
              vector(100,n,n--; 2*A022113(n)));
}


\\------------------------------------------------------------------------------
\\ EGF of A022120  Fibonacci starting 3,7

{
  my(a(n) = n>=0 || error();
     3*fibonacci(n-1) + 7*fibonacci(n));
  OEIS_check_func("A022120",a,['offset,0]);

  my(sqrt5=quadgen(20),
     want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ egf by Stefano Spezia in A022120
  check_equal(exp(x/2)
              * (15*cosh(sqrt5*x/2)
                 + 11*sqrt5*sinh(sqrt5*x/2)) /5 ==
              want_egf,
              1);
}


\\------------------------------------------------------------------------------
\\ EGF of A006478 Fibonacci thing

{
  my(a(n) = n>=0 || error();
     1 + ((n-5)*fibonacci(n-1) 
          + (3*n-8)*fibonacci(n))/5);
  OEIS_check_func("A006478",a,['offset,0]);

  my(sqrt5=quadgen(20),
     want=vector(100,n,n--; if(n<1,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(exp(x)
         + exp(x/2)
         * (5*(3*x - 5)*cosh(sqrt5*x/2)
            + sqrt5*(5*x - 11)*sinh(sqrt5*x/2))/25 ==
              want_egf, 1);
}


\\------------------------------------------------------------------------------
\\ EGF of A042963 1,2 mod 4

{
  my(a(n) = n>=1 || error(); n--; 1+2*n-n%2);
  OEIS_check_func("A042963",a,['offset,1]);
  if(0, OEIS_check_func("A042963",a,'bfile));

  my(want=vector(100,n,n--; if(n<1,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(2 + (2*x - 1)*sinh(x) + 2*(x- 1)*cosh(x),
              want_egf);
  check_equal(2 + (2*x - 1)*exp(x) - cosh(x),
              want_egf);
  check_equal(2 + (2*x - 1)*exp(x) - cosh(x) + O('x^4),
              'x + 'x^2 + 5/6*'x^3 + O('x^4));
}

\\------------------------------------------------------------------------------
\\ EGF of A091999 2,10 mod 12

{
  my(a(n) = n>=1 || error(); 6*n - 3 + (-1)^n);
  OEIS_check_func("A091999",a,['offset,1]);
  if(0, OEIS_check_func("A091999",a,'bfile));

  my(want=vector(100,n,n--; if(n<1,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(2 + (6*x - 2)*cosh(x) + 2*(3*x - 2)*sinh(x),
              want_egf);

  check_equal(2 + (6*x - 4)*exp(x) + 2*cosh(x),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A024868 convol product

{
  my(a(n) = n>=2 || error();
     (4*n^3 + 33*n^2 + 38*n - 27
      + 3*(-1)^n*(n+3)^2)/48);
  OEIS_check_func("A024868",a,['offset,2]);

  my(want=vector(100,n,n--; if(n<2,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ EGF by G. C. Greubel in A024868
  check_equal((1/24)*( x*(2*x^2 + 24*x + 27)*cosh(x)
           + (2*x^3 + 21*x^2 + 48*x - 27)*sinh(x) ),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A047238 = 0,2 mod 6

{
  
  my(a(n) = n>=1 || error(); bitand(3*(n-1),-2));
  OEIS_check_func("A047238",a,['offset,1]);

  my(want=vector(100,n,n--; if(n==0,0, a(n))),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ EGF by David Lovler in A047238
  check_equal(3*(x-1)*exp(x) - cosh(x) + 4,
              want_egf);
}

\\------------------------------------------------------------------------------
\\ EGF of A018902 = 3^n - (n + 1)*(n + 2)/2

{
  my(sqrt13=quadgen(4*13),
     a(n) = n>=0 || error();
     \\ per formula by Colin Barker in A018902
     simplify(
     (2^(-1-n)*((5-sqrt13)^n*(-3+sqrt13)
                + (3+sqrt13)*(5+sqrt13)^n))
     / sqrt13 ));
  OEIS_check_func("A018902",a,['offset,0]);

  my(want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  \\ EGF by Stefano Spezia in A018902
  check_equal(exp(5*x/2)
              *(13*cosh(sqrt13*x/2)
                + 3*sqrt13*sinh(sqrt13*x/2) )/13 ==
              want_egf,
              1);
}


\\------------------------------------------------------------------------------
\\ EGF of A061982 = 3^n - (n + 1)*(n + 2)/2

{
  my(a(n) = n>=0 || error();
     3^n - (n + 1)*(n + 2)/2);
  OEIS_check_func("A061982",a,['offset,0]);
  \\ OEIS_check_func("A061982",a,'bfile);
  \\ recurrence_guess(vector(100,n,n--; a(n)));

  check_equal(vector(100,n,n--; a(n)),
              vector(100,n,n--;
                     3^n - binomial(n+2, 2)));

  check_equal(test_gf_terms( x^2*(3-x)/((1-x)^3 * (1-3*x)), 100),
              vector(100,n,n--; a(n)),
              "gf");

  my(want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(exp(3*x) - (1/2)*(2 + 4*x + x^2)*exp(x),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A190976 Lucas Sequence 8,-3

{
  my(sqrt13=quadgen(4*13),
     a(n) =  n>=0 || error();
     simplify(((4 + sqrt13)^n
             - (4 - sqrt13)^n)/(2*sqrt13)));
  OEIS_check_func("A190976",a,['offset,0]);
  \\ OEIS_check_func("A190976",a,'bfile);

  check_equal(test_gf_terms( x/(1 - 8*x + 3*x^2), 100),
              vector(100,n,n--; a(n)),
              "gf");

  my(sqrt3=quadgen(4*3));
  check_equal(sqrt3^2 == 3, 1);
  check_equal(vector(100,n,n--;
    simplify( sqrt3^(n-1)*polchebyshev(n-1, 2, 4/sqrt3)) ),
              vector(100,n,n--; a(n)));

  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++; 8*a(n-1) - 3*a(n-2)),
              "recurrence");

  my(want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(simplify( (1/sqrt13)*exp(4*x)*sinh (sqrt13*x) ),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A190974 Lucas Sequence 7,-5

{
  my(sqrt5=quadgen(20),
     a(n) =  n>=0 || error();
     simplify(sqrt5^(n-1)*polchebyshev(n-1,2, 7/(2*sqrt5))));
  OEIS_check_func("A190974",a,['offset,0]);
  \\ OEIS_check_func("A190974",a,'bfile);

  check_equal(vector(100,n,n--; type(a(n))),
              vector(100,n,n--; "t_INT"));

  my(sqrt29=quadgen(4*29));
  check_equal(vector(100,n,n--;
    simplify(  ((7/2 + 1/2*sqrt29)^n
             - (7/2 - 1/2*sqrt29)^n)/sqrt29) ),
              vector(100,n,n--; a(n)));

  check_equal(test_gf_terms( x/(1 - 7*x + 5*x^2), 100),
              vector(100,n,n--; a(n)),
              "gf");

  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++; 7*a(n-1) - 5*a(n-2)),
              "recurrence");

  my(want=vector(100,n,n--; a(n)),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(simplify( (2/sqrt29)*exp(7*x/2)*sinh(sqrt29*x/2) ),
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A052698 (3^n - 1)*n!/2

{
  my(a(n) =
     n>=0 || error();
     (3^n - 1)*n!/2);
  OEIS_check_func("A052698",a,['offset,0]);
  \\ OEIS_check_func("A052698",a,'bfile);

  check_equal(vector(20,n,
     (3*n^2+9*n+6)*a(n) + (-4*n-8)*a(n+1) + a(n+2)),
              vector(20,n, 0),
              "D-finite");

  check_equal(vector(20,n,n++; a(n)),
              vector(20,n,n++;
                     4*n*a(n-1) - 3*n*(n-1)*a(n-2)),
              "new D-finite");

  my(want=OEIS_data("A052698"),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(x/((1-x)*(1-3*x)),
              want_egf);

  \\ OEIS_data("A052698")*2
}


\\------------------------------------------------------------------------------
\\ EGF of A086443 4^n - 3^n - n*3^(n-1)

{
  my(want=OEIS_data("A086443"),
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(exp(3*x)*(exp(x)-x-1),
              want_egf);
}
{
  my(want=OEIS_data("A086443"));
  check_equal(test_gf_terms(x^2/((1-4*x)*(1-3*x)^2), #want),
              want);
  check_equal(((1-4*x)*(1-3*x)^2),
              1 - 10*x + 33*x^2 - 36*x^3);

  \\ no 0s = 3^n
  \\ one 0 = n*3^(n-1)
  my(is(v) = sum(i=1,#v,v[i]==0) >= 2);
  my(a_by_count(n) = my(ret=0);
     forvec(v=vector(n,i,[0,3]), ret+=is(v)); ret);
  check_equal(vector(7,n,n--; a_by_count(n)),
              want[1..7]);
  check_equal(a_by_count(4), 67);

  \\ Return a vector of all permutations of vector v
  my(perms_of_vector(v) =
     Set(vector((#v)!,n,
                vecextract(v,numtoperm(#v,n)))));
  \\ perms_of_vector([1,1,2])

  check_equal(#perms_of_vector([1,0,0,0]),  4);
  check_equal(#perms_of_vector([1,1,0,0]),  6);
  check_equal(#perms_of_vector([1,2,0,0]), 12);

}


\\------------------------------------------------------------------------------
progress("A354017 Ttoal Num Subtrees All Free Tree N Vertices");
\\
\\ apply(t->t+1,want)

num_subtrees(vpar) =
{
  my(ret=0);
  forvec(flags=vector(#vpar,i,[0,1]),
     my(sub=vpar_keep_flags(vpar,flags));
     ret += #sub && vpar_is_tree(sub));
  ret;
}
{
  my(a(n) =
     my(vpar=vpar_free_first(n),
        ret=0);
     until((vpar=vpar_free_next(vpar))=='none,
           ret += vpar_subfree_count(vpar) - 1;
           \\ ret += num_subtrees(vpar) + 1;
          );
     ret);

  my(want=[1,1,3,6,21,52,165,466,1489,4564,15339],
     len=min(12,#want));
  check_equal(vector(len,n,n--; if(n==0,1,a(n))),
              want[1..len]);

  \\ with empty subtree counted too
  \\ not in OEIS: 1, 2, 4, 7, 23, 55, 171
}

\\      v       with *= c_with
\\     /        without += c_without
\\    c


\\------------------------------------------------------------------------------
progress("Ttoal Num Subtrees All Free Tree N Vertices");
\\ Up to subtree free isomorphism.

all_subtrees_to_free_isomorphism(vpar) =
{
  my(ret=[],
     pending=[vpar]);
  while(#pending,
        my(pruned=Map());
        for(i=1,#pending,
           my(vpar=pending[i],
              leaves=vpar_leaves(vpar));
           for(j=1,#leaves,
              my(sub=vpar_delete_vertex(vpar,leaves[j]));
              mapput(pruned,vpar_to_free(sub),1)));
        #pruned || break;
        pruned=Mat(pruned)[,1]~;
        ret=concat(ret,pruned);
        pending=pruned);
  ret;
}
{
  my(a(n) =
     my(vpar=vpar_free_first(n),
        ret=0);
     until((vpar=vpar_free_next(vpar))=='none,
           ret += #all_subtrees_to_free_isomorphism(vpar));
     ret);

  \\ print(vector(8,n,a(n)));
  \\ \\ not in OEIS: 1, 2, 3, 8, 16, 43, 106, 300
  \\ print(vector(8,n,a(n) - 1));
  \\ \\ not in OEIS: 0, 1, 2, 7, 15, 42, 105, 299
  \\ quit;
}

\\------------------------------------------------------------------------------
\\ EGF of A069126 Centred 13-gonal numbers.

{
  my(want=concat(0,OEIS_data("A069126")), \\ OFFSET=1
     want_egf=test_vector_to_egf(want,0),
     x = x + O(x^#want));
  check_equal(exp(x)*(1 + 13*x^2/2) - 1,
              want_egf);
}


\\------------------------------------------------------------------------------
\\ EGF of A050441

{
  my(want=OEIS_data("A050441"),
     want_egf=test_vector_to_egf(want,0));
  check_equal(exp(x + O(x^(#want-1)))
              * x*(6 + 36*x + 11*x^2)/6,
              want_egf);
}

\\------------------------------------------------------------------------------
\\ EGF of A062075

{
  my(want=OEIS_data("A062075"),
     want_egf=test_vector_to_egf(want,0));
  check_equal(4*x*(1 + 28*x + 96*x^2 + 64*x^3)
              * exp(4*x + O(x^(#want-1))),
              want_egf);
}

\\-----------------------------------------------------------------------------
\\ vpar_is_caterpillar()
\\ on free forests

\\ A005418 1, 2, 3, 6, 10, 20, 36, 72, 136, 272, 528
\\ num caterpillars 1,1,2,3,6,10,20,36,72,136,272,
test_A005418_formula(n) = 2^(n-2) + 2^(floor(n/2) - 1);
test_A005418(n) = n>=1 || error(); test_A005418_formula(n);
test_count_free_caterpillars(n) = if(n<=2,1, test_A005418(n-2));

{
  \\ count free forests of caterpillars
  \\          n = 1  2  3  4  5
  \\ not in OEIS: 1, 2, 3, 6, 10, 20, 36, 72, 137, 275, 540, 1094, 2190
  \\
  \\ recurrence_guess(vpar_INTERNAL_Euler_transform(vector(100-1,n, test_count_free_caterpillars(n)))); quit;

  \\ starting n=0
  my(want=[1,1,2,3,6,10,20,36,72,137,275,540,1094,2190]); \\ n>=1
  check_equal(vpar_INTERNAL_Euler_transform(vector(#want-1,n, test_count_free_caterpillars(n))),
              want[2..#want],
             "num caterpillar forests by Euler transform");

  check_equal(vector(#want,n,n--;
                     my(vpar=vpar_free_first(n),
                        count=0);
                     until((vpar=vpar_free_next_forest(vpar,1))=='none,
                           count += vpar_is_caterpillar(vpar));
                     count),
              want,
              "vpar_is_caterpillar() count forests");
}


\\-----------------------------------------------------------------------------
\\   num rooted trees with 2 leaves
\\  [0, 0, 1, 2, 4, 6, 9, 12, 16, 20, 25, 30]
\\ n=1  2  3  4
\\
\\     a       *
\\    / \      |        a >= depth of branch
\\   *   *     a        b >= c so no double count
\\   |        / \
\\   b       b   c
\\
\\ Binomial choose a and b but that double counts b,c.
\\ Add 1 for each n-a even so as to bring up to double all.
\\ Then halve.
{
  check_equal(vector(12,n,
                     (binomial((n-1),2) + (n-1)\2)/2),
              vector(12,n, (n-1)^2\4));

  check_equal(vector(100,n,
                    sum(a=0,n-2, (n-a-1)\2)),
              vector(100,n, (n-1)^2\4));

  check_equal(vector(12,n, (n-1)^2\4),
              vector(12,n,
                     my(vpar=vpar_unlabelled_first(n),
                        total=0);
                     until((vpar=vpar_unlabelled_next(vpar))=='none,
                           if(vpar_num_childless(vpar)==2, total++));
                     total));
}
\\-----------------------------------------------------------------------------
\\ A055278  num rooted trees with 3 leaves
\\    1, 3, 8, 18, 35, 62, 103, 161, 241,
\\  n=4  5  6
\\
\\        R           a >= 0 down
\\        |           b >= 0 down
\\        a           c >= 1 arm length
\\        | \         d >= 1 arm length
\\        b  c        e >= 1 arm length
\\        | \         d >= e so no double count
\\        d  e        c >= d if b=0 no double count
\\
\\
\\ n=5   *      *        *
\\      /|\     |\       |
\\     * * *    * *      *
\\     |        |\      /|\
\\     *        * *    * * *
\\     a=0     a=0      a=1
\\     b=0     b=1      b=0
\\     c=0     c=0      c=0
\\     d=1     d=0      d=0

\\ recurrence_guess(concat([0,0,0,0],OEIS_data("A055278")))
\\   cases mod 6
\\   1/48 * n^4 \
\\   + -5/36 * n^3 \
\\   + 3/8 * n^2 \
\\   + -5/12 * n
\\   + vector_modulo([1, 119/144, 7/9, 15/16, 8/9, 103/144],n)
{
  my(limit=13);
  check_equal(vector(limit,n,
                     if(n<4,0,
                     n*(3*n^3 - 20*n^2 + 54*n - 60)\/144)),
              vector(limit,n,
                     my(vpar=vpar_unlabelled_first(n),
                        total=0);
                     until((vpar=vpar_unlabelled_next(vpar))=='none,
                           if(vpar_num_childless(vpar)==3, total++));
                     total));
}


\\-----------------------------------------------------------------------------
\\ A001809 = n!  * n(n-1)/4
\\ Nikos Apostolakis: a(n-1) = num labelled rooted trees with exactly 2\
\\ childless vertices

A001809(n) = n>=0 || error();   n! * n*(n-1)/4;
OEIS_check_func("A001809",,['offset,0]);
check_equal(A001809(0), 0);
{
  for(n=1, test_limit_trees_search,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,-1), \\ trees root=1
        my(vpar=vpar_from_upascend(vec));
        if(vpar_num_childless(vpar)==2, count++));
     check_equal(count, A001809(n-1), "A001809 num with 2 childless"));
}


\\-----------------------------------------------------------------------------
progress("Wiener Index");

join_identify(vpar,v,sub) =
{
  sub=vpar_delete_vertices(sub,vpar_roots(sub));
  concat(vpar,apply(p->if(p==0,v,p+#vpar), sub));
}
check_equal(join_identify([0,1],1,[0,1]),[0,1,1]);
check_equal(join_identify([0,1,2],1,[0,1,2,3,4]),[0,1,2,1,4,5,6]);
{
  \\ path lengths in complete binary tree growing by level down
  \\   1
  \\   |
  \\   3
  \\  / \
  \\ 4   5
  my(make_n(n) =
     if(n==0,[0,1],
        my(sub=self()(n-1));
        sub=join_identify(sub,1,sub);
        vpar_join(vpar_make_path(n),n,sub)));
  check_equal(make_n(0), [0,1]);
  check_equal(make_n(1), [0,1,2,2]);
  \\ check_equal(vpar_Wiener_index(make_n(4)), 3+3+2);
  my(want=OEIS_data("A227713"));  \\ OFFSET=0
  want=want[1..min(10,#want)];
  check_equal(vector(#want,n,n--; vpar_Wiener_index(make_n(n))),
              want,
              "A227713 Wiener index of binary construction");
}

{
  \\ A186235 total over bistars
  \\ (n+4)*2^(n+1) + (n-2)*2^(2n+2)
  my(want=OEIS_data("A186235"));  \\ OFFSET=4
  want=want[1..min(14,#want)];
  check_equal(vector(#want,n,n+=3;
                     sum(x=2,n\2, my(y=n-x);
                         vpar_Wiener_index(vpar_make_bistar(x,y)))),
              want,
              "A186235 Wiener index total over bistars");
}
{
  \\ (n+4)*2^(n+1) + (n-2)*2^(2n+2)
  my(want=OEIS_data("A158681"));  \\ OFFSET=1
  want=want[1..min(14,#want)];
  check_equal(vector(#want,n, vpar_Wiener_index(vpar_make_complete_tree(2,n))),
              want, "A158681 Wiener index of complete binary tree");
}
{
  \\ caterpillar 3,2,2,2,3
  \\ recurrence_guess(OEIS_data("A180571"))
  my(want=OEIS_data("A180571"));  \\ OFFSET=2
  check_equal(vector(#want,n,n++;
    vpar_Wiener_index(vpar_make_caterpillar(vector(n,i,if(i==1||i==n,3,2))))),
              want, "A180571 Wiener index of caterpillar 3,2,2,3");
}
{
  \\ #OEIS_data("A122681") == 27
  \\ https://oeis.org/w/images/8/80/FreeWiener.txt
  my(want=OEIS_data("A122681"));  \\ OFFSET=1
  want=want[1..min(14,#want)];
  my(got=vector(#want,n,
                my(vpar=vpar_free_first(n),
                   total=0);
                until((vpar=vpar_free_next(vpar))=='none,
                      total+=vpar_Wiener_index(vpar));
                total));
  check_equal(got,want, "A122681 Wiener index total");
}

{
  \\ #OEIS_data("A122682") == 25
  \\ https://oeis.org/w/images/3/33/Rootedwiener.txt
  my(want=OEIS_data("A122682"));  \\ OFFSET=1
  want=want[1..min(12,#want)];
  my(got=vector(#want,n,
                my(vpar=vpar_unlabelled_first(n),
                   total=0);
                until((vpar=vpar_unlabelled_next(vpar))=='none,
                      total+=vpar_Wiener_index(vpar));
                total));
  check_equal(got,want, "A122681 Wiener index total");
}



\\-----------------------------------------------------------------------------
progress("Count Free Trees by Num Leaves");

\\ A055290
\\     k=2 k=3 k=4 k=5 k=6
\\ n=2: 1;
\\ n=3: 1,  0;
\\ n=4: 1,  1,  0;
\\ n=5: 1,  1,  1,  0;
\\ n=6: 1,  2,  2,  1,  0;

{
  my(want=OEIS_data("A055290"),  \\ OFFSET=2
     got=[]);
  for(n=2,14,
     if(#got>=#want,break);
     my(counts=vector(n),
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           counts[vpar_num_leaves(vpar)]++);
     got=concat(got,counts[2..#counts]));
  if(#got>#want,
     \\ partial row in A055290
     got=got[1..#want]);
  if(#got<#want,
     print("A055290 truncate to ",#got);
     want=want[1..#got]);
  check_equal(got,want, "A055290 num free trees by leaves");
}

\\-----------------------------------------------------------------------------
progress("Path()");

\\ A253412 number of minimal dominating sets
{
  my(want=OEIS_data("A253412"));  \\ OFFSET=1
  check_equal(vector(#want,n,
                     vpar_minimal_domsets_count(vpar_make_path(n))),
              want,
              "A238462 samples path minimal domsets count");
}
\\ recurrence_guess(OEIS_data("A253412"))
\\ nothing beyond gf

\\ vector(20,n, vpar_minimal_domsets_count(vpar_make_path(n)) - vpar_minimal_domsets_count(vpar_make_path(n-1)))
\\ not in OEIS: 4, 5, 7, 11, 13, 21, 27, 40, 54, 77, 108, 150, 212

\\-----------
\\ binomial tree minimal domsets count = 2^(2^(n-1))
{
  check_equal(vector(6,n,n--;
                     vpar_minimal_domsets_count(vpar_make_binomial_tree(2^n))),
              vector(6,n,n--; if(n==0,1, 2^(2^(n-1)))),
              "binomial tree 2^n minimal domsets count");

  check_equal(vector(6,n,n--;
                     vpar_minimal_domsets_count(vpar_make_binomial_tree(2^n))),
              vector(6,n,n--; if(n==0,1, 2^(2^(n-1)))),
              "binomial tree 2^n minimal domsets count");

  my(want=OEIS_data("A001146"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; 2^(2^n)),
              want,
              "A001146 2^(2^n)");
  check_equal(vector(#want,n,n--; vpar_minimal_domsets_count(vpar_make_binomial_tree(2^(n+1)))),
              want,
              "A001146 samples binomial tree 2^(n+1) minimal domsets count");
}


\\-----------------------------------------------------------------------------
progress("vpar_count_unlabelled()");

{
  my(want=OEIS_data("A000081",'bfile));
  check_equal(want[1], 0);  \\ n=0 reckoned 0 in OEIS, unlike 1 here
  check_equal(vpar_count_unlabelled(0), 1);
  want=want[^1];
  print("b000081.txt to n=",#want);
  check_equal(vpar_count_unlabelled_vector(#want), want,
              "vpar_count_unlabelled_vector() vs ~/OEIS/b000081.txt");
}
{
  my(inds=OEIS_data("A051420"));
  \\ 3, 10, 15, 343, 387, 1087, 2981
  inds=select(t->t<=1500, inds);
  if(0,
     print("inds ",inds);
     my(vec=vpar_count_unlabelled_vector(vecmax(inds)),
        terms=vecextract(vec,inds));
     system("rm -f /tmp/b185667.txt");
     for(n=1,#terms,
        write("/tmp/b185667.txt", Str(n," ",terms[n])));
     system("ls -l /tmp/b185667.txt");

     for(n=1,#terms,
        check_equal(isprime(terms[n]),1));
     quit);

  \\ # A185667 (b-file synthesized from sequence entry)
  \\ 1 2
  \\ 2 719
  \\ 3 87811
}


\\-----------------------------------------------------------------------------
progress("Dominating Sets");

{
  \\ A212633 path-n dompoly rows
  \\ starting domset size k=1
  my(want=OEIS_data("A212633"));  \\ OFFSET=1
  my(got=[], n=1);
  while(#got < #want,
        got=concat(got,Vecrev(Vec(vpar_dompoly(vpar_make_path(n))/x)));
        n++);
  check_equal(got[1..#want], want,
              "A212633 path-n dompoly rows");

  \\ recurrence_guess(vector(100,n,n--; vpar_dompoly(vpar_make_path(n),'y)))
  my(limit=50,
     g = (1 + y*x^2)/(1 - y*x*(1+x+x^2)));
  check_equal(g
              - sum(n=0,limit, x^n * vpar_dompoly(vpar_make_path(n),'y))
              + O(x^(limit+1)),
              0 + O(x^(limit+1)),
              "A212633 path-n dompoly bivar gf");
}
{
  \\ A000213 path-n num domsets = Tribonacci numbers
  my(want=OEIS_data("A000213"));
  want=want[^1];
  check_equal(vector(#want,n,n--; vpar_domsets_count(vpar_make_path(n))),
              want,
              "A212633 Tribonacci = path-n num domsets");
}

{
  \\ A213669 bistar-n+1,n+1 dompoly rows
  \\ starting domset size k=2
  my(want=OEIS_data("A213669"));  \\ OFFSET=1
  my(got=[], n=1);
  while(#got < #want,
        got=concat(got,Vecrev(Vec(vpar_dompoly(vpar_make_bistar(n+1,n+1))/x^2)));
        n++);
  check_equal(got[1..#want], want,
              "A213669 bistar-n+1,n+1 dompoly rows");

  \\ row poly per A213669
  check_equal(vector(20,n, vpar_dompoly(vpar_make_bistar(n+1,n+1))),
              vector(20,n, (x^n + x*(1+x)^n)^2),
              "bistar-n+1,n+1 dompoly");
  check_equal(vector(20,n, vpar_dompoly(vpar_make_bistar(n+1,n+1))),
              vector(20,n, x^2*(1+x)^(2*n) + 2*x^(n+1)*(1+x)^n + x^(2*n) ),
              "bistar-n+1,n+1 dompoly expanded");

  \\ centres and k-2 choices around them
  \\ or n leaves around omitted centre, the other centre, and k-n-1 around that
  \\ or both centres omitted and all 2n leaves
  check_equal(vector(20,n, vpar_dompoly(vpar_make_bistar(n+1,n+1))),
              vector(20,n, sum(k=0,2*n+2,
                      ( binomial(2*n,k-2) + 2*binomial(n,k-n-1) + if(k==2*n,1)
                      ) *x^k )));
}

\\-----------------------------------------------------------------------------
progress("Binomial Tree - Independent Sets");

\\ A052129, per formula by Jonathan Sondow in A052129
\\ n * (n-1)^2 * (n-2)^4 * ... * 2^(2^n-2)
A052129(n) =
{
  n>=0 || error();
  my(ret=1); for(k=2,n, ret=sqr(ret)*k); ret;
}
{
  my(want=OEIS_data("A052129"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; A052129(n)),
              want,
              "A052129 samples");

  \\ similar to PARI by Michael Somos in A052129
  check_equal(vector(16,n,n--; A052129(n)),
              vector(16,n,n--; prod(i=2,n, i^(2^(n-i)));),
              "A052129 product formula");

  \\ product_(k>=1) ((k+1)/k)^(1/2^k)
  \\ log(s) = Sum_(k>=1) 1/2^k* log((k+1)/k)
  \\ log(s) = Sum_(k>=1) 1/2^k* (log(k+1) - log(k))
  \\ suminf(k=1, 1/2^k* log((k+1)/k))
  \\ suminf(k=1, 1/2^k*(log(k+1) - log(k)))
  \\ 1/2*log(2) - 1/2*log(1)
  \\ 1/4*log(3) - 1/4*log(2)
  \\ 1/8*log(4) - 1/8*log(2)
  \\ suminf(k=1, 1/2^k*(log(k+1) - log(k)))
  \\ -1/2*log(1) == 0
  \\ suminf(k=1, log(1 + (k-1))/2^k)
  \\ suminf(k=1, -log(1/k)/2^k)
  \\ suminf(k=1, log(k^(1/2^k)))
  \\ log(1+k)

  \\ A(x)^2  = (1+x)^2 * A(x/(1+x))
  \\ my(A(n) = (n + 2 - n^(-1) + 4*n^(-2) - 21*n^(-3) + 138*n^(-4) - 1091*n^(-5) ));
  \\ print(A(A(x))/A(x/(1+x)));
  \\   print(A(x)^2);
  \\   print(A(x/(1+x)) * (1+x)^2);
  \\ quit;
  \\   print(A(x)^2/A(x/(1+x)));
  \\ print();
  \\   print(A(x)*x^6,"\n");
  \\   print((A(x)^2)/(A(x/(1+x))) * x^12,"\nxx\n");
  \\   quit;
  \\   my(g(x) = sum(n=0,6, A052129(n)*1/x^(n+1)) );
  \\   print(g(x)*x^6,"\n");
  \\   \\ print(g(x)^2*x^12,"\n");
  \\   \\ print(g(x/(1+x)) * (1+x)^2 *x^12,"\n");
  \\   \\ print(g(g(x)));
  \\   \\ print(g(x)^2);
  \\   print();
  \\   check_equal(g(x)^2, g(x/(1+x)) * (1+x)^2, "A052129 gf satisfies");
  \\   quit;

  \\ s^(2^n) / (n + 2 - 1/n + 4/n^2 - 21/n^3 + 138/n^4 - 1091/n^5 + ...)
  \\ s = 1.661687949633... = A112302
  \\ A112302 quadratic recurrence constant sqrt(1 * sqrt(2 * sqrt(3 * sqrt(4 * ...)))).
  \\ A114124 Decimal expansion of logarithm of A112302.

  \\ s^(2^n) / P(n) = n*(s^(2^(n-1)) / P(n-1))^2
  \\ P(n-1))^2 = n * P(n)
  \\ for(n=1.0,50, print(A052129(n)/A052129(n)^2*1.0));quit;
  \\ R(n) = a(n)/a(n-1)^2 = n
  \\ P(n) = (n + 2 - 1/n + 4/n^2 - 21/n^3 + 138/n^4 - 1091/n^5)
  \\ P(n)
  \\ P(n-1)^2/n
}
\\ A238462 low 0s of A052129 (which is a(n) = n*a(n-1)^2)
A238462(n) = fromdigits(vector(n,i,valuation(i,2)), 2);
{
  \\  2, 6, 12, 25, 50, 103, 206, 413
  my(want=OEIS_data("A238462"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; A238462(n)),
              want,
              "A238462 samples");
  check_equal(vector(16,n,n--; A238462(n)),
              vector(16,n,n--; valuation(A052129(n),2)),
              "A238462() vs 0s of A052129");

  \\ formula by Ridouane Oudra
  check_equal(vector(256,n,n--; A238462(n)),
              vector(256,n,n--; sum(k=2,n, valuation(k,2)<<(n-k))),
              "A238462() by sum");

  \\ formula with log2(n) steps
  my(floor_log2(n) = if(n==0,-oo, logint(n,2)));
  check_equal(vector(256,n,n--; sum(i=1,floor_log2(n),
                                    floor(2^n / (2^(2^i)-1)))),
              vector(256,n,n--; A238462(n)),
              "A238462() by_powers()");

  check_equal(vector(13,i, valuation(i,2)),
                                   [0,1,0,2,0,1,0,3,0,1,0,2,0]);
  check_equal(binary((1<<13)\3  ), [  1,0,1,0,1,0,1,0,1,0,1,0], " 3");
  check_equal(binary((1<<13)\15 ), [      1,0,0,0,1,0,0,0,1,0], "15");
  check_equal(binary((1<<13)\255), [              1,0,0,0,0,0], "63");
  check_equal(test_to_binary(13), 1101);

  check_equal(vector(10,i, valuation(i,2)),
                                  [0,1,0,2,0,1,0,3,0,1]);
  check_equal(binary(1<<10 \3  ), [  1,0,1,0,1,0,1,0,1], "A");
  check_equal(binary(1<<10 \15 ), [      1,0,0,0,1,0,0], "B");
  check_equal(binary(1<<10 \255), [              1,0,0], "C");
  check_equal(test_to_binary(10), 1010);
  check_equal(A238462(10), 413);
  check_equal((1<< 11)   \3, 682);
  check_equal(1/3, 5/15);
  check_equal(1/3, 85/255);

  \\ but making repeating bit patterns by divisions is not efficient
  \\ (would go word-wise adding if really needed)
  my(by_powers(n) =
     my(h=1<<n,i=1,ret=0);
     while(n>>=1,
           h\bitneg(0,1<<i) > 0 || error(n" "i);
           ret += h\bitneg(0,1<<i);
           i++);
     ret);
  check_equal(vector(256,n,n--; by_powers(n)),
              vector(256,n,n--; A238462(n)),
              "A238462() by_powers()");
}
check_equal(A238462(27)\8, 6 776 935);

\\ ENHANCE-ME: go by which of 1..n have valuation=1,2,3 etc

\\--------------
\\ A088679 a(n) = a(n-1)^2 * n / (n-1), n>1,  a(0) = 0, a(1) = 1.
\\ 0,1,2,6,48,2880,9953280,115579079884800
A088679(n) =
{
  n>=0 || error();
  if(n==0,0, prod(i=2,n, i^(2^max(0,n-i-1))));
}
{
  check_equal(A088679(3), 2*3);
  check_equal(A088679(4), 2^2*3*4);
  check_equal(A088679(5), 2^4*3^2*4*5, "n=5");

  my(want=OEIS_data("A088679"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; A088679(n)),
              want,
              "A088679 samples");

  \\ my(n=5,i=2); apply(i->n-i-1, [2..5])
  \\ my(n=5,i=2); apply(i->2^max(0,n-i-1), [2..5])

  check_equal(A088679(5), 2^4*3^2*4*5, "n=5");

  \\ c^(2^n)*(1-1/n+2/n^2-6/n^3+25/n^4-137/n^5+...)
  \\ c=1.28906475773
  \\ c^2 = 1.6616879
}

\\-----------
A347289_binomial_tree_indsets_count(n) =
{
  \\ my line of code
  my(P=1); for(k=2,n,P=sqr(P)*k); (n+2)*P;
}
{
  \\ Squaring is faster by up to 10* when get big.
  \\ Shift for factors of 2 about 2x faster for big.
  \\ But would want to work with prime factorization if actually using big.
  if(0,
     my(limit=22);
     my(x,y,z);
     gettime(); for(n=1,limit, x=A347289_binomial_tree_indsets_count(n)); print(gettime()"  A347289_binomial_tree_indsets_count()");
     my(A347289_other(n) = my(P=1,s=0);
        for(k=2,n, my(v=valuation(k,2)); P=sqr(P)*(k>>v); s=s<<1+v);
        (P<<s)*(n+2);
        );
     gettime(); for(n=1,limit, y=A347289_other(n)); print(gettime()"  A347289_other()");
     my(A347289_by_product(n) = (n+2) * prod(k=2,n, k^(2^(n-k))));
     gettime(); for(n=1,limit, z=A347289_by_product(n)); print(gettime()"  A347289_by_product()");
        my(want=A347289_binomial_tree_indsets_count(limit));
        x==want||error();
        y==want||error();
        z==want||error();
       );
}
{
  \\ A347289 = indsets of binomial tree level k (mine)

  my(a=A347289_binomial_tree_indsets_count,
     with(n)    = n>=0||error(); A052129(n),
     without(n) = n>=0||error(); A088679(n+1));

  check_equal(logint(a(20),2), 768 238);
  check_equal(vector(10,n,n--; a(n)),
              vector(10,n,n--; vpar_indsets_count(vpar_make_binomial_tree(2^n))),
              "A347289_binomial_tree_indsets_count() vs vpar");

  check_equal(vector(12,n,n--; a(n)),
              vector(12,n,n--; (n+2) * prod(k=2,n, k^(2^(n-k)))),
              "A347289_binomial_tree_indsets_count() formula product");
  check_equal(vector(12,n,n--; a(n)),
              vector(12,n,n--; A052129(n) + A088679(n+1)),
              "A347289_binomial_tree_indsets_count() formula sum");
  check_equal(vector(12,n, a(n)),
              vector(12,n, a(n-1)^2 - A052129(n-1)^2),
              "A347289_binomial_tree_indsets_count() formula not-both-with");
  check_equal(a(5), 7 * 5 * 4^2 * 3^4 * 2^8, "example");
  check_equal(a(5), 11612160,                "example");

  check_equal(vpar_is_free_isomorphic(vpar_make_binomial_tree(2^2),
                                      vpar_make_path(4)), 1);

  \\ tree without root and without its children to force root in the set
  check_equal(vector(8,n,n--;
                     my(vpar=vpar_make_binomial_tree(2^n),
                        r=vpar_root(vpar),
                        children=vpar_children_of_vertex(vpar,r));
                     vpar=vpar_delete_vertices(vpar,concat(r,children));
                     vpar_indsets_count(vpar)),
              vector(8,n,n--; with(n)),
              "A052129 binomial tree 2^n num indsets with root");

  \\ tree without root to force it not in the set
  check_equal(vector(7,n,n--;
                     my(vpar=vpar_make_binomial_tree(2^n),
                        r=vpar_root(vpar));
                     vpar=vpar_delete_vertex(vpar,r);
                     vpar_indsets_count(vpar)),
              vector(7,n,n--; without(n)),
              "A088679 binomial tree 2^n num indsets without root");

  check_equal(vector(7,n, with(n)),
              vector(7,n, with(n-1)*without(n-1)),
              "with by product");
  check_equal(vector(7,n, without(n)),
              vector(7,n, without(n-1)*(with(n-1) + without(n-1))),
              "without by product with+without");
  check_equal(vector(7,n, without(n)),
              vector(7,n, without(n-1)*a(n-1)),
              "without by product a(n)");

  check_equal(vector(7,n, a(n)),
              vector(7,n, with(n) + without(n)),
              "with + without");

  check_equal(vector(10,n,n--; without(n)/a(n)),
              vector(10,n,n--; (n+1)/(n+2)),
              "A347289_binomial_tree_indsets_count() vs vpar");

  check_equal(vector(10,n,n--; with(n)/a(n)),
              vector(10,n,n--; 1/(n+2)),
              "A347289_binomial_tree_indsets_count() vs vpar");

  \\ DATA section
  if(0,
     for(n=0,9,print1(A347289_binomial_tree_indsets_count(n),",");if(n==7||n==8,print()));
     print(); quit);

  my(want=OEIS_data("A347289"));
  check_equal(vector(#want,n,n--; a(n)),
              want,
              "A347289_binomial_tree_indsets_count() samples");

  \\ ~/OEIS/b347289.txt
  if(0,
     system("rm -f /tmp/b347289.txt");
     for(n=0,oo,
        my(t=a(n), s=Str(t));
        print("n="n" length ",#s);
        if(#s>=1000, print("stop");break);
        write("/tmp/b347289.txt", Str(n," ",t)));
     system("ls -l /tmp/b347289.txt");
     quit);

  if(0,
     my(want=OEIS_data("A347289",'bfile));
     print("b347289.txt length ",#want);
     check_equal(vector(#want,n,n--; a(n)),
                 want,
                 "A347289_binomial_tree_indsets_count() b-file samples"));

  \\ for(n=1.0,50, print(without(n*1.0)/A347289_binomial_tree_indsets_count(n*1.0)));quit;
  \\ for(n=1.0,50, print((with(n*1.0)+without(n*1.0))/A347289_binomial_tree_indsets_count(n*1.0)));quit;
  \\ for(n=1.0,50, print(with(n*1.0)/A347289_binomial_tree_indsets_count(n*1.0)));quit;
  \\ for(n=1,20, print((A347289_binomial_tree_indsets_count(n*1.0))^(1/2^n))); quit;
  \\ for(n=1,20, print( (without(n*1.0))^(1/2^n) )); quit;
  \\ for(n=1,20, print( (A088679(n*1.0))^(1/2^n) )); quit;
  \\ for(n=1,20, print( (n*A052129(n*1.0))^(1/2^n) )); quit;
  \\ for(n=1,20, print( (1/n*without(n*1.0))^(1/2^n) )); quit;
  \\ for(n=1.0,26, print(with(n)/without(n)*1.0));quit;
}
\\ vector(8,n,n+=2-; valuation(A347289_binomial_tree_indsets_count(n),2))
\\ not in OEIS: 2, 7, 12, 28, 50
\\ vector(8,n,n+=2; valuation(A088679(n),2))
\\ not in OEIS: 1, 4, 6, 13, 25

{
  \\ print(vector(7,n,n--; vpar_indsets_count(vpar_make_path(2^n)))); quit;
  \\ not in OEIS: 2, 3, 8, 55, 2584, 5702887, 27777890035288

  \\ A000045
  check_equal(vector(20,n,n--; vpar_indsets_count(vpar_make_path(n))),
              vector(20,n,n--; fibonacci(n+2)));
  check_equal(vector(10,n, vpar_indsets_count(vpar_make_path(2^n-2))),
              vector(10,n, fibonacci(2^n)));
  my(want=OEIS_data("A058635"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; fibonacci(2^n)),
              want,
              "A058635 Fibonacci 2^n samples");

}
\\ vector(8,n,n+=2; vpar_indomsets_count(vpar_make_binomial_tree(2^n)))

\\ is ThueMorse 0<->1 complement, 1 at root then all of even num 1-bits
binomial_tree_lexmax_indset_flags(len) =
{
  my(ret=Vec([1],len));
  for(v=1,len-1, if(!ret[v-1<<valuation(v,2)+1],ret[v+1]=1));
  ret;
}
\\ print(binomial_tree_lexmax_indset_flags(100)); quit;
ThueMorse(n) = hammingweight(n)%2;
{
  my(len=2048,
     got=binomial_tree_lexmax_indset_flags(len));
  check_equal(got,
              vector(len,v,v--; 1-ThueMorse(v)));
  check_equal(got,
              vector(len,v,v--; hammingweight(v)%2==0));
}

\\--------------
\\ Binomial Tree - Maximum Independent Sets Same

\\ return 1 if every childless vertex is in/out of "set" opposite to its
\\ parent
is_set_childless_opposite_parent(vpar,set) =
{
  foreach(vpar_childless_vertices(vpar),v,
     if(vpar[v] && (setsearch(set,v)!=0) == (setsearch(set,vpar[v])!=0),
        return(0)));
  1;
}

binomial_tree_set_opposite_parent(n,set) =
{
  my(parents=2*set,
     all_children=vector(n,v,v--; 2*v+1),
     set_children=apply(v->v+1,parents),
     opp_children=setminus(all_children,set_children));
  setunion(parents,opp_children);
}

test_Set_to_flags(n,set) = vector(n,i, setsearch(set,i)!=0);
{
  my(with(n)    = n>=0 || error(); if(n==0,1,A052129(n-1)));
  my(without(n) = n>=0 || error(); if(n==0,1, A088679(n)));
  my(without_is_same(n) = n>1);

  \\ tree without root and without its children to force root in the set
  check_equal(vector(7,n,n--;
                     my(vpar=vpar_make_binomial_tree(2^n),
                        r=vpar_root(vpar),
                        children=vpar_children_of_vertex(vpar,r));
                     vpar=vpar_delete_vertices(vpar,concat(r,children));
                     vpar_indnum_and_count(vpar)),
              vector(7,n,n--; [bitneg(0,max(0,n-1)), with(n)]),
              "A052129 binomial tree 2^n num indsets with root");

  \\ tree without root to force it not in the set
  check_equal(vector(7,n,n--;
                     my(vpar=vpar_make_binomial_tree(2^n),
                        r=vpar_root(vpar));
                     vpar=vpar_delete_vertex(vpar,r);
                     vpar_indnum_and_count(vpar)[2]),
              vector(7,n,n--; without(n)),
              "A088679 binomial tree 2^n num indsets without root");


  check_equal(vector(10,n,n--; [2^max(0,n-1), with(n)+if(n,without(n))]),
              vector(10,n,n--; vpar_indnum_and_count(vpar_make_binomial_tree(2^n))),
              "binomial_tree_indsets_count() maximums vs vpar");

  \\ check_equal(vector(10,n,n--; [2^max(0,n-1), with(n)+if(n,without(n))]),
  \\             vector(10,n,n--; vpar_indomsets_count(vpar_make_binomial_tree(2^n))),
  \\             "binomial_tree_indsets_count() maximvs vpar");

  for(n=1,5,
     my(num=2^n,
        vpar=vpar_make_binomial_tree(num),
        sets=vpar_indnum_sets(vpar),
        flagsvecs=apply(set->test_Set_to_flags(num,set), sets));
     foreach(sets,set,
        check_equal(is_set_childless_opposite_parent(vpar,set), 1));

     \\ make indsets of n-1 by chopping maximums of n
     my(childful=vpar_childful_vertices(vpar),
        chop_flagsvecs=apply(flags->vecextract(flags,childful), flagsvecs),
        chop_sets=apply(test_flags_to_Set,chop_flagsvecs),
        prev_num=2^(n-1),
        prev_vpar=vpar_make_binomial_tree(prev_num),
        prev_sets=vpar_indsets(prev_vpar));
     check_equal(#prev_sets,#sets);
     chop_sets=vecsort(chop_sets);
     prev_sets=vecsort(prev_sets);
     check_equal(chop_sets,prev_sets,
                 "binomial tree indsets by chop maximum"));
}


\\-----------------------------------------------------------------------------
progress("Degree Sequences");

A265127(n) = prime(n) * 2^n;
{
  my(v=OEIS_data("A265127"));
  check_equal(vector(#v,n,A265127(n)), v, "A265127 samples");
}
\\ each element vec[i] is a prime index, return product all prime(vec[i])
vector_primeprod(vec) = vecprod(apply(prime,Vec(vec)));

\\--------
\\ partitions coded
\\ A036036 partitions in Abramowitz-Stegun order
\\ sort by length, then lex vectors with terms in increasing order
\\ partitions(8)

{
  my(want=OEIS_data("A036036"),
     got=[]);
  for(n=0,oo,
     if(#got>=#want,break);
     got=concat(got, Vec(concat(partitions(n)))));
  got=Vec(got,#want);  \\ shorten, if necessary
  check_equal(got,want, "A036036 partitions Abramowitz-Stegun order");
  check_equal(partitions(6),
              [Vecsmall([6]),           \\ sort first length
               Vecsmall([1, 5]),        \\ then lex ascending
               Vecsmall([2, 4]),        \\ with terms ascending
               Vecsmall([3, 3]),
               Vecsmall([1, 1, 4]),
               Vecsmall([1, 2, 3]),
               Vecsmall([2, 2, 2]),
               Vecsmall([1, 1, 1, 3]),
               Vecsmall([1, 1, 2, 2]),
               Vecsmall([1, 1, 1, 1, 2]),
               Vecsmall([1, 1, 1, 1, 1, 1])]);
}

\\ partitions coded by primes, in Abramowitz-Stegun order
A185974_row_AbSt_partitions(n) = apply(vector_primeprod,partitions(n));
\\ print(A185974_row_AbSt_partitions(2*6-2));
{
  my(want=OEIS_data("A185974"),
     got=[]);
  for(n=0,oo,
     if(#got>=#want,break);
     got=concat(got, A185974_row_AbSt_partitions(n)));
  got=got[1..#want];
  check_equal(got,want, "A185974 partitions prime products");

  check_equal(A185974_row_AbSt_partitions(9),
              [23,38,51,65,77,68,78,110, 98, 99, 105,125,104,132,140,126,150,135,176,168,200,180,162,224,240,216,320,288,384,512],
              "A185974 row 9");
}

\\ partitions sorted first by sum, then by length,
\\ then by lex descending with terms in descending order
A334438_cmp_partitions(x,y) = my(c=lex(#x,#y)); if(c,c, lex(Vecrev(y),Vecrev(x)));
A334438_partitions(n) = vecsort(partitions(n),A334438_cmp_partitions);
A334438_row(n) = apply(vector_primeprod, A334438_partitions(n));
\\ for(n=1,6,print(A334438_partitions(n)));
\\ quit
{
  check_equal(A334438_partitions(4),
              [Vecsmall([4]),
               Vecsmall([1,3]),
               Vecsmall([2,2]),
               Vecsmall([1,1,2]),
               Vecsmall([1,1,1,1])]);
  check_equal(A334438_partitions(4), partitions(4));

  check_equal(A334438_row(9),
              [23,38,51,65,77,68,78,110, 99, 98, 105,125,104,132,140,126,150,135,176,168,200,180,162,224,240,216,320,288,384,512],
              "A185974 row 9");

  my(want=OEIS_data("A334438"));
  my(got=[]);
  for(n=0,oo,
     if(#got>=#want,break);
     got=concat(got, A334438_row(n)));
  got=got[1..#want];
  check_equal(got,want, "A334438 partitions prime products, revlex");

  check_equal(vector(15,n, A185974_row_AbSt_partitions(n) == A334438_row(n)),
              vector(15,n, n<=8))
}
{
  my(want=OEIS_data("A215366"));
  my(got=[]);
  for(n=0,oo,
     if(#got>=#want,break);
     got=concat(got, vecsort(apply(vector_primeprod,partitions(n)))));
  got=got[1..#want];
  check_equal(got,want, "A215366 partitions prime products - sorted");
}


\\--------
\\ A345970 series reduced degree sequences coded by primes

\\ return 1 if vector "vec" contains within it vector "sub" as a
\\ subsequence, with terms in same order but not necessarily consecutive
vector_is_subsequence(vec,sub) =
{
  my(s=1);
  for(i=1,#vec,
     if(vec[i]==sub[s], s++;
        if(s++>#sub,return(1))));
  s==#sub+1;
}

\\ p is a partition as from partitions() or forpart()
\\ return a similar partition which is a tree degrees sequence
partition_to_tree_degrees(p) =
{
  p=Vec(p);
  my(t=vecsum(p));
  concat(vector(t+2-#p,i,1),apply(e->e+1,p));
}

\\ Return a vector of all the degree sequences (vectors in partition form)
\\ of the series-reduced trees of n vertices
series_reduced_degree_sequences(n) = \
  if(n<4, [], apply(partition_to_tree_degrees, partitions(n-2,[2,n-2])));
{
  \\ code by Charles R G in A002865
  my(A002865(n) = if(n, numbpart(n)-numbpart(n-1), 1));
  OEIS_check_func("A002865",A002865);

  for(n=1,20,
     my(vec=series_reduced_degree_sequences(n));
     check_equal(vecsort(vec),vec,
                 "series_reduced_degree_sequences() is sorted");
     check_equal(#vec, if(n<=2,0, A002865(n-2)),
                 "series_reduced_degree_sequences() count"));
}

A345970_row(n) = apply(vector_primeprod,series_reduced_degree_sequences(n));
{
  my(A345970_samps=[
 40,112,352,400,832,1120,2176,3520,3136,4000,4864,8320,9856,11200,
 11776,21760,23296,30976,35200,31360,40000,29696,48640,60928,73216,
 83200,98560,87808,112000,63488,117760,136192,191488,173056,217600,232960,309760,275968,352000,313600,400000
                   ]);
  my(got=[]);
  for(n=1,12,
     got=concat(got, A345970_row(n));
  );
  check_equal(got, A345970_samps);

  for(n=4,20,
     check_equal(vector_is_subsequence(A185974_row_AbSt_partitions(2*n-2),
                                       A345970_row(n)),
                 1));
  for(n=4,30,
     my(vec=apply(vector_primeprod,series_reduced_degree_sequences(n)));
     check_equal(vec[1], A265127(n-1),
                 "column k=1 = A265127  2^k*prime(k)"));
}

\\--------
\\ A345971 Series Reduced Trees Count by Degree Sequence

{
  my(A345971_samps=[
                    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 2, 3, 1, 4, 1, 1, 1, 1, 1, 2, 3, 2, 2, 4, 6, 2, 1, 1, 1, 1, 1, 2, 3, 3, 2, 2, 4, 9, 4, 8, 1, 1, 1, 1, 1, 1, 2, 3, 3, 2, 2, 3, 1, 4, 9, 6, 9, 2, 8, 14, 4, 1, 1, 1, 1, 1, 1, 2, 3, 3, 3, 2, 3, 2, 2, 4, 9, 9, 9, 9, 4, 8, 25, 14, 15
]);
  my(A345971_row(n) =
     my(m=Map(),
        vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(degrees=vecsort(vpar_degrees_vector(vpar)));
           for(i=1,#degrees,if(degrees[i]==2,next(2)));  \\ only series-reduced
           my(c=0); mapisdefined(m,degrees,&c);
           mapput(m, degrees, c+1));
     m=Mat(m);
     my(degs=m[,1]~,
        counts=m[,2]~);
     degs==Set(degs) || error();
     degs==vecsort(degs,lex) || error();  \\ counts to be in deg increasing order
     counts);

  my(got=[]);
  for(n=4,18,
     if(#got>=#A345971_samps,break);
     my(row=A345971_row(n));
     got=concat(got, row);
     for(i=1,(n-2)\2,      \\ first (n-2)\2 are uniques, one tree of deg seq
        check_equal(row[i], 1));
     \\ my(after=(n-2)\2+1);
     \\ if(#row>=after,
     \\    print("n="n" elem "after" = "row[after]));
  );
  check_equal(got,A345971_samps,
              "A345971 series reduced count by degree sequence");

  check_equal(A345971_row(15),
 [ 1, 1, 1, 1, 1, 1, 2, 3, 3, 3, 2, 3, 2, 2, 4, 9, 9, 9, 9, 4, 8, 25, 14, 15 ],
              "A345971_row(15)");
}


\\-----------------------------------------------------------------------------
\\ A005196  total num components in free forests, and more

{
  my(max_n=12,
     total_components=vector(12),
     total_singletons=vector(12),
     count=vector(12),
     count_no_singletons=vector(12),
     all_by_num_components=[],
     all_by_num_edges=[]);
  for(n=1,max_n,
     my(vpar=vpar_free_first(n),
        by_num_components=vector(n),
        by_num_edges=vector(n));
     until((vpar=vpar_free_next_forest(vpar,1))=='none,
           my(num_components=vpar_num_components(vpar),
              num_edges=vpar_num_edges(vpar),
              num_singletons=vpar_num_singletons(vpar));
           total_components[n] += num_components;
           total_singletons[n] += num_singletons;
           count_no_singletons[n] += (num_singletons==0);
           count[n]++;
           by_num_components[num_components]++;
           by_num_edges[num_edges+1]++;
           num_components + num_edges == n || error();
          );
     all_by_num_components=concat(all_by_num_components,by_num_components);
     all_by_num_edges=concat(all_by_num_edges,by_num_edges);
     \\ print("mean ",total_components[n]/count[n]*1.0);
  );

  check_equal(total_components,
              OEIS_data("A005196")[1..max_n],);
  check_equal(count,
              vpar_count_free_vector(max_n,1), \\ forests
              "count free forests");
  check_equal(count_no_singletons,
              OEIS_data("A144958")[1..max_n],
              "A144958 num free forests no singletons");

  \\ by components and by edges are row reversals,
  \\ since components + edges = n
  check_equal(all_by_num_components,
              OEIS_data("A095133")[1..#all_by_num_components],
              "A095133 num free forests no singletons");
  check_equal(all_by_num_edges,
              OEIS_data("A136605")[1..#all_by_num_edges],
              "A136605 num free forests no singletons");
}
\\ total singletons
\\ not in OEIS: 1, 2, 4, 7, 13, 23, 43, 80, 156, 309, 638, 1348, 2949, 6607, 15206, 35720

\\ num forests no singletons
\\ A144958 = A005195(n) - A005195(n-1)
\\ 0, 1, 1, 3, 4, 10, 17, 39, 77, 176, 381, 891, 2057, 4941, 11915, 29391
{
  my(want=OEIS_data("A144958"),
     count=vpar_count_free_vector(#want,1)); \\ forests
  check_equal(vector(#want,i, if(i==1,0, count[i] - count[i-1])),
              want,
              "A144958 by vpar_count_free_vector() differences");
}


\\-----------------------------------------------------------------------------
\\ A005197  total num components in unlabelled forests
\\ mean about 2.19

{
  my(v=OEIS_data("A005197"));  \\ OFFSET=1
  v=v[1..min(12,#v)];
  check_equal(vector(#v,n,
                     my(vpar=vpar_unlabelled_first(n),
                        total=0, count=0);
                     until((vpar=vpar_unlabelled_next(vpar,1))=='none,
                           total += vpar_num_components(vpar);
                           \\ total += vpar_num_singletons(vpar);
                           count++;
                          );
                     \\ print("mean ",total/count*1.0);
                     count==vpar_count_unlabelled(n,1) || error();
                     total),
              v, "A005197 samples");

  \\ a(n) ~ c * d^n / n^(3/2), where d = A051491  (Vaclav Kotesovec in A005197)

  \\ A. Meir and J. W. Moon, On the Altitude of Nodes in Random
  \\ Trees.  Canad.  J. Math.  30(1978),
  \\ http://dx.doi.org/10.4153/CJM-1978-085-0
  \\ https://www.cambridge.org/core/journals/canadian-journal-of-mathematics/article/on-the-altitude-of-nodes-in-random-trees/F0149515107EDDB855A2EEA50929073C
  \\ https://www.cambridge.org/core/services/aop-cambridge-core/content/view/F0149515107EDDB855A2EEA50929073C/S0008414X0002811Xa.pdf/on-the-altitude-of-nodes-in-random-trees.pdf
}
{
  my(d = 2.955765285,    \\ A051491
     c = 2.85007275);
  A005197_approx(n) = c * d^n / n^(3/2);
  \\ not in OEIS: 2.850072
}
{
  my(c = 0.439924,      \\ A187770
     d = 2.955765285);  \\ A051491
  num_trees_approx(n) = c * d^n * n^(-3/2);
}
num_forests_approx(n) = num_trees_approx(n+1);
\\ for(n=1,10, print(vpar_count_unlabelled(n)," ",num_trees_approx(n))); quit;
\\ for(n=1,20, print(vpar_count_unlabelled(n,1)," ",num_forests_approx(n))); quit;
\\ for(n=1000,1000,print(A005197_approx(n)/num_forests_approx(n))); quit

\\ 2.19183 * 0.439924 * 2.955765285

\\ 2.85007275 / 0.439924 / 2.955765285
\\ 2.19183
\\ A261124 Decimal expansion of 'theta', the expected degree (valency) of the root of a random rooted tree with n vertices.
\\ A. Meir and J. W. Moon, On the Altitude of Nodes in Random Trees.
\\ Canad.  J. Math.  30(1978),
\\ http://dx.doi.org/10.4153/CJM-1978-085-0

\\ 2.19183740319712630647869950285753649110618350758245 / 2.95576528565199497471481752412319458837549230466359659535
\\ not in OEIS: 0.7415465002

{
  \\ nu is radius of convergence of T(x) rooted trees poly
  \\ slowish convergence?  want to overcome constant factor?
  \\ 5000 terms  0.9875576086903

  \\ localprec(105);
  \\ my(d=OEIS_data("A051491"));
  \\ d=fromdigits(d)/10.0^(#d-1);
  \\ my(nu=1/d);
  \\ print("nu="nu);
  \\ my(g=x*Polrev(vpar_count_unlabelled_vector(5000)));
  \\ print(poldegree(g));
  \\ print(subst(g,x,nu));
  \\ quit
}

{
  \\ E. M. Palmer and A. J. Schwenk, On the Number of Trees in a
  \\ Random Forest, J. Combin.  Theory, B 27 (1979), 109-121.
  \\ http://dx.doi.org/10.1016/0095-8956(79)90073-X
  \\ https://www.sciencedirect.com/science/article/abs/pii/009589567990073X
  \\ http://www.sciencedirect.com/science/article/pii/009589567990073X/pdf?pid=1-s2.0-009589567990073X-main.pdf
  \\ https://core.ac.uk/download/pdf/82660679.pdf  600k
  \\
  \\ 1.755510 mean num trees in an unlabelled forest
  \\ nu = .3383219
  \\ 1/.3383219 = 2.95576 = rooted tree growth A051491
  \\ gamma(-3/2) = 4*sqrt(Pi)/3 = 2.363271 = A245886

  my(want=[1,1,1,2,3,6,11,23,47,106,235,551,1301,3159,7741,19320]);
  check_equal(vector(#want,n,vpar_count_free(n)), want,
              "Palmer and Schwenk table t_n = num free trees");

  my(want=[1,2,3,6,10,20,37,76,153,329,710,1601,3658,8599,20514,49905]);
  check_equal(vector(#want,n,vpar_count_free(n,1)), want,
              "Palmer and Schwenk table F_n = num free forests");

  my(want=[1,3,6,13,24,49,93,190,381,803,1703,3755,8401,19338,45275,108229]);
  check_equal(OEIS_data("A005196")[1..#want], want,
      "Palmer and Schwenk table Sum n*F_p,n = free forests total components");

  \\ d=2.955765285651994974714
  \\ nu=0.33832185689920769519
  \\ sum_nu=0.51130879934124059897
  \\ 1.75549886
  \\
  \\ my(i=20); 1 / (1/nu^i - 1)
  my(d=OEIS_data("A051491"));
  d=fromdigits(d)/10.0^(#d-1);
  my(nu=1/d,
     tvec=vpar_count_free_vector(100),
     tpol=x*Polrev(tvec));
  check_equal(nearly_equal(suminf(k=1,nu^k), nu/(1-nu)), 1,
             "sum k>=1 of nu^k = nu/(1-nu)");
  check_equal(nearly_equal(suminf(k=1,nu^(2*k)), 1/(1/nu^2-1)), 1,
             "sum k>=1 of nu^(2k)");
  \\ my(e=1 + sum(k=1,1000, subst(tpol,'x, nu^k)));
  \\ my(e = 1 + sum(i=1,#tvec, tvec[i]* 1 / (1/nu^i - 1)));
  my(e = 1 + sum(k=1,1000, sum(i=1,#tvec, tvec[i]*(nu^(i*k)))));
  print("d=",d);
  print("nu=",nu);
  \\ print("tpol=",tpol);
  print("e=",e);
  print("subst ",sum(i=1,#tvec, tvec[i]*nu^i)); \\ 0.565387

  \\ for(i=1,#tvec, print(tvec[i]* 1 / (1/nu^i - 1))); quit;

  \\ wrong?
  \\ my(e = 2 + sum(i=1,#tvec, my(ni=nu^i);
  \\                tvec[i]* ni^2/(1-ni)));  \\ equation (37)
  \\ my(e = 2 + sum(i=1,#tvec, tvec[i]*sum(k=2,1000, (nu^(i*k)))));

  \\ Otter
  \\ alpha 0.3383219
  \\ beta 7.924780
}
{
  \\ equation 4
  my(t(n) = vpar_count_free(n),
     F(n) = vpar_count_free(n,1));
  check_equal(vector(50,p, F(p)),
              vector(50,p, 1/p * sum(k=1,p, F(p-k)*sumdiv(k,d, d*t(d)))),
              "Palmer and Schwenk (4)");
}


\\-----------------------------------------------------------------------------
progress("Vecsmall Support");

{
  \\ vpar.gp intro comments "Vector Types" about apply() and select() not on
  \\ Vecsmall
  if(lex(version,[2,13,1]) <= 0,
     check_equal(iferr(apply(n->n+1, Vecsmall([1,2,3])); 1,
                       e,
                       \\ print(e);
                       0),
                 0,
                 "apply() not on Vecsmall in gp up to 2.13.1");

     check_equal(iferr(select(n->n%2==0, Vecsmall([1,2,3])); 1,
                       e,
                       \\ print(e);
                       0),
                 0,
                 "select() not on Vecsmall in gp up to 2.13.1");
    );
}


\\-----------------------------------------------------------------------------
progress("Caterpillars");

\\ E. Czabarka, L. Szekely, and S. Wagner, "The Inverse Problem for Certain
\\ Tree Parameters", Discrete Applied Mathematics, volume 157, 2009, pages
\\ 3314-3319, section 3.  http://doi.org/10.1016/j.dam.2009.07.004

{
  \\ Czabarka Szekely, and Wagner, notes a tree of n vertices has at least
  \\ binomial(n+1,2) subtrees, by the path between two vertices, possibly
  \\ the same vertex.
  \\
  my(n_limit=20,
     limit=200);
  check_equal(binomial(20 +1,2), 210);
  check_equal(binomial(n_limit+1,2) > limit, 1);

  my(seen_all=Map(),
     seen_caterpillars=Map(),
     list48=List([]));
  for(n=0,n_limit,
     if(n>=13, print("caterpillar subcons n="n));
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar)) == 'none,
           my(num_subcons=vpar_num_subcons(vpar));
           if(num_subcons>limit, next);
           mapput(seen_all,num_subcons,1);
           if(num_subcons==48, listput(list48,vpar));
           if(vpar_is_caterpillar(vpar),
              mapput(seen_caterpillars,num_subcons,1))));
  my(unseen_all          = setminus([1..limit],Mat(seen_all)[,1]~));
  my(unseen_caterpillars = setminus([1..limit],Mat(seen_caterpillars)[,1]~));
  list48=Vec(list48);

  \\ 2, 4, 5, 7, 8, 9, 12, 13, 14, 16, 18, 19
  my(want_unseen_all=OEIS_data("A184164"));
  check_equal(setisset(want_unseen_all),1);
  check_equal(unseen_all, want_unseen_all,
              "A184164 not number of subtrees in all trees");

  \\ cut and paste from paper
  my(want_unseen_caterpillars=[
2,4,5,7,8,9,12,13,14,16,18,19,22,23,26,27,29,31,33,35,38,39,42,43,46,48,
50,52,54,60,61,64,65,68,72,77,79,85,89,93,96,123,157,183]);
  check_equal(setisset(want_unseen_caterpillars),1);
  check_equal(unseen_caterpillars, want_unseen_caterpillars,
              "A184164 not number of subtrees in caterpillars");

  check_equal(test_set_is_subset(unseen_all,unseen_caterpillars), 1);
  check_equal(setminus(unseen_caterpillars,unseen_all),
              [48, 61, 64, 79, 85, 93, 96, 123, 157, 183]);

  \\ 48 ways
  \\        1          *--*--*--*--*--*
  \\      / | \                 |
  \\     2  5  7                *--*
  \\     |  |  |
  \\     3  6  8
  \\     |
  \\     4
  \\ with    [36, 3, 2, 1,  2, 1, 2, 1]
  \\ without [12, 3, 1, 0,  1, 0, 1, 0]
  \\
  my(vpar=[0,1,2,3, 1,5, 1,7]);
  check_equal(list48, [vpar], "sole way for 48 subcons");
  check_equal(vpar_num_subcons(vpar),          48);
  check_equal(#vpar, 8);
  check_equal((3+1)*(2+1)*(2+1), 36);
  check_equal(binomial(4,2), 6);
  check_equal(binomial(2,2), 1);
  check_equal(3*(3+1)/2, 6);
  check_equal(2*(2+1)/2, 3);  \\ twice
  check_equal(36 + 6 + 3 + 3, 48);


  \\ binomial(n+1,2) minimum subtrees, by picking the path between 2
  \\ possibly same vertices
  check_equal(binomial(2 +1,2),  3);
  check_equal(binomial(3 +1,2),  6);
  check_equal(binomial(4 +1,2), 10);

  check_equal(vpar_count_free(1), 1);
  check_equal(vpar_count_free(2), 1);
  check_equal(vpar_count_free(3), 1);
}

\\-----------------------------------------------------------------------------
\\ test_make_caterpillar_gaps()
\\
\\ This style of gaps specification used in Czabarka et al, but think
\\ vpar_make_caterpillar() num leaves per spine is clearer.

\\ Could root first of the given vec, or last like Czabarka et al.
\\ Code now is last.  First would be vecrev of the vec.
test_make_caterpillar_gaps(vec) =
{
  my(v=vecsum(vec),p=1,
     vpar=vector(v+#vec,i,i-1));
  v++; forstep(i=#vec,2,-1, p+=vec[i]; vpar[v++]=p);
  vpar;
}
{
addhelp(test_make_caterpillar_gaps,
"vpar = test_make_caterpillar_gaps(vec)
vec is a vector of gaps between successive leaves of a caterpillar.
Return a vpar which is the caterpillar.

Vertex 1 is the root and the spine is s = vecsum(vec)+1 vertices long.");
}

\\ Czabarka et al example
\\   8   7   6   5   4   3    2   1
\\                             root
\\   *---*---*---*---*-----*---*---*
\\      / \          |    /|\
\\     *   *         *   * * *
\\    1,   0,        3,  1,0,0,    2
\\
\\   8---7---6---5---4-----3---2---1
\\      / \          |    /|\
\\    14   13        12  11 10 9
{
  my(C=[1,   0,        3, 1,0,0,    2]);
  check_equal(#C, 7);
  check_equal(vecsum(C), 7);
  my(vpar=test_make_caterpillar_gaps(C));
  \\                 1 2 3 4 5 6 7 8  9 10 11  12 13 14
  check_equal(vpar, [0,1,2,3,4,5,6,7, 3, 3, 3,  4, 7, 7]);
  check_equal(#vpar, 14);
}


\\-----------------------------------------------------------------------------
progress("A055327 triangle of asymmetric trees n vertices k childless");

{
  check_equal(vector(13,n, max(1,n\2)),
              [1,1,1,2,2,3,3,4,4,5,5,6,6],
              "A055327 row widths");

  my(want=OEIS_data("A055327"),
     got=[]);
  for(n=1,12,  \\ bit slow to search and discard the symmetrics
     my(vpar=vpar_unlabelled_first(n),
        row=vector(max(1,n\2)));
     until((vpar=vpar_unlabelled_next(vpar))=='none,  \\ trees
           if(vpar_is_asymmetric(vpar),
              row[vpar_num_childless(vpar)]++));
     got=concat(got,row);
     if(#got>=#want,break));
  want=Vec(want,min(#got,#want));
  got =Vec(got,min(#got,#want));
  check_equal(got,want,
              "A055327 triangle of asymmetric trees n vertices k childless");
}

\\-----------------------------------------------------------------------------
progress("A337114 how many distinct unordered orbit sizes");

\\ has terms n=1 .. 12
{
  my(a=(n)->
     my(m=Map,vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           mapput(m, vecsort(vpar_free_orbit_sizes(vpar)), 1));
     #m);

  my(samples=A337114_samples);
  samples=samples[1..min(#samples,14)];
  check_equal(vector(#samples,n, a(n)),
              samples,
  "A337114 how many vpar_free_orbit_sizes() as partitions of n, with Map()");
}

\\-----------------------------------------------------------------------------
progress("A168083 Fibonacci 12");

{
  \\ Fibonacci-12 compositions parts 1 <= x <= 12
  my(want=OEIS_data("A168083"),
     a=(n)->want[n],
     num_comps=(n)->#select(c->#c==0||vecmax(c)<=12, test_compositions(n)));
  check_equal(a(1), 0);
  check_equal(a(11), 0);
  check_equal(a(12), 1);
  check_equal(a(13), 1);
  check_equal(a(14), 2);
  check_equal(num_comps(0), 1);
  check_equal(num_comps(1), 1);
  check_equal(num_comps(2), 2);
  check_equal(num_comps(3), 1+2+1);
  check_equal(vector(8,n, a(n+12)),
              vector(8,n, num_comps(n)),
              "A168083 Fibonacci 12  a(n+12)");

  check_equal(vector(14,n, a(n)),
              vector(14,n, num_comps(n-12)),
              "A168083 Fibonacci 12  comps(n-12)");
}


\\-----------------------------------------------------------------------------
progress("A092921 table of m-naccis");

\\ my line in A092921
A092921_by_polmod(k,n) =
{
  k>=1 || error();
  n>=0 || error();
  if(n==0,0, polcoeff(lift(Mod('x,Pol(vector(k+1,i,if(i==1,1,-1))))^(n+k-2)), k-1));
}

{
  \\ T() code by Max Hasler in A092921
  my(T= (m,n)->
     m>=1 || error();
     n>=0 || error();
     \\ m>=1 || error("m="m);
     \\  if(m==0,'z,
     !!n*(matrix(m, m, i, j, j==i+1||i==m)^(n+m-2))[1,m]
    );
  check_equal(matrix(10,100,k,n,n--; A092921_by_polmod(k,n)),
              matrix(10,100,k,n,n--; T(k,n)),
              "A092921_by_polmod() vs T(m,n)");

  my(want=OEIS_data("A092921"));
  check_equal(test_antidiagonals_of_func(#want,A092921_by_polmod,
                                    1,0,  \\ row k>=1, col n>=0
                                    1),   \\ but downwards
              want,
              "A092921_by_polmod() vs A092921 samples");

  \\ for(k=1,10,
  \\    for(n=2,k+1,
  \\       check_equal(A092921_by_polmod(k,n), 2^(n-2),
  \\                   "A092921 pow 2")));
}


\\-----------------------------------------------------------------------------
progress("A106640 ordered trees with somehere an only-child");

{
  my(limit=12,
     got=vector(limit));
  for(n=2,limit,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar))=='none,  \\ trees
           my(num_children=vpar_num_children_vector(vpar));
           num_children=select(c->c!=0, num_children);
           if(vecmin(num_children)==1,
              got[n]++)));
  got=got[^1];
  my(want=OEIS_data("A106640"));
  check_equal(got, want[1..#got],
              "A106640 ordered trees");
}


\\-----------------------------------------------------------------------------
progress("A117356 A117357 num trees weight n");
\\ "weight" of vertex = depth
\\ root as depth 0, or root as depth 1

{
  \\ A117356 OFFSET=0
  \\ A117357 OFFSET=0
  for(onebase=0,1,
     my(limit=12,
        got=vector(limit));
     for(n=1,limit+1,
        my(vpar=vpar_unlabelled_first(n));
        until((vpar=vpar_unlabelled_next(vpar))=='none,  \\ trees
              my(weight=vecsum(vpar_depths_vector(vpar))
                        + onebase * #vpar);  \\ for 1-based depths
              \\ print(vpar" weight="weight);
              my(w=weight+1);
              if(w<=#got, got[w]++)));
     my(anum=Str("A", 117356+onebase),
        want=OEIS_data(anum));
     check_equal(got, want[1..#got],
                 "A117356, A117357 unlabelled with total depths"));
}


\\-----------------------------------------------------------------------------
progress("A228180 total only-child vertices on boundary");
\\
\\ Dennis E. Davenport, Lara K. Pudwell, Louis W. Shapiro, Leon C. Woodson,
\\ "The Boundary of Ordered Trees", section 4 theorem 6.
\\
\\ Num singleton edges on boundary of all ordered trees.
\\ Singleton edge = below a parent which has just 1 child.
\\ Boundary edges = left-most and right-most sides.
\\
\\ n=4
\\   *     *       *       *       *
\\   |     |      / \     / \     /|\
\\   *     *     *   *   *   *   * * *
\\   |    / \    |           |
\\   *   *   *   *           *
\\   |
\\   *
\\ vpar_count_ordered(4) == 5
\\ 3+1+1+1+0 == 6

{
  my(want=OEIS_data("A228180"),  \\ OFFSET=0 edges
     L = O(x^#want),
     gA228180 = Polrev(want) + L,
     gC = test_func_to_gf(n->vpar_count_ordered(n,1), #want));  \\ Catalans
  check_equal(gC ==
              1 + 1*x + 2*x^2 + 5*x^3 + 14*x^4 + 42*x^5 + 132*x^6 + O(x^7),
              1,
              "gC Catalans samples");
  check_equal((1 - sqrt(1-4*x + x*L))/(2*x),
              gC + L,
              "gC Catalans formula");

  check_equal(gA228180 ==
              1*x + 2*x^2 + 6*x^3 + 19*x^4 + 61*x^5 + 199*x^6 + 661*x^7
              + 2234*x^8 + O(x^9),
              1,
              "gA228180 samples");
  check_equal(x* gC /(1-x)  + 2*x^3* gC^4 /(1-x) ==
              gA228180,
              1,
              "gA228180 from gC");
}

\\ left most and right most vertices, not including root(s)
boundary_vertices_preorder(vpar) =
{
  for(v=2,#vpar,
     if(vpar[v]!=v-1,
        my(a=vpar_ancestors(vpar,#vpar));
        return(Set(concat([2..v-1], a[^#a])))));
  [2..#vpar];
}
check_equal(boundary_vertices_preorder([0,1,2,3]), [2,3,4]);
check_equal(boundary_vertices_preorder([0,1,1,1,1,5]), [2,5,6]);
{
  my(want=OEIS_data("A228180"));  \\ OFFSET=0 edges, so 1 vertex
  for(n=1,min(12,#want),
     my(vpar=vpar_preorder_first(n),
        total=0);
     until((vpar=vpar_preorder_next(vpar))=='none,
           my(boundary=boundary_vertices_preorder(vpar),
              num_children=vpar_num_children_vector(vpar));
           for(i=1,#boundary,
              my(v=boundary[i]);
              if(num_children[vpar[v]]==1,
                total++)));
     check_equal(total, want[n]));
     \\ print(n"  "total, " ", want[n]));
}

boundary_vertices(vpar,side=0) =
{
  my(roots=vpar_roots(vpar),
     child=vpar_minmax_child_vector(vpar,if(side,max,min)));
  #roots || return([]);
  my(v=roots[1],len=0);
  while(child[v],len++;v=child[v]);
  my(ret=vector(len),upto=0);
  v=roots[1];
  while(child[v], ret[upto++]=v;v=child[v]);
  ret;
}

\\-----------------------------------------------------------------------------
progress("test_poly_num_real_roots()");

\\ polrootsreal() new in gp 2.9
check_equal(#polrootsreal(x^2+1), 0, "polrootsreal() no imaginaries");
check_equal(#polrootsreal(x^2-1), 2, "polrootsreal() 2");
check_equal(#polrootsreal(x^3), 3, "polrootsreal() cube");

{
  for(n=1,10,
     for(rep=1,3,
        my(poly=Pol(vector(n,i,random(20)-10)));
        check_equal(test_poly_num_real_roots(poly),
                    if(poly==0,0, #polrootsreal(poly)),
                    "test_poly_num_real_roots() vs polrootsreal()")));
}


\\-----------------------------------------------------------------------------
progress("abelian_group()");

{
  my(orders=Vecsmall([2,3]),
     group=abelian_group(orders));
  check_equal(group[1], [Vecsmall([2,1, 4,3, 6,5]),
                         Vecsmall([3,4, 5,6, 1,2])]);
  check_equal(group[2], orders);
  check_equal(group_order(group), 6);
  check_equal(group_isabelian(group), 1);

  my(n=group_domain(group));
  check_equal(n,6);
  my(elts=group_elts(group,n));
  check_equal(apply(test_permorder,elts), [1, 2, 3, 6, 3, 6]);

  my(C2 = test_perms_Cn(2),
     C3 = test_perms_Cn(3),
     C2xC3 = test_perms_direct_product(C2,C3));
  check_equal(apply(test_permorder,C2xC3), [1, 3, 3, 2, 6, 6]);
  check_equal(test_perms_is_isomorphic(C2xC3,elts), 1);
  check_equal(test_perms_is_isomorphic(C2xC3,test_perms_Cn(6)), 1);

  check_equal(group_abelianHNF(group,elts), [2,0; 0,3]);
  check_equal(group_abelianSNF(group,elts), [6]);

  check_equal(groupelts_abelian_group(elts), group,
              "groupelts_abelian_group()");
  check_equal(groupelts_exponent(elts), 6, "groupelts_exponent()");
  \\ check_equal(groupelts_centre(elts), []);

  \\ cf galoisexport()
  check_equal(group_export(group,0),  \\ GAP
              "Group((1, 2)(3, 4)(5, 6), (1, 3, 5)(2, 4, 6))");
  check_equal(group_export(group,1),  \\ Magma
              "PermutationGroup<6|[2, 1, 4, 3, 6, 5], [3, 4, 5, 6, 1, 2]>");
  check_equal(group_ident(group,elts), 2);
  check_equal(group_ident_trans(group,elts), 1);

  my(subgroup2=[group[1][1..1], group[2][1..1]]);
  my(subgroup3=[group[1][2..2], group[2][2..2]]);
  check_equal(group_subgroups(group),
              [group, subgroup2, subgroup3, [[], Vecsmall([])]]);
  check_equal(group_subgroup_isnormal(group,group), 1);
  check_equal(group_subgroup_isnormal(group,subgroup3), 1);
  check_equal(group_subgroup_isnormal(group,subgroup3), 1);

  my(n2=group_domain(subgroup2));
  check_equal(n2,6);
  my(elts2=group_elts(subgroup2,n2));
  check_equal(group_ident(subgroup2,elts2), 1);
  check_equal(group_ident_trans(subgroup2,elts2), 1);

  my(n3=group_domain(subgroup3));
  check_equal(n3,6);
  my(elts3=group_elts(subgroup3,n3));
  check_equal(group_ident(subgroup3,elts3), 1);
  check_equal(group_ident_trans(subgroup3,elts3), 1);

  my(q3=group_quotient(group,subgroup3));
  check_equal(q3,
              [[Vecsmall([1, 2, 3, 4, 5, 6]), Vecsmall([2, 1, 4, 3, 6, 5])],
               Vecsmall([1, 2, 1, 2, 1, 2])]);
  check_equal(quotient_group(q3,group),
              [[Vecsmall([2, 1])], Vecsmall([2])]);
}


\\-----------------------------------------------------------------------------
progress("test_smallgroup_validate()");

{
  my(data=[x^2+x+1,                 "S2",    2;
           x^3+x^2-2*x-1,           "A3=C3", 3;
           nfsplitting(x^3 + 2),    "S3",    6;
           x^4+x^3+x^2+x+1,         "C4",    4;
           nfsplitting(x^4+8*x+12), "A4",    12;
           nfsplitting(x^4+x+1),    "S4",    24;
           x^4-42*x^2+144,          "E4 = 2[x]2", 4;
           x^4 + 1,                 "E4 = 2[x]2", 4;
           x^6 + 108,               "D6 = [3]2", 6
          ]);
  for(i=1,matsize(data)[1],
     my(name=data[i,2],
        gal=galoisinit(data[i,1]));
     check_equal(gal!=0, 1, Str("galoisinit of "name));
     my(group=[gal.gen, gal.orders]);
     print("  "name" validate");
     \\ print(group);

     check_equal(galois_group(gal), [gal.gen,gal.orders]);

     test_smallgroup_validate(group);
     check_equal(#test_perms_generators_to_group(gal.gen),
                 data[i,3],
                 Str(name" group size"));

     check_equal(group_isA4S4(group), if(name=="A4", 1,
                                              name=="S4", 2,
                                              0),
                 "group_isA4S4()");
     check_equal(test_smallgroup_is_A4(group), name=="A4",
                 Str("test_smallgroup_is_A4() on "name));
     check_equal(test_smallgroup_is_S4(group), name=="S4",
                 Str("test_smallgroup_is_S4() on "name));
  );
}

{
  my(gens=[Vecsmall([2,3,1, 5,6,4]),
           Vecsmall([6,5,4, 3,2,1])],
     perms=test_perms_generators_to_group(gens));
  check_equal(test_perms_is_isomorphic(perms, test_perms_Sn(3)), 1,
             "S3");
  my(smallgroup=test_gens_to_smallgroup(gens));
  check_equal(smallgroup[2], Vecsmall([3,2]));
  test_smallgroup_validate(smallgroup);
}
\\ WRONG
\\ {
\\   my(gens=[Vecsmall([6,4,5, 2,3,1]),
\\            Vecsmall([2,1, 4,3, 6,5])],
\\      perms=test_perms_generators_to_group(gens));
\\   check_equal(test_perms_is_isomorphic(perms, test_perms_Sn(3)), 1,
\\              "S3");
\\   my(smallgroup=test_gens_to_smallgroup(gens));
\\   check_equal(smallgroup[2], Vecsmall([2,2]), "orders");
\\   test_smallgroup_validate(smallgroup);
\\ }


\\-----------------------------------------------------------------------------
progress("S4 Subgroups");

{
  my(K = nfsplitting(x^4+x+1),  \\ S4
     gal = galoisinit(K),
     smallgroup=[gal.gen,gal.orders]);
  check_equal(vector(#gal.group,n, test_smallgroup_n_to_elem(smallgroup,n)),
              gal.group);
  check_equal(gal.orders, Vecsmall([2,2,3,2]));
  check_equal(galois_group(gal), [gal.gen,gal.orders]);
  check_equal(group_isabelian(smallgroup), 0);
  check_equal(smallgroup, galois_group(gal));

  my(S4 = test_perms_Sn(4));
  check_equal(test_perms_is_isomorphic(gal.group,S4), 1);

  \\ first two are S2xS2
  my(perms=test_perms_generators_to_group(gal.gen[1..2]),
     S2 = test_perms_Sn(2),
     S2xS2 = test_perms_direct_product(S2,S2));
  check_equal(test_perms_is_isomorphic(perms,S2xS2), 1);

  \\ first three are A4
  perms=test_perms_generators_to_group(gal.gen[1..3]);
  my(A4 = test_perms_An(4));
  check_equal(test_perms_is_isomorphic(perms,A4), 1);

  \\ 2,3 are 12of
  perms=test_perms_generators_to_group(gal.gen[2..3]);
  check_equal(#perms,12);
  check_equal(test_perms_is_isomorphic(perms,A4), 1);
}


\\-----------------------------------------------------------------------------
progress("length() of Map()");

{
  my(m=Map());
  check_equal(#m, 0);
  mapput(m,123,456);
  check_equal(#m, 1);
  mapput(m,456,789);
  check_equal(#m, 2);
  mapdelete(m,456);
  check_equal(#m, 1);
}


\\------------------------------------------------------------------------------
progress("A4 making S2xS2");

{
  my(A4 = test_perms_An(4));
  check_equal(test_perm_to_cycles(A4[1]),   \\ id
              [Vecsmall([1]),Vecsmall([2]),Vecsmall([3]),Vecsmall([4])]);
  check_equal(test_perm_to_cycles(A4[4]),   \\ swaps (1 2)(3 4)
              [Vecsmall([1,2]),Vecsmall([3,4])]);

  my(sub1 = [A4[1],A4[4]],
     S2 = test_perms_Sn(2));
  check_equal(test_perms_is_isomorphic(sub1,S2), 1);

  check_equal(test_perm_to_cycles(A4[9]),   \\ cross-swaps (1 3)(2 4)
              [Vecsmall([1,3]),Vecsmall([2,4])]);
  my(sub2 = test_perms_generators_to_group([A4[4],A4[9]]),
     S2xS2 = test_perms_direct_product(S2,S2));
  check_equal(test_perms_is_isomorphic(sub2,S2xS2), 1);

  my(K = nfsplitting(x^4+8*x+12),  \\ A4
     gal = galoisinit(K),
     smallgroup=[gal.gen,gal.orders]);
  check_equal(vector(#gal.group,n, test_smallgroup_n_to_elem(smallgroup,n)),
              gal.group);

  \\ orders 2,2,3
  \\ first two are S2xS2
  check_equal(gal.orders, Vecsmall([2,2,3]));
  my(perms=test_perms_generators_to_group(gal.gen[1..2]));
  check_equal(test_perms_is_isomorphic(perms,S2xS2), 1);

  \\ 2,3 are 12of
  perms=test_perms_generators_to_group(gal.gen[2..3]);
  check_equal(#perms,12);
}

{
  my(C2 = test_perms_Cn(2),
     C3 = test_perms_Cn(3),
     C2xC3 = test_perms_direct_product(C2,C3),
     S3 = test_perms_Sn(3),
     D3 = test_perms_Dn(3));
  check_equal(#C2xC3, 6);
  check_equal(#S3, 6);
  check_equal(#D3, 6);
  check_equal(apply(test_permorder,C2xC3), [1, 3, 3, 2, 6, 6]);
  check_equal(apply(test_permorder,S3),    [1, 2, 2, 3, 3, 2]);
  check_equal(apply(test_permorder,D3),    [1, 3, 3, 2, 2, 2]);

  check_equal(test_perms_is_isomorphic(C2xC3,S3), 0);
  check_equal(test_perms_is_isomorphic(D3,S3), 1);
}


\\------------------------------------------------------------------------------
progress("vpar-to-graph6.c");
\\
\\ Compare vpar_to_graph6() etc with Nauty gtools.c run through program
\\ vpar-to-graph6.c here.  Nauty copyg does this sort of conversion, but
\\ circa its version 2.6 it doesn't read vpar.

test_vpar_to_graph6(vpar,opt="") =
{
  my(option_verbose=0);
  system("rm -f test-slow-in.tmp test-slow-out.tmp");

  for(v=1,#vpar, write1("test-slow-in.tmp", Str(" "vpar[v])));
  write("test-slow-in.tmp", "");
  system(Str("./vpar-to-graph6 ",
             if(option_verbose,"-v "," "),
             opt,
             " <test-slow-in.tmp >test-slow-out.tmp"));
  my(vec=readstr("test-slow-out.tmp"));

  system("rm -f test-slow-in.tmp test-slow-out.tmp");
  if(#vec==0, error("vpar-to-graph6 empty output",vec,
                    "\nIs ./vpar-to-graph6 program running ok?"),
     #vec!=1, error("vpar-to-graph6 bad output ",vec,
                    "\nIs ./vpar-to-graph6 program running ok?"));
  vec[1];
}

check_graph6_formats(vpar) =
{
  check_equal(vpar_to_graph6(vpar),
              test_vpar_to_graph6(vpar,"-g"),
              Str("vpar_to_graph6() "vpar));

  check_equal(vpar_to_sparse6(vpar),
              test_vpar_to_graph6(vpar,"-s"),
              "vpar_to_sparse6()");

  check_equal(vpar_to_digraph6(vpar,0),
              test_vpar_to_graph6(vpar,"-z0"),
              "vpar_to_digraph6() down=0");
  check_equal(vpar_to_digraph6(vpar,1),
              test_vpar_to_graph6(vpar,"-z1"),
              "vpar_to_digraph6() down=1");
  check_equal(vpar_to_digraph6(vpar,2),
              test_vpar_to_graph6(vpar,"-z2"),
              "vpar_to_digraph6() down=2");
}

{
  \\ cannot system() in secure mode (or not without querying the user)
  if(default(secure),
     print("skip vpar-to_graph6.c in secure mode");
     ,
     \\ graph6
     check_equal(test_vpar_to_graph6([],"-g"), "?");
     check_equal(test_vpar_to_graph6([0],"-g"), "@");
     check_equal(test_vpar_to_graph6([0,0],"-g"), "A?");

     \\ sparse6
     check_equal(test_vpar_to_graph6([],"-s"), ":?");
     check_equal(test_vpar_to_graph6([0],"-s"), ":@");
     check_equal(test_vpar_to_graph6([0,0],"-s"), ":A");

     \\ digraph6
     check_equal(test_vpar_to_graph6([],"-z0"), "&?");
     check_equal(test_vpar_to_graph6([0],"-z0"), "&@?");
     check_equal(test_vpar_to_graph6([0,0],"-z0"), "&A?");

     for(i=1,#test_sample_forests,
        my(vpar=test_sample_forests[i]);
        check_graph6_formats(vpar));

     for(n=1,50,
        for(reps=1,min(n,4),
           my(vpar=vpar_make_random(n));
           check_graph6_formats(vpar)));
     );
}


\\-----------------------------------------------------------------------------
progress("Symmetric Group S(n) Generators");

S2 = test_perms_Sn(2);
S2xS2xS2 = test_perms_direct_power(S2,3);
S2wrS2 = test_perms_wreath_product(S2,S2);
C3 = test_perms_Cn(3);
C3wrS2 = test_perms_wreath_product(C3,S2);

{
  \\ g1 = rot3 and other unchanged
  \\ g2 = 3s unchanged rot3
  \\ g3 = swap
  my(size=#C3wrS2,
     table=test_perms_Cayley_table(C3wrS2));
  check_equal(table,
      [1, 2, 3,  4, 5, 6,  7, 8, 9,  10,11,12, 13,14,15, 16,17,18;
       2, 3, 1,  5, 6, 4,  8, 9, 7,  13,14,15, 16,17,18, 10,11,12; \\ <-- g1
       3, 1, 2,  6, 4, 5,  9, 7, 8,  16,17,18, 10,11,12, 13,14,15;
       4, 5, 6,  7, 8, 9,  1, 2, 3,  11,12,10, 14,15,13, 17,18,16; \\ <-- g2
       5, 6, 4,  8, 9, 7,  2, 3, 1,  14,15,13, 17,18,16, 11,12,10;
       6, 4, 5,  9, 7, 8,  3, 1, 2,  17,18,16, 11,12,10, 14,15,13;
       7, 8, 9,  1, 2, 3,  4, 5, 6,  12,10,11, 15,13,14, 18,16,17;
       8, 9, 7,  2, 3, 1,  5, 6, 4,  15,13,14, 18,16,17, 12,10,11;
       9, 7, 8,  3, 1, 2,  6, 4, 5,  18,16,17, 12,10,11, 15,13,14;
       10,11,12, 13,14,15, 16,17,18,  1, 2, 3,  4, 5, 6,  7, 8, 9; \\ <-- g3
       11,12,10, 14,15,13, 17,18,16,  4, 5, 6,  7, 8, 9,  1, 2, 3;
       12,10,11, 15,13,14, 18,16,17,  7, 8, 9,  1, 2, 3,  4, 5, 6;
       13,14,15, 16,17,18, 10,11,12,  2, 3, 1,  5, 6, 4,  8, 9, 7;
       14,15,13, 17,18,16, 11,12,10,  5, 6, 4,  8, 9, 7,  2, 3, 1;
       15,13,14, 18,16,17, 12,10,11,  8, 9, 7,  2, 3, 1,  5, 6, 4;
       16,17,18, 10,11,12, 13,14,15,  3, 1, 2,  6, 4, 5,  9, 7, 8;
       17,18,16, 11,12,10, 14,15,13,  6, 4, 5,  9, 7, 8,  3, 1, 2;
       18,16,17, 12,10,11, 15,13,14,  9, 7, 8,  3, 1, 2,  6, 4, 5]);

  my(gs=vector(size,i, Vecsmall(table[i,])),
     g_to_i=((g)->test_vector_search(gs,g)),
     g1=gs[2],
     g2=gs[4],
     g3=gs[10],
     gens=[g1,g2,g3]);
  check_equal(test_permorder(g1), 3);
  check_equal(test_permorder(g2), 3);
  check_equal(test_permorder(g3), 2);
  check_equal(vector(size,i, test_permorder(Vecsmall(table[i,]))),
              [1,3,3, 3,3,3, 3,3,3,  2,6,6, 6,6,2, 6,2,6]);
  check_equal(g3*g1, g2*g3);
  check_equal(g3*g2, g1*g3);
  check_equal(g1*g2, g2*g1, "g1,g2 commute");

  my(sub9=(vec)->apply(n->n-9,vec));
  check_equal(apply(test_permorder,C3wrS2),
              [1, 3, 3, 3, 3, 3, 3, 3, 3, 2, 6, 6, 6, 6, 2, 6, 2, 6]);
  check_equal(sub9([13,14,15, 16,17,18, 10,11,12]),
              [4,5,6, 7,8,9, 1,2,3], "g1 upper");
  check_equal(sub9([11,12,10, 14,15,13, 17,18,16]),
              [2,3,1, 5,6,4, 8,9,7], "g2 upper");
  check_equal(sub9(table[2, 10..18]), table[4, 1..9]);
  check_equal(sub9(table[4, 10..18]), table[2, 1..9]);

  my(got_table=matrix(size,size));
  for(i=1,size,
     for(j=1,size,
        got_table[i,j] = g_to_i(gs[i]*gs[j])));
  check_equal(got_table, table);

  my(orders=[3, 3, 2],
     n_to_coords = ((n) -> my(vec=vector(#orders)); n--;
                    for(i=1,#orders, vec[i]=n%orders[i]; n\=orders[i]);
                    if(vec[3], my(t=vec[1]);vec[1]=vec[2];vec[2]=t);
                    n==0 || error();
                    vec),
     coords_to_n = ((vec) -> my(n=0, p=1);
                    if(vec[3], my(t=vec[1]);vec[1]=vec[2];vec[2]=t);
                    for(i=1,#orders, n += vec[i]*p; p*=orders[i]);
                    n+1),
     coords_to_g = ((vec) -> my(g=test_identity_perm(size));
                    for(i=1,#orders, g *= gens[i]^vec[i]);
                    g),
     g_list=vector(size,i, coords_to_g(n_to_coords(i))));
  \\ print("g_list");
  \\ for(i=1,18,print(g_list[i]"  "n_to_coords(i)));
  \\ print("gs");
  \\ for(i=1,18,print(gs[i]));
  for(i=1,18, check_equal(g_list[i], gs[i]));

  my(coords_prod = ((a,b) ->
                    if(a[3], my(t=b[1]);b[1]=b[2];b[2]=t);
                    vector(#orders,i, (a[i] + b[i]) % orders[i])),
     got_table= matrix(18,18,i,j,
                       coords_to_n(coords_prod(n_to_coords(i),n_to_coords(j)))));
  my(d=got_table- table);
  \\ for(i=1,18,print(got_table[i,]));
  \\ for(i=1,18,print(d[i,]));
  check_equal(got_table, table);
}


\\------------------------------------------------------------------------------
progress("vpar_is_preorder()");
\\ vpar_is_postorder()

\\ For post=0 return 1 if vpar is labelled in pre-order.
\\ For post=1 return 1 if varp is labelled in post-order.
test_is_porder_by_old_code(vpar,post) =
{
  \\ depths[v] is the depth of vertex v.
  \\ Depths d are an index into ancestors[] vector so start root depths[r]=1.
  \\ depths[v]=1 if not yet determined too.
  \\ ancestors[d] is the vertex number which is most recently seen at depth d.
  \\
  \\ prev_depth is the depth of vertex v-1.  The parent p of v must have
  \\ depth <= prev_depth.
  \\
  \\ For pre-order, each v must have its parent p as the most recently seen
  \\ vertex at the parent depth d=depths[p], ie. must ancestors[d]==p.
  \\ If p has not yet been seen, ie. p>v, then it is not pre-order.  This is
  \\ detected by depths[p]=1 for as yet unseen so fail ancestors[d]==p.
  \\ When p is ok set depths[v]=d+1 and recent at d+1 so ancestors[d+1]=v.
  \\
  \\ For post-order, the same but working end to start in vpar, so that an
  \\ unseen p is p<v so not post-order.

  my(depths    = vectorsmall(#vpar,v,1),
     ancestors = vectorsmall(#vpar),
     prev_depth=1, p);
  forstep(v=if(post,#vpar,1), if(post,1,#vpar), if(post,-1,1),
     my(d=1);
     if((p=vpar[v]),
        d=depths[p];
        if(d>prev_depth || ancestors[d]!=p,
           \\ print("  p="p" depth "depths[p]" "depths);
           return(0));
        d++);
     prev_depth=depths[v]=d;
     ancestors[d]=v);
  1;
}

{
  for(n=1,7,
     print("  n="n);
     forvec(vec=vpar_forvec_from(n),
           my(vpar=vpar_from_upascend(vec));
           check_equal(vpar_is_preorder(vpar),
                       test_is_porder_by_old_code(vpar,0));
           check_equal(vpar_is_postorder(vpar),
                       test_is_porder_by_old_code(vpar,1))));
}


\\-----------------------------------------------------------------------------
progress("Wreath Supersolvable");

Triv3 = [test_identity_perm(3)];
Triv4 = [test_identity_perm(4)];
C2 = test_perms_Cn(2);
C3 = test_perms_Cn(3);
C4 = test_perms_Cn(4);
check_equal(test_perms_is_Abelian(C2), 1);
check_equal(test_perms_is_Abelian(C3), 1);
check_equal(test_perms_is_Abelian(C4), 1);
check_equal(test_perms_is_2group(C2),  1);
check_equal(test_perms_is_2group(C3),  0);
check_equal(test_perms_is_2group(C4),  1);
check_equal(test_perms_is_nilpotent(C2),  1);
check_equal(test_perms_is_nilpotent(C3),  1);
check_equal(test_perms_is_nilpotent(C4),  1);
check_equal(test_perms_nilpotency_class(C3), 2);
check_equal(test_perms_nilpotency_class(C4), 2);
check_equal(test_perms_is_solvable(C3),  1);
check_equal(test_perms_is_solvable(C4),  1);
check_equal(test_perms_subgroups(C3), [Triv3,C3], "C3 subgroups");
C2_in_pairs = [Vecsmall([1,2,3,4]), Vecsmall([3,4,1,2])];
check_equal(test_perms_is_isomorphic(C2_in_pairs, C2), 1);
check_equal(test_perms_subgroups(C4), [Triv4, C2_in_pairs, C4], \
            "C4 subgroups");
{
  check_equal(test_perms_Cayley_table(C3),
              [1, 2, 3;
               2, 3, 1;
               3, 1, 2]);
  check_equal(test_perms_Cayley_table(C4),
              [1, 2, 3, 4;
               2, 3, 4, 1;
               3, 4, 1, 2;
               4, 1, 2, 3]);
}

C2xC2 = test_perms_direct_product(C2,C2);
C2xC3 = test_perms_direct_product(C2,C3);
C2wrC3 = test_perms_wreath_product(C2,C3);
C3wrC2 = test_perms_wreath_product(C3,C2);
check_equal(#C2xC3, 6);
check_equal(#C2wrC3, 24);
check_equal(#C3wrC2, 18);
check_equal(vecsort(apply(test_permorder,C2xC3)), [1,2, 3,3, 6,6]);
S3 = test_perms_Sn(3);
check_equal(vecsort(apply(test_permorder,S3)), [1, 2,2,2, 3,3]);
check_equal(apply(test_perm_is_even,C2xC2), [1,0,0,1]);
{
  check_equal(test_perms_Cayley_table(C2xC3),
              [1,2,3, 4,5,6;
               2,3,1, 5,6,4;
               3,1,2, 6,4,5;
               4,5,6, 1,2,3;
               5,6,4, 2,3,1;
               6,4,5, 3,1,2]);
  check_equal(test_perms_Cayley_table(S3),
              [1,2, 3,4, 5,6;
               2,1, 5,6, 3,4;
               3,4, 1,2, 6,5;
               4,3, 6,5, 1,2;
               5,6, 2,1, 4,3;
               6,5, 4,3, 2,1]);
  my(perms=concat([S3[1..2], S3[5..6], S3[3..4]]));
  check_equal(test_perms_Cayley_table(perms),
              [1,2, 3,4, 5,6;
               2,1, 5,6, 3,4;
               3,4, 6,5, 2,1;
               4,3, 2,1, 6,5;
               5,6, 4,3, 1,2;
               6,5, 1,2, 4,3]);
}
{
  \\ "D6" generators
  \\   [2,3,1, 5,6,4]
  \\   [4,6,5, 1,3,2]
  my(p = x^6 + 108,   \\ D_6(6) = [3]2
     G = galoisinit(p));
  check_equal(#G.group, 6);
  check_equal(vecsort(apply(test_permorder,G.group)), [1, 2,2,2, 3,3]);
  check_equal(test_perms_is_isomorphic(C2xC3,G.group), 0, "D6 != C2xC3");
  check_equal(test_permorder(Vecsmall([4,6,5, 1,3,2])), 2);
  check_equal(test_permorder(Vecsmall([6,5,4, 3,2,1])), 2);
  check_equal(test_vector_search(G.group, Vecsmall([6,5,4, 3,2,1])) !=0,  1);

  my(perms=test_perms_generators_to_group([Vecsmall([2,3,1, 5,6,4]),
                                           Vecsmall([6,5,4, 3,2,1])]));
  check_equal(test_perms_is_isomorphic(perms,G.group), 1);
  check_equal(test_perms_is_isomorphic(G.group, S3),  1);
}
{
  \\ C2xC3
  my(gens = [Vecsmall([2,3,1, 5,6,4]),
             Vecsmall([4,5,6, 1,2,3])],
     perms = test_perms_generators_to_group(gens));
  check_equal(#perms, 6);
  check_equal(apply(test_permorder,gens), [3,2]);
  check_equal(test_perms_is_isomorphic(perms,C2xC3), 1);
  check_equal(test_vector_search(C2xC3, Vecsmall([4,6,5, 1,3,2])), 0);
  check_equal(test_vector_search(C2xC3, Vecsmall([1,3,2, 4,6,5])), 0);
}
C2xC2_commutator = test_perms_commutator_subgroup(C2xC2);
check_equal(C2xC2_commutator, Triv4);
check_equal(test_perms_is_isomorphic(C2xC2_commutator, Triv3), 1);

A3 = test_perms_An(3);
A4 = test_perms_An(4);
A5 = test_perms_An(5);
A6 = test_perms_An(6);
A7 = test_perms_An(7);
check_equal(#A3, 3!/2);  \\ = 3
check_equal(#A4, 4!/2);
check_equal(12,  4!/2);
check_equal(#A6, 6!/2);
check_equal(#A7, 7!/2);
check_equal(A3, C3, "A3 = C3");
check_equal(test_perms_is_Abelian(A4),   0, "A4 not Abelian");
check_equal(test_perms_is_Abelian(A5),   0);
check_equal(test_perms_is_Abelian(A6),   0);
check_equal(test_perms_is_2group(A4),  0);
check_equal(test_perms_is_2group(A5),  0);
check_equal(test_perms_is_2group(A6),  0);
check_equal(test_perms_is_nilpotent(A4), 0, "A4 not nilpotent");
check_equal(test_perms_is_nilpotent(A5), 0);
check_equal(test_perms_is_nilpotent(A6), 0);
check_equal(test_perms_is_solvable(A4),  1, "A4 solvable");
check_equal(test_perms_is_solvable(A5),  0);
check_equal(test_perms_is_solvable(A6),  0);
\\ FIXME:
\\check_equal(test_perms_is_supersolvable(A4), 0, "A4 not supersolvable");
check_equal(test_perms_is_supersolvable(A5), 0);
C2_in_rev4 = [Vecsmall([1,2,3,4]), Vecsmall([4,3,2,1])];
check_equal(test_perms_is_isomorphic(C2_in_rev4, C2), 1);

\\ C3_fix[i] is C3 done in perms length 4 with i fixed
C3_fix = vector(4,n, apply(perm->test_perm_insert_n(perm,n), C3));
for(i=1,#C3_fix, check_equal(test_perms_is_isomorphic(C3_fix[i], C3), 1));

\\ C2xC2 but done with even permutations
C2xC2_evens = [Vecsmall([1, 2, 3, 4]), Vecsmall([2, 1, 4, 3]), \
               Vecsmall([3, 4, 1, 2]), Vecsmall([4, 3, 2, 1])];
check_equal(test_perms_is_isomorphic(C2xC2_evens, C2xC2), 1);
check_equal(apply(test_perm_is_even,C2xC2_evens), [1,1,1,1]);
{
  my(subs=test_perms_subgroups(A4),
     want=[Triv4,
           C2_in_rev4,
           C2_in_pairs,
           [Vecsmall([1,2, 3,4]), Vecsmall([2,1, 4,3])],
           C3_fix[2],
           C3_fix[3],
           C3_fix[4],
           C3_fix[1],
           C2xC2_evens,   \\ commutator
           A4
          ]);
  \\ print(C2xC2);
  \\ for(i=1,#subs, print(subs[i]"  ",if(i<=#want,subs[i]==want[i],"none")));
  check_equal(subs, want, "A4 subgroups");
}
{
  check_equal(test_perms_Cayley_table(A4),
              [1, 2, 3,  4, 5, 6, 7, 8, 9, 10, 11, 12;
               2, 3, 1,  7, 9, 8, 10, 12, 11, 4, 5, 6;
               3, 1, 2,  10, 11, 12, 4, 6, 5, 7, 9, 8;
               4, 6, 5,  1, 3, 2, 11, 10, 12, 8, 7, 9;
               5, 4, 6,  8, 7, 9, 1, 2, 3, 11, 12, 10;
               6, 5, 4,  11, 12, 10, 8, 9, 7, 1, 3, 2;
               7, 8, 9,  2, 1, 3, 5, 4, 6, 12, 10, 11;
               8, 9, 7,  5, 6, 4, 12, 11, 10, 2, 1, 3;
               9, 7, 8,  12, 10, 11, 2, 3, 1, 5, 6, 4;
               10, 12,  11, 3, 2, 1, 9, 7, 8, 6, 4, 5;
               11, 10,  12, 6, 4, 5, 3, 1, 2, 9, 8, 7;
               12, 11,  10, 9, 8, 7, 6, 5, 4, 3, 2, 1]);
}
A4_commutator = test_perms_commutator_subgroup(A4);  \\ C2xC2
check_equal(A4_commutator, C2xC2_evens);
check_equal(test_perms_is_isomorphic(A4_commutator, C2xC2), 1);

S2 = test_perms_Sn(2);
S3 = test_perms_Sn(3);
S4 = test_perms_Sn(4);
S5 = test_perms_Sn(5);
S6 = test_perms_Sn(6);
S7 = test_perms_Sn(7);
check_equal(S2, C2);
check_equal(#S2, 2);
check_equal(#S3, 6);
check_equal(#S3[1], 3);
check_equal(test_perms_is_2group(S2),  1);
check_equal(test_perms_is_2group(S3),  0);
check_equal(test_perms_is_nilpotent(S2), 1, "S2 nilpotent");
check_equal(test_perms_is_nilpotent(S3), 0, "S3 nilpotent");
check_equal(test_perms_is_nilpotent(S4), 0, "S4 nilpotent");
check_equal(test_perms_nilpotency_class(S2), 2, "S2 nilpotency class");
check_equal(test_perms_nilpotency_class(S3), 'none, "S3 nilpotency class");
check_equal(test_perms_nilpotency_class(S4), 'none, "S4 nilpotency class");
check_equal(test_perms_is_supersolvable(S2), 1);
check_equal(test_perms_is_supersolvable(S3), 1);
check_equal(test_perms_is_supersolvable(S4), 0);
check_equal(test_perms_is_Abelian(S2), 1);
check_equal(test_perms_is_Abelian(S3), 0);
check_equal(test_perms_is_Abelian(S4), 0);
S2_commutator = test_perms_commutator_subgroup(S2);  \\ trivial
S3_commutator = test_perms_commutator_subgroup(S3);  \\ C3
S4_commutator = test_perms_commutator_subgroup(S4);  \\ A4
S5_commutator = test_perms_commutator_subgroup(S5);  \\ A5
S6_commutator = test_perms_commutator_subgroup(S6);
S7_commutator = test_perms_commutator_subgroup(S7);
check_equal(#S2_commutator, 1, "S2_commutator size");
check_equal(#S3_commutator, 3);
check_equal(#S4_commutator, 12);
check_equal(#S5_commutator, 60);
check_equal(#S6_commutator, 6!/2);
check_equal(S3_commutator, C3, "S2_commutator = C3"); \\ = A3
check_equal(S4_commutator, A4);
check_equal(S5_commutator, A5);
check_equal(S6_commutator, A6);
check_equal(S7_commutator, A7);

S2xS2 = test_perms_direct_product(S2,S2);
S2xS3 = test_perms_direct_product(S2,S3);
S3xS3 = test_perms_direct_product(S3,S3);
check_equal(test_perms_is_involutions(S2xS2), 1, "S2xS2 involutions");
check_equal(test_perms_is_involutions(S2xS3), 0);
check_equal(test_perms_is_involutions(S3xS3), 0);
check_equal(test_perms_is_2group(S2xS2),  1, "S2xS2 2-group");
check_equal(test_perms_is_2group(S2xS3),  0);
check_equal(test_perms_is_2group(S3xS3),  0);
check_equal(test_perms_is_Abelian(S2xS2),     1, "S2xS2 Abelian");
check_equal(test_perms_is_Abelian(S2xS3),     0);
check_equal(test_perms_is_Abelian(S3xS3),     0);
check_equal(test_perms_is_nilpotent(S2xS2),     1, "S2xS2 nilpotent");
check_equal(test_perms_is_nilpotent(S2xS3),     0, "S2xS3 nilpotent");
check_equal(test_perms_is_nilpotent(S3xS3),     0, "S3xS3 nilpotent");
check_equal(test_perms_nilpotency_class(S2xS2), 2);
check_equal(test_perms_is_supersolvable(S2xS2), 1);
check_equal(test_perms_is_supersolvable(S2xS3), 1);
check_equal(test_perms_is_supersolvable(S3xS3), 1);
S2xS2_commutator = test_perms_commutator_subgroup(S2xS2);  \\ S2xS2
check_equal(S2xS2_commutator, [Vecsmall([1,2,3,4])], "S2xS2 commutator");

S2wrS2 = test_perms_wreath_product(S2,S2);
S2wrS3 = test_perms_wreath_product(S2,S3);
S3wrS2 = test_perms_wreath_product(S3,S2);
S3wrS3 = test_perms_wreath_product(S3,S3);
check_equal(#S2wrS2[1], 2*2);
check_equal(#S2wrS2,    2^2 * 2!);
check_equal(8,          2^2 * 2!);
check_equal(#S2wrS3[1], 2*3,       "S2wrS3 perm len");
check_equal(#S2wrS3,    2^3 * 3!,  "S2wrS3 num perms");
check_equal(48,         2^3 * 3!);
check_equal(#S3wrS2[1], 3*2,       "S3wrS2 perm len");
check_equal(#S3wrS2,    3!^2 * 2!, "S3wrS2 num perms");
check_equal(72,         3!^2 * 2!);
check_equal(#S3wrS3[1], 3*3,       "S3wrS3 perm len");
check_equal(#S3wrS3,    3!^3 * 3!, "S3wrS3 num perms");
check_equal(1296,       3!^3 * 3!);
check_equal(#select(test_perm_is_involution, S2wrS2), 6);
check_equal(select(perm->!test_perm_is_involution(perm), S2wrS2), \
            [ Vecsmall([3, 4, 2, 1]), Vecsmall([4, 3, 1, 2]) ]);
check_equal(Vecsmall([3,4, 1,2]) * Vecsmall([2,1, 3,4]),  Vecsmall([4,3, 1,2]));
check_equal(Vecsmall([2,1, 3,4]) * Vecsmall([3,4, 1,2]),  Vecsmall([3,4, 2,1]));
check_equal(Vecsmall([3, 4, 2, 1])^2,  Vecsmall([2,1, 4,3]));
check_equal(Vecsmall([3, 4, 2, 1])^3,  Vecsmall([4,3, 1,2]));
check_equal(Vecsmall([3, 4, 2, 1])^4,  Vecsmall([1,2, 3,4]));
check_equal(test_perms_is_involutions(S2wrS2), 0, "S2wrS2 is involutions");
check_equal(test_perms_is_involutions(S2wrS3), 0);
check_equal(test_perms_is_involutions(S3wrS2), 0);
check_equal(test_perms_is_involutions(S3wrS3), 0);
check_equal(test_perms_is_Abelian(S2wrS2),     0, "S2wrS2 Abelian");
check_equal(test_perms_is_Abelian(S2wrS3),     0);
check_equal(test_perms_is_Abelian(S3wrS2),     0);
check_equal(test_perms_is_Abelian(S3wrS3),     0);
check_equal(test_perms_is_2group(S2wrS2),  1, "S2wrS2 2-group");
check_equal(test_perms_is_2group(S3wrS2),  0);
check_equal(test_perms_is_2group(S2wrS3),  0);
check_equal(test_perms_is_2group(S3wrS3),  0);
check_equal(test_perms_is_nilpotent(S2wrS2), 1, "S2wrS2 nilpotent");
check_equal(test_perms_is_nilpotent(S3wrS2), 0);
check_equal(test_perms_is_nilpotent(S2wrS3), 0);
check_equal(test_perms_is_nilpotent(S3wrS3), 0);
check_equal(test_perms_nilpotency_class(S2wrS2), 3);
check_equal(test_perms_is_supersolvable(S2wrS2), 1, "S2wrS2");
check_equal(test_perms_is_supersolvable(S3wrS2), 0);
check_equal(test_perms_is_supersolvable(S2wrS3), 0);
check_equal(test_perms_is_supersolvable(S3wrS3), 0);

check_equal(#S2xS2xS2, #S2wrS2);
check_equal(test_perms_is_isomorphic(S2xS2xS2,S2wrS2), 0);
{
  check_equal(test_perms_Cayley_table(S2xS2xS2),
              [1, 2, 3, 4, 5, 6, 7, 8;
               2,1,  4,3,  6,5,  8,7;
               3, 4, 1, 2,  7, 8, 5, 6;
               4, 3, 2, 1,  8, 7, 6, 5;   \\ 4-revs
               5, 6, 7, 8,  1, 2, 3, 4;   \\ 4-swaps
               6, 5, 8, 7, 2, 1, 4, 3;
               7, 8, 5, 6, 3, 4, 1, 2;
               8, 7, 6, 5, 4, 3, 2, 1]);  \\ rev
  check_equal(apply(test_permorder,S2xS2xS2),
              [1, 2, 2, 2, 2, 2, 2, 2]);
  check_equal(test_perms_Cayley_table(S2wrS2),
              [1,2, 3,4,  5,6, 7,8;
               2,1, 4,3,  7,8, 5,6;
               3,4, 1,2,  6,5, 8,7;
               4,3, 2,1,  8,7, 6,5;    \\ 4-revs
               5,6, 7,8,  1,2, 3,4;    \\ 4-swaps
               6,5, 8,7,  3,4, 1,2;    \\ order 4
               7,8, 5,6,  2,1, 4,3;    \\ order 4
               8,7, 6,5,  4,3, 2,1]);  \\ rev
  check_equal(apply(test_permorder,S2wrS2),
              [1, 2, 2, 2, 2, 4, 4, 2]);

  my(gens=[Vecsmall([2,1,  4,3,  7,8, 5,6]),
           Vecsmall([3,4,  1,2,  6,5, 8,7]),
           Vecsmall([5,6,  7,8,  1,2, 3,4])],
     perms = test_perms_generators_to_group(gens));
  check_equal(test_perms_is_isomorphic(perms,S2wrS2), 1);
}

S2wrS2_commutator = test_perms_commutator_subgroup(S2wrS2);
check_equal(#S2wrS2, 8);
check_equal(#S2wrS2_commutator, 2, "S2wrS2 commutator");  \\ C2 in 4 elems
check_equal(test_perms_is_2group(S2wrS2_commutator), 1);
check_equal(test_perms_is_nilpotent(S2wrS2_commutator), 1);
check_equal(test_perms_nilpotency_class(S2wrS2_commutator), 2);

S3wrS2_commutator = test_perms_commutator_subgroup(S3wrS2);
check_equal(#S3wrS2, 72);
check_equal(#S3wrS2_commutator, 18, "S3wrS2 commutator");
check_equal(72/18, 4);
check_equal(test_perms_is_nilpotent(S3wrS2_commutator), 0, "S3wrS2 commutator");

S2wrS2_x_S3 = test_perms_direct_product(S2wrS2, S3);
check_equal(#S2wrS2_x_S3, 48);  \\ 6*8
check_equal(test_perms_is_supersolvable(S2wrS2_x_S3), 1, "S2wrS2_x_S3");
S2wrS2_x_S3_wr_S2 = test_perms_wreath_product(S2wrS2_x_S3, S2);
check_equal(#S2wrS2_x_S3_wr_S2, 48^2 * 2);  \\ 4608
check_equal(test_perms_is_supersolvable(S2wrS2_x_S3_wr_S2), 0, \
            "S2wrS2_x_S3_wr_S2");

S2xS2_wrS2 = test_perms_wreath_product(S2xS2, S2);
S2xS2_wrS3 = test_perms_wreath_product(S2xS2, S3);
check_equal(#S2xS2_wrS2[1], 4*2);
check_equal(#S2xS2_wrS2,    4^2 * 2!);
check_equal(#S2xS2_wrS3[1], 4*3,      "S2xS2_wrS3 perm len");
check_equal(#S2xS2_wrS3,    4^3 * 3!, "S2xS2_wrS3 num perms");
check_equal(test_perms_is_2group(S2xS2_wrS2), 1);
check_equal(test_perms_is_2group(S2xS2_wrS3), 0);
check_equal(test_perms_is_nilpotent(S2xS2_wrS2), 1);
check_equal(test_perms_is_nilpotent(S2xS2_wrS3), 0);
check_equal(test_perms_nilpotency_class(S2xS2_wrS2), 3);
check_equal(test_perms_is_supersolvable(S2xS2_wrS2), 1);
check_equal(test_perms_is_supersolvable(S2xS2_wrS3), 0);

S2wrS2wrS2 = test_perms_wreath_product(S2wrS2, S2);
S2wrS2wrS3 = test_perms_wreath_product(S2wrS2, S3);
check_equal(#S2wrS2wrS2,  8^2 * 2!, "S2wrS2wrS2 num perms");
check_equal(128,          8^2 * 2!);
check_equal(#S2wrS2wrS3,  8^3 * 3!, "S2wrS2wrS3 num perms");
check_equal(3072,         8^3 * 3!);
check_equal(test_perms_is_2group(S2wrS2wrS2), 1);
check_equal(test_perms_is_2group(S2wrS2wrS3), 0);
check_equal(test_perms_is_nilpotent(S2wrS2wrS2), 1);
check_equal(test_perms_is_nilpotent(S2wrS2wrS3), 0, "S2wrS2wrS3");
check_equal(test_perms_nilpotency_class(S2wrS2wrS2), 5, "S2wrS2wrS2 nilpotency class");
check_equal(test_perms_is_supersolvable(S2wrS2wrS2), 1);
check_equal(test_perms_is_supersolvable(S2wrS2wrS3), 0, "S2wrS2wrS3");

\\ bit big, bit slow
\\ S2wrS2wrS2wrS2 = test_perms_wreath_product(S2wrS2wrS2, S2);
\\ check_equal(#S2wrS2wrS2wrS2, 128^2 * 2!, "S2wrS2wrS2 num perms");
\\ check_equal(32768,           128^2 * 2!);
\\ check_equal(test_perms_is_Abelian(S2wrS2wrS2wrS2),   0);
\\ check_equal(test_perms_is_nilpotent(S2wrS2wrS2wrS2), 1);
\\ check_equal(test_perms_nilpotency_class(S2wrS2wrS2wrS2), 9, "S2wrS2wrS2wsS2 nilpotency class");
\\ \\ check_equal(test_perms_is_solvable(S2wrS2wrS2wrS2),  1);

S2wrC3 = test_perms_wreath_product(S2,C3);
C3wrS2 = test_perms_wreath_product(C3,S2);
C3wrC3 = test_perms_wreath_product(C3,C3);
check_equal(#S2wrC3[1], 2*3,     "S2wrC3 perm len");
check_equal(#S2wrC3,    2^3 * 3, "S2wrC3 num perms");
check_equal(#C3wrC3[1], 3*3,     "C3wrC3 perm len");
check_equal(#C3wrC3,    3^3 * 3, "C3wrC3 num perms");
check_equal(81,         3^3 * 3);
check_equal(test_perms_is_supersolvable(C3wrS2), 1, "C3wrS2");
check_equal(test_perms_is_supersolvable(S2wrC3), 1, "S2wrC3");
check_equal(test_perms_is_supersolvable(C3wrC3), 1, "C3wrC3");


\\------------------------------------------------------------------------------
progress("test_perms_minimum_num_generators()");

{
  check_equal(test_perms_minimum_generators([ Vecsmall([1,2,3]) ]), [],
             "test_perms_minimum_generators() identity 3");
}
{
  check_equal(test_perms_minimum_generators([Vecsmall([1,2,3]),Vecsmall([2,1,3])]),
              [ Vecsmall([2,1,3]) ],
              "test_perms_minimum_generators() C2 in 3 elems");
}
{
  my(perms=[Vecsmall([1, 2, 3]), Vecsmall([1, 3, 2]),
          Vecsmall([2, 1, 3]), Vecsmall([2, 3, 1]),
          Vecsmall([3, 1, 2]), Vecsmall([3, 2, 1])]);
  check_equal(test_perms_is_group(perms), 1, "S3 group");
  check_equal(test_perms_minimum_num_generators(perms),
              2,
              "test_perms_minimum_num_generators() S3");
}
{
  my(vec=vector(4!,i, Vecsmall(numtoperm(4,i))));
  check_equal(test_perms_is_group(vec), 1, "S4 group");
  check_equal(test_perms_minimum_num_generators
              (vec),
              2,
              "test_perms_minimum_num_generators() S4");
}

{
  \\ C2 x C2
  my(gens=[ Vecsmall([2,1, 3,4]),
            Vecsmall([1,2, 4,3]) ],
     perms = test_perms_generators_to_group(gens));
  check_equal(#perms, 4);
  check_equal(#Set(perms), 4);

  my(got_gens=test_perms_minimum_generators(perms),
     got_perms=test_perms_generators_to_group(got_gens));
  check_equal(vecsort(got_perms), perms);

  check_equal(#got_gens, 2, "test_perms_minimum_num_generators() C2 x C2");
}
{
  \\ C2 x C3
  my(gens=[ Vecsmall([2,1, 3,4,5]),
           Vecsmall([1,2, 4,3,5]), Vecsmall([1,2, 5,3,4]) ],
     perms = test_perms_generators_to_group(gens));
  check_equal(#perms, 12);
  check_equal(#Set(perms), 12);

  my(got_gens=test_perms_minimum_generators(perms),
     got_perms=test_perms_generators_to_group(got_gens));
  check_equal(got_perms, perms);

  check_equal(#got_gens, 2, "test_perms_minimum_num_generators() C2 x C3");
}
{
  \\ C3 x C3
  my(gens=[ Vecsmall([2,1,3, 4,5,6]), Vecsmall([2,3,1, 4,5,6]),
           Vecsmall([1,2,3, 5,4,6]), Vecsmall([1,2,3, 5,6,4]) ],
     perms = test_perms_generators_to_group(gens));
  check_equal(#perms, 36);
  check_equal(#Set(perms), 36);

  my(got_gens=test_perms_minimum_generators(perms),
     got_perms=test_perms_generators_to_group(got_gens));
  check_equal(vecsort(got_perms), vecsort(perms));

  check_equal(#got_gens, 2, "test_perms_minimum_num_generators() C3 x C3");
}
{
  \\ S(n) requires 2 generators for n>=3
  for(n=3,6,
     check_equal(test_perms_minimum_num_generators
                 (vector(n!,i, Vecsmall(numtoperm(n,i)))),
                 2,
                 Str("test_perms_minimum_num_generators() S"n)));
}


\\------------------------------------------------------------------------------
progress("vpar_num_automorphism_generators()");

{
  for(forest=0,1,
     for(n=1,14-forest,
        my(vpar=vpar_unlabelled_first(n),
           m=0,
           reps=0);
        until((vpar=vpar_unlabelled_next(vpar,forest))=='none,
              my(got=vpar_num_automorphism_generators(vpar));
              if(m==got, reps++,
                 m<got, m=got;reps=1));
        check_equal(m, test_num_automorphism_generators_max(n,forest),
                    Str("test_num_automorphism_generators_max() "
                        "vs search n="n" forest="forest))));
}


\\------------------------------------------------------------------------------
progress("no overflow vecsmall on 2^16 star repeated \"none\" in calculations");

{
  \\ 2^16*4 = 256k memory
  my(n=2^16,
     vpar=vectorsmall(n,v, v!=1));  \\ vpar_make_star but in Vecsmall
  vpar_validate(vpar);

  check_equal(vpar_perfect_domnum(vpar), 1,
             "vpar_perfect_domnum() no overflow");
  check_equal(vpar_perfect_domnum_and_count(vpar), [1,1],
             "vpar_perfect_domnum_and_count() no overflow");

  check_equal(vpar_disjoint_domnum(vpar), n,
              "vpar_disjoint_domnum() no overflow");
  check_equal(vpar_disjoint_domnum_and_count(vpar), [n,1],
              "vpar_disjoint_domnum_and_count() no overflow");

  check_equal(vpar_totdomnum(vpar), 2,
             "vpar_totdomnum() no overflow");

  check_equal(vpar_totdomnum_and_count(vpar), [2, n-1],
             "vpar_totdomnum_and_count() no overflow");

  check_equal(vpar_indomnum(vpar), 1,
             "vpar_indomnum() no overflow");

}


\\------------------------------------------------------------------------------
progress("Automorphisms by Dreadnaut");

print("Automorphisms by Dreadnaut");

\\ cycles is a vector of vectors like [ [1,4,5], [2,6] ] which specify a
\\ permutation of n things as cycles 1->4, 4->5, etc.
\\ cycles is similar to the Pari library "cyc" representation, but here
\\ don't need all 1..n appearing.  Any omitted are taken as fixed i->i.
\\ Return a Vecsmall perm of these cycles.
\\
test_cycles_to_perm(cycles,n) =
{
  my(perm=vpar_INTERNAL_identity_perm(n));
  for(c=1,#cycles,
     for(i=1,#cycles[c],
        perm[cycles[c][i]] = cycles[c][(i%#cycles[c])+1]));
  perm;
}
check_equal(test_cycles_to_perm([ [1,2] ], 2), Vecsmall([2,1]));
check_equal(test_cycles_to_perm([ [1,2], [3,4] ], 4), Vecsmall([2,1, 4,3]));
check_equal(test_cycles_to_perm([ [3,4] ], 4), Vecsmall([1,2, 4,3]));

keys_to_partition(vec) =
{
  my(distinct=vecsort(vec,,8));
  vector(#distinct,i, Vec(select(k->k==distinct[i], vec, 1)));
}
check_equal(keys_to_partition([1,4,3,3]), [ [1], [3,4], [2] ]);

dreadnaut_partition_keys(vec) =
{
  vec=keys_to_partition(vec);
  my(l=List(["f=["]),
     sep="");
  for(i=1,#vec,
     if(#vec[i],
        for(j=1,#vec[i],
           listput(l,Str(sep,vec[i][j]));
           sep=","));
     sep="|");
  listput(l,"]");
  concat(l);
}
check_equal(dreadnaut_partition_keys([1,4,3,3]), "f=[1|3,4|2]");

\\ flags=0  undirected
\\ flags=1  directed by "d"
\\ flags=2  directed by partitioning roots
\\
run_dreadnaut_automorphisms(vpar,flags=0) =
{
  system("rm -f test-slow-in.tmp test-slow-out.tmp");

  \\ "As" sparse seems a bit quicker, sometimes
  write("test-slow-in.tmp", "As");
  if(flags==1,  \\ directed
     write("test-slow-in.tmp", "d"));

  write("test-slow-in.tmp", test_to_dreadnaut(vpar));

  if(flags==2,  \\ roots partition
     write("test-slow-in.tmp", test_dreadnaut_roots(vpar)));

  \\ attempt speedup by initial partition into depths or eccentricities
  \\ write("test-slow-in.tmp",
  \\          dreadnaut_partition_keys(if(flags,
  \\                                      vpar_depths_vector(vpar),
  \\                                      vpar_eccentricities_vector(vpar))));

  write("test-slow-in.tmp", "x");  \\ execute

  system("dreadnaut <test-slow-in.tmp | perl test-dreadnaut-x-automorphisms.pl >test-slow-out.tmp");
  dreadnaut_generators                 = 'not_set;
  dreadnaut_num_orbits                 = 'not_set;
  dreadnaut_num_automorphisms          = 'not_set;
  dreadnaut_num_automorphisms_is_exact = 'not_set;
  read("test-slow-out.tmp");

  \\ print("cycles ",dreadnaut_generator_cycles);

  dreadnaut_generators = apply(cycles->test_cycles_to_perm(cycles,#vpar),
                              dreadnaut_generator_cycles);
  \\ print("perms  ",dreadnaut_generators);

  system("rm -f test-slow-in.tmp test-slow-out.tmp");
}

test_dreadnaut_num_automorphisms(got_num_automorphisms) =
{
  \\ dreadnaut_num_automorphisms vs got_num_automorphisms.
  \\ Exact if dreadnaut gave exact, or within some close enough range if
  \\ not.  Nauty MULTIPLY() goes to its exponent scheme above 1e10 which
  \\ writegroupsize() prints as inexact.

  if(dreadnaut_num_automorphisms_is_exact,
     check_equal(got_num_automorphisms,
                 dreadnaut_num_automorphisms,
                 "vpar_num_free_automorphisms() vs dreadnaut");
     ,
     check_equal(1/3*dreadnaut_num_automorphisms <= got_num_automorphisms
                 &&
                 got_num_automorphisms <= 3*dreadnaut_num_automorphisms,
                 ,
                 "vpar_num_free_automorphisms() vs dreadnaut"));
}

test_dreadnaut_automorphisms(vpar) =
{
  my(progress=0,
     limit_all = 10000);

  if(option_verbose,
     print("dreadnaut ",vpar));
  \\--------
  \\ free automorphisms

  if(progress, print1("n="#vpar," run free  ",Strchr(13)));
  run_dreadnaut_automorphisms(vpar,0);

  if(progress, print1("n="#vpar," check     ",Strchr(13)));
  test_dreadnaut_num_automorphisms(vpar_num_free_automorphisms(vpar));

  if(dreadnaut_num_automorphisms <= limit_all,
     my(got_generators=vpar_free_automorphism_generators(vpar));
     check_equal(test_perms_generators_to_group(got_generators),
                 test_perms_generators_to_group(dreadnaut_generators),
                 "vpar_free_automorphism_generators() vs dreadnaut"));

  \\--------
  \\ rooted automorphisms, by directed or by partition
  my(got_num_automorphisms=vpar_num_automorphisms(vpar),
     got_generators=vpar_automorphism_generators(vpar),
     got_all=if(dreadnaut_num_automorphisms <= limit_all,
                test_perms_generators_to_group(got_generators)));
  for(flags=1,2,
     \\ "d" directed is a bit slow
     if(flags==1 && #vpar>25, next());

     if(progress,
        print1("n="#vpar," run flags="flags"   ",Strchr(13)));
     run_dreadnaut_automorphisms(vpar,flags);

     if(progress, print1("n="#vpar," check     ",Strchr(13)));
     test_dreadnaut_num_automorphisms(got_num_automorphisms);

     if(dreadnaut_num_automorphisms <= limit_all,
        check_equal(got_all,
                    test_perms_generators_to_group(dreadnaut_generators),
                    "vpar_automorphism_generators() vs dreadnaut")));

  if(progress, print1("                ",Strchr(13)));
}


{
  \\ Nauty seems a touch slow sometimes for n >= 30.
  for(rep=1,20,
     my(n=random(50)+1,
        vpar=vpar_make_random(n,1));
     \\ print(vpar);
     test_dreadnaut_automorphisms(vpar));
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     #vpar>=1 || next();  \\ no empties in dreadnaut
     \\ print(vpar);
     test_dreadnaut_automorphisms(vpar));
}


\\------------------------------------------------------------------------------
progress("Groups");

\\
\\ PENDING vpar_automorphism_group()
\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i],
\\         group=vpar_automorphism_group(vpar));
\\      print(vpar"  "group);
\\      test_smallgroup_validate(group);
\\
\\      \\ Abelian when all generators order <= 2, not Abelian when any >= 3.
\\      \\ Cf group_isabelian() checks that the generators commute, pairwise.
\\      my(orders=group[2],
\\         max_order=if(#orders,vecmax(orders),0));
\\      check_equal(group_isabelian(group),
\\                  max_order <= 2,
\\                  "vpar_automorphism_group() vs group_isabelian()");
\\
\\      my(num_automorphisms=vpar_num_automorphisms(vpar));
\\      check_equal(group_order(group), num_automorphisms));
\\      if(num_automorphisms <= test_limit_sets,
\\         check_equal(#group_elts(group,#vpar), num_automorphisms));
\\
\\      );
\\ }


\\------------------------------------------------------------------------------
progress("Stable Equimatchable - No Degree <= 3 after N=10");

{
  for(n=11,15,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           vpar_is_degrees_le(vpar,3) || next;
           check_equal(vpar_is_stable_equimatchable(vpar), 0,
                       "vpar_is_stable_equimatchable() none n>10 and degrees<=3")));
}


\\------------------------------------------------------------------------------
progress("test_to_dreadnaut()");

run_dreadnaut(str) =
{
  system("rm -f test-slow-in.tmp test-slow-out.tmp");
  write("test-slow-in.tmp", "d");  \\ digraph mode
  write("test-slow-in.tmp", str);
  write("test-slow-in.tmp", "t");  \\ print directed neighbours

  system("dreadnaut <test-slow-in.tmp | perl test-dreadnaut-t-vector.pl >test-slow-out.tmp");
  dreadnaut_neighbours = 'not_set;
  read("test-slow-out.tmp");

  system("rm -f test-slow-in.tmp test-slow-out.tmp");
}

test_dreadnaut(vpar) =
{
  my(str=test_to_dreadnaut(vpar));
  run_dreadnaut(str);
  check_equal(dreadnaut_neighbours,
              vector(#vpar,v, if(vpar[v], [vpar[v]], [])));

  run_dreadnaut(test_to_dreadnaut(vpar,1));
  check_equal(dreadnaut_neighbours,
              vpar_children_vector(vpar),
              "test_to_dreadnaut() dreadnaut_neighbours");
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     #vpar>=1 || next();  \\ no empties in dreadnaut
     \\ print(vpar);
     test_dreadnaut(vpar));
}

{
  for(rep=1,100,
     my(n=random(100)+1,
        vpar=vpar_make_random(n,1));
     test_dreadnaut(vpar));
}
{
  my(forest=1);
  for(n=1, test_limit_trees_search - forest,
     print("test_dreadnaut() n="n);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests
           my(vpar=vpar_from_upascend(vec));
           test_dreadnaut(vpar)));
}


\\------------------------------------------------------------------------------
progress("Determined by Maximal Trees");

\\ Harary and Palmer, "The Reconstruction of a Tree From Its Maximal
\\ Subtrees", Canadian J. Math., volume 18, 1966, pages 803-810.

\\ Return a vector of maximal proper subtrees of vpar, so with 1 vertex
\\ deleted, up to isomorphism, without repeats.
maximal_proper_subtrees(vpar) =
{
  my(leaves=vpar_leaves(vpar),
     l=List([]));
  for(i=1,#leaves,
     my(sub=vpar_delete_vertex(vpar,leaves[i]));
     sub=vpar_to_free(sub);
     my(pos=setsearch(l,sub,1));
     if(pos, listinsert(l,sub,pos)));
  vecsort(Vec(l));
}

\\ FIXME: Not right.
\\ {
\\   for(n=3,14,
\\      my(l=List([]));
\\      my(vpar=vpar_free_first(n));
\\      until((vpar=vpar_free_next(vpar))=='none,
\\            my(subtrees=maximal_proper_subtrees(vpar));
\\            print(vpar"  "subtrees);
\\            check_equal(setsearch(l,subtrees), 0,
\\                        "maximal_proper_subtrees() unique");
\\            listinsert(l,subtrees, setsearch(l,subtrees,1))));
\\ }


\\-----------------------------------------------------------------------------
progress("vpar_is_free_automorphism()");

test_is_free_automorphism_by_rooted(vpar,perm) =
{
  \\ perm[] extended to the new roots of vpar_INTERNAL_for_free_automorphisms()
  \\ must permute the roots according to how perm[] maps their children.

  \\ print("test_is_free_automorphism_by_rooted() "vpar" perm "perm);

  my(orig_n=#vpar);
  vpar=vpar_INTERNAL_for_free_automorphisms(vpar);
  perm=Vecsmall(perm,#vpar);
  \\ print(" extended vpar "vpar);

  my(roots=vpar_roots(vpar));
  for(i=1,#roots,
     my(r=roots[i]);
     if(r<=orig_n, next());

     my(children=vpar_children_of_vertex(vpar,r));
     \\ print(" r="r" children="children);
     #children==1 || #children==2 || error("root r="r" children "children);
     my(new_children=vecextract(perm,children),
        new_root=if(#new_children==0,r,
                    vpar[new_children[1]]));
     if(#new_children==2,
        \\ print(" new_children "new_children);
        new_root==vpar[new_children[2]] || return(0));
     vpar[new_root]==0 || return(0);

     perm[r] = new_root);

  \\ print(" mangled to vpar="vpar"  perm="perm);
  test_is_perm(perm) || error();
  vpar_validate(vpar);

  vpar_is_automorphism(vpar,perm);
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     #vpar <= test_limit_perms_search || next();

     for(i=1,(#vpar)!,
        my(perm=numtoperm(#vpar,i),
           got=vpar_is_free_automorphism(vpar,perm),
           by_test=test_is_free_automorphism_by_rooted(vpar,perm));
        if(got!=by_test,
           print(vpar"  perm="perm);
           print("mangled "vpar_INTERNAL_for_free_automorphisms(vpar));
          );
        check_equal(got, by_test,
                    "vpar_is_free_automorphism() vs test_is_free_automorphism_by_rooted()");
        );
  );
}


\\-----------------------------------------------------------------------------
progress("vpar_semitotdomnum_and_count()");

{
  my(forest=1);
  for(n=0, test_limit_trees_search - forest,    \\ or more for more testing
     print("vpar_semitotdomnum_and_count() n="n);
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests
           my(vpar=vpar_from_upascend(vec));
           check_equal(vpar_semitotdomnum_and_count(vpar),
                       test_semitotdomnum_and_count_by_flags(vpar),
                       Str("vpar_semitotdomnum_and_count() "
                           "vs test_semitotdomnum_and_count_by_flags()"))));
}


\\------------------------------------------------------------------------------
progress("vpar_free_next()");

test_free_next_by_skip(vpar) =
{
  while((vpar=vpar_unlabelled_next(vpar))!='none,
        my(depths=vpar_depths_vector(vpar));
        if(vpar_preorder_depths_is_free_primary(depths),
           break()));
  vpar;
}

{
  for(n=0,16,
     print("Free Next n="n);
     my(vpar=vpar_free_first(n),
        count=0);
     until(vpar=='none,
           \\ print(vpar);
           count++;
           vpar_validate(vpar);
           check_equal(vpar_is_preorder(vpar), 1,
                       "vpar_free_next() is preorder");
           my(depths=vpar_depths_vector(vpar));
           check_equal(vpar_preorder_depths_is_free_primary(depths), 1,
             "vpar_free_next() vs vpar_preorder_depths_is_free_primary()");

           my(next_vpar=vpar_free_next(vpar),
              want_vpar=test_free_next_by_skip(vpar));
           check_equal(next_vpar,want_vpar,
                 Str("vpar_free_next() vs test_free_next_by_skip()"
                     " n="n" count="count));

           vpar=next_vpar);

     check_equal(count,
                 vpar_count_free(n),
                 Str("vpar_free_next() vs vpar_count_free() n="n)));
}


\\------------------------------------------------------------------------------
progress("Almost Equimatchable");

check_almost_equimatchable(vpar) =
{
  my(got=vpar_is_almost_equimatchable(vpar));
  check_equal(got,
              test_is_almost_equimatchable_by_matchpoly(vpar),
              "vpar_is_almost_equimatchable() vs test_is_almost_equimatchable_by_matchpoly()");

  check_equal(got && vpar_is_equimatchable(vpar), 0,
              "vpar_is_almost_equimatchable() never also vpar_is_equimatchable()");
}

{
  print("Almost Equimatchable - all");
  for(n=1,14,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           check_almost_equimatchable(vpar_relabel_random(vpar))));
}
{
  print("Almost Equimatchable - random");
  for(n=5,40,
     for(reps=1,n,
        my(vpar=vpar_make_random(n));
        check_almost_equimatchable(vpar)));
}


\\------------------------------------------------------------------------------
progress("Stable Equimatchable");

check_stable_equimatchable(vpar) =
{
  my(got=vpar_is_stable_equimatchable(vpar));
  check_equal(got,
              test_is_edge_stable_equimatchable_by_cut(vpar),
              Str("vpar_is_stable_equimatchable()"
                  " vs test_is_edge_stable_equimatchable_by_cut()"));

  check_equal(got,
              test_is_vertex_stable_equimatchable_by_removals(vpar),
              Str("vpar_is_stable_equimatchable()"
                  " vs test_is_vertex_stable_equimatchable_by_removals()"));

  check_equal(got <= vpar_is_equimatchable(vpar), 1,
              "vpar_is_stable_equimatchable() <= vpar_is_equimatchable()");
}

{
  print("Stable Equimatchable - all");
  for(n=1,14,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           check_stable_equimatchable(vpar_relabel_random(vpar))));
}
{
  print("Stable Equimatchable - random");
  for(n=5,40,
     for(reps=1,n,
        my(vpar=vpar_make_random(n));
        check_stable_equimatchable(vpar)));
}


\\-----------------------------------------------------------------------------
progress("perm_commute()");

{
  \\ reverse permutation does not commute, in general
  my(perm=Vecsmall([1, 3, 2]),
     rev=test_perm_reversal(3));
  check_equal(rev*perm, Vecsmall([3, 1, 2]));
  check_equal(perm*rev, Vecsmall([2, 3, 1]));
  check_equal(perm*rev != rev*perm, 1);
  check_equal(perm_commute(perm,rev), 0);
}


\\-----------------------------------------------------------------------------
progress("vpar_minimal_domsets()");

{
  for(n=1,8,
     my(vpar=vpar_preorder_first(n));
     until((vpar=vpar_preorder_next(vpar))=='none,
           my(got=vecsort(vpar_minimal_domsets(vpar)),
              want=vecsort(test_minimal_domsets_by_flags(vpar)));
           if(got!=want,print(vpar));
           check_equal(got,want,
           "vpar_minimal_domsets() vs test_minimal_domsets_by_flags()")));
}
{
  for(n=1,16,
     for(reps=1,min(n,3),
        my(vpar=vpar_make_random(n),
           want_count=vpar_indomsets_count(vpar));
        if(want_count > test_limit_sets, next());

        my(got=vecsort(vpar_minimal_domsets(vpar)),
           want=vecsort(test_minimal_domsets_by_flags(vpar)));
        if(got!=want,print(vpar));
        check_equal(got,want,
                    Str("vpar_minimal_domsets() "
                        "vs test_minimal_domsets_by_flags() n="n))));
}


\\-----------------------------------------------------------------------------
progress("vpar_is_leastchildmonk()");

{
  my(forest=1);
  for(n=0, test_limit_trees_search - forest,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,forest), \\ forests
           my(vpar=vpar_from_upascend(vec));
           check_equal(vpar_is_leastchildmonk(vpar),
                       test_is_leastchildmonk_by_min_child(vpar),
                       Str("vpar_is_leastchildmonk() "
                           "vs test_is_leastchildmonk_by_min_child()"))));
}


\\-----------------------------------------------------------------------------
progress("vpar_count()");

{
  my(forest=1);
  for(n=0, test_limit_trees_search - forest,
     my(count_trees=0,
        count_forests=0);
        forvec(vec=vpar_forvec_from(n,forest),  \\ forests
              my(vpar=vpar_from_upascend(vec));
              count_forests++;
              count_trees += vpar_is_tree(vpar));
     check_equal(vpar_count(n), count_trees,
                 "vpar_count() vs vpar_forvec_from() trees");
     check_equal(vpar_count(n,1), count_forests,
                 "vpar_count() vs vpar_forvec_from() forests");
     );
}

\\-----------------------------------------------------------------------------
