\\ Copyright 2018, 2019, 2020, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <Bulgarian-solitaire.gp
\\
\\ This is some code making and examining "Bulgarian solitaire" trees.
\\
\\ For given n, each tree vertex represents a partition of n, meaning a set
\\ of integers summing to n, as per partitions() or forpart().  There are
\\ numbpart(n) many vertices.
\\
\\ Bulgarian solitaire step on a partition is subtract 1 from each term,
\\ sum those 1s as a new term, delete any 0s.  For example,
\\
\\     [1, 2, 5, 6]  -->  [1, 4, 4, 5]
\\
\\ The vpar is constructed with parent as next vertex by this step.
\\ For triangular numbers n = triangular(t), the partition [1,2,3,..,t]
\\ steps to itself (a fixed point) and is reckoned the root.
\\
\\ For non-triangular n, there are some cycles in the steps.  They become
\\ cycles in the vpar built.  Only some vpar functions can act on a cyclic
\\ vpar, but creating it is fine.
\\
\\ Other Ways
\\ ----------
\\
\\ make_Bulgarian_solitaire() below uses partitions() for the full set of
\\ partitions, and numbers vertices accordingly.  The next partition is
\\ looked-up in these partitions to find the parent vertex number.  This
\\ holds the full list of partitions in memory.  An alternative would be to
\\ loop over partitions with forpart() and get the parent vertex number by
\\ an "unranking" of partition -> index.  Such an unranking is more code,
\\ but only moderately complicated (and can be helped by some tables of
\\ counts).  partitions() and forpart() order first by partition length then
\\ lex() among equal lengths.  Could instead go purely lex(), if don't mind
\\ needing an "unrank" to enumerate.
\\
\\ For triangular n, making the "fixed point" the root vertex suits vpar
\\ functions not accepting cycles, which is most functions.  Having a
\\ self-loop at the fixed point is another possibility.  It is attractive
\\ for some of the Garden of Eden counts.  Their general-case formulas are
\\ for n=1 as a single vertex self-loop and hence not a Garden of Eden.
\\

\\ p is a partition in the form of a Vecsmall like partitions() gives.
\\ Return the partition which is next in Bulgarian solitaire.
\\
Bulgarian_solitaire_step(p) =
{
  for(i=1,#p, p[i]--);
  p=Vec(p);
  Vecsmall(vecsort(select(x->x!=0, concat(p,[#p]))));
}

\\ vpar = make_Bulgarian_solitaire(n)
\\ Return a vpar where each vertex v represents a partition of n,
\\ and vpar[v] is its Bulgarian_solitaire_step() next.
\\ Vertex numbers are the order partitions() gives partitions.
\\ partitions() gives a Set(), ie. sorted by cmp(), so partitions first by
\\ length and among equal length by lexicographic order.
\\ Fixed-point partitions are roots.
\\ If n is not a triangular number then there will be cycles in vpar, which
\\ only a few vpar functions will act on.
\\
make_Bulgarian_solitaire(n) =
{
  my(parts=partitions(n));
  setisset(parts) || error("oops");
  vector(#parts,v,
         my(parent=Bulgarian_solitaire_step(parts[v]));
         if(parent==parts[v], 0,        \\ self-step, is root
            setsearch(parts,parent)));  \\ else lookup
}

\\ p is a partition (a Vecsmall), return its vertex number.
\\ This could be some unranking code, but a naive search is enough.
part_to_v(p) = \
  my(v=0); forpart(q=vecsum(Vec(p)), v++; if(q==p,return(v))); error();


\\-----------------------------------------------------------------------------
print("Triangular n is trees:");

triangular(t) = t*(t+1)/2;
vpars=vector(7,t, make_Bulgarian_solitaire(triangular(t)));

\\ Bulgarian solitaire is a tree when n=triangular(t).
{
  for(t=1,#vpars,
     !vpar_is_cyclic(vpars[t]) || error();
     vpar_num_roots(vpars[t]) || error();
     #vpars[t] == numbpart(triangular(t)) || error();
  );
}
print("tree sizes    ",vector(#vpars,t, #vpars[t]), \
      "    (A066655)");

\\ vpar_height() is the pronic numbers (A002378) t*(t-1).
\\ This is shown by Igusa,
\\
\\     Kiyoshi Igusa, "Solution of the Bulgarian Solitaire Conjecture",
\\     Mathematics Magazine, volume 58, number 5, November 1985, pages
\\     259-271.  http://www.jstor.org/stable/2690174
\\
print("tree heights  ",vector(#vpars,t, vpar_height(vpars[t])), \
      "         (A002378)");
{
  for(t=2,#vpars,
     my(vpar = vpars[t],
        height = vpar_height(vpar));
     height == t*(t-1) || error();
     height == 2*triangular(t-1) || error();
  );
}

\\ One partition attaining the height is [1, 1,2,3,...,t-2,t-1, t-1],
\\ for t >= 2.  Bentz in
\\
\\     H. J. Bentz, "Proof of the Bulgarian Solitaire Conjectures", Ars
\\     Combin. volume 23, 1987, pages 151-170.
\\
\\ shows how this partition makes "conjugate" pairs going down and up.
\\ This is a kind of "trunk" in the tree, distinguishing this vertex from
\\ among the several, in general, which attain the height
\\ (vpar_height_vertices(), vpar_height_num_vertices()).
{
  for(t=2,#vpars,
     my(vpar = vpars[t],
        height = vpar_height(vpar),
        p = Vecsmall(concat([1, [1..t-1], t-1])),
        v = part_to_v(p));
     vpar_depth_of_vertex(vpar,v) == height || error();
  );
}
print("num vertices at height ", \
      vector(#vpars,t, vpar_height_num_vertices(vpars[t])));


\\-----------------------------------------------------------------------------
print();
print("All n has cyclics:");
vpars=vector(20,n, make_Bulgarian_solitaire(n));

\\ Akin and Davis show that for n == triangular(t-1) + r with 1 <= r <= t,
\\ the number of disconnected components in the digraph of n is the number
\\ of compositions of t into r parts (OEIS A037306).
\\
\\     Ethan Akin and Morton Davis, "Bulgarian Solitaire", American
\\     Mathematical Monthly, volume 92, number 4, April 1985, pages 237-250.
\\     http://www.jstor.org/stable/2323643
\\
\\ (And Brandt a formula or more previously too?)
{
  print("num components, is compositions t,r up to cyclic shifts");
  print1("  ");
  for(n=1,#vpars,
     print1(vpar_num_components(vpars[n]),","));
  print("   (A037306)");
}
\\ num_compositions_up_to_rotation(t,r) returns the number of compositions
\\ (partitions with order) of t which have r many parts and which are
\\ distinct up to rotating the terms.  For example [1,3,2,1] and [3,2,1,1]
\\ are the same by rotation.
\\ Formula as given by Paul Weisenhorn in OEIS A037306.
num_compositions_up_to_rotation(t,r) = \
  sumdiv(gcd(t,r),d, eulerphi(d)*binomial(t/d,r/d)) / t;
\\
{
  for(t=1,6,
     for(r=1,t,
        my(n=triangular(t-1) + r);
        if(n<=#vpars,
           vpar_num_components(vpars[n]) == num_compositions_up_to_rotation(t,r)
             || error())));
}

\\ The partitions which are part of cycles are the triangular and the
\\ "almost triangular" meaning a triangle and at most 1 extra diagonal.
\\ This follows from the several authors who show the solitaire goes by
\\ non-increasing number of diagonals in the partitions.
\\
\\ The number of partitions in a cycle is then binomial(t,r) ways to place
\\ the extras in the last diagonal.  This includes the single triangle
\\ partition when n is triangular.  The latter is a root in the vpar built
\\ here, so cyclic vertices + root vertices = binomial(t,r).  Could make a
\\ vpar with the triangular partition as a self-loop.  In that case it would
\\ be cyclic and vpar_num_cyclic_vertices() would count it.
{
  print("cyclic + root vertices = binomial(t,r)");
  print1("  ");
  for(t=1,6,
     for(r=1,t,
        my(n=triangular(t-1) + r);
        if(n<=#vpars,
           my(c = vpar_num_cyclic_vertices(vpars[n])
                  + vpar_num_roots(vpars[n]));
           print1(c,",");
           c == binomial(t,r) || error())));
  print("    (A074909)");
}

\\ The "almost triangular" but not triangular partitions are counted in
\\ A325191 = A074909 - A010054.  It is the vpar_num_cyclic_vertices() of the
\\ triangular as root form here.


\\ "Garden of Eden" partitions are those which have no predecessor in the
\\ solitaire step.  These are childless vertices in the vpar, including in
\\ the cyclic non-triangular vpars.
{
  print("Garden of Eden (childless)");
  print1("  ");
  for(n=1,#vpars,
     print1(vpar_num_childless(vpars[n]),","));
  print("   (A123975)");

  print("with child");
  print1("  ");
  for(n=1,#vpars,
     print1(vpar_num_childful(vpars[n]),","));
  print("   (A260894)");
}
\\
\\ The Garden of Eden partitions are characterized by
\\
\\     Brian Hopkins and James A. Sellers, "Exact Enumeration of Garden of
\\     Eden Partitions", INTEGERS: Electronic Journal of Combinatorial
\\     Number Theory, volume 7, number 2, 2007, #A19.
\\
\\ as those partitions with rank <= -2, where rank is Dyson's definition
\\ LargestTerm - NumTerms.  In GP, partitions have terms increasing so p[#p]
\\ is LargestTerm.
\\
partition_Dyson_rank(p) = if(#p, p[#p]-#p);
\\
{
  for(n=2,#vpars,
     my(vpar=vpars[n],
        parts=partitions(n));
     for(v=1,#vpar,
        (partition_Dyson_rank(parts[v]) <= -2) == vpar_is_childless(vpar,v)
          || error()));
}

\\ Hopkins and Sellers give the following formula counting these partitions,
\\ based on forms possible for rank <= -2.
\\ The j sum is 1 upwards while the numbpart arg n - 3*j*(j+1)/2 >= 0.
\\ The j limit expression is solution of 3/2*j^2 + 3/2*j - n == 0, rounded
\\ down to an integer.
\\ A simple over-estimate for j would be sqrtint(n).  numbpart() returns 0
\\ for negative args so it's ok to have j go too big.
\\
num_Garden_of_Eden(n) =        \\ Hopkins and Sellers
{                      
  if(n==1, 1,
     sum(j=1, (sqrtint(8*n\3 + 1) - 1)\2,
         (-1)^(j+1) * numbpart(n - 3*j*(j+1)/2)));
}
\\
{
  for(n=1,#vpars,
     vpar_num_childless(vpars[n]) == num_Garden_of_Eden(n) || error());
}


\\=============================================================================
\\ Compositions

\\ A similar step rule can be applied to compositions, being partitions with
\\ ordered elements.  Griggs and Ho call this "Carolina solitaire"
\\
\\     Jerrold R. Griggs and Chih-Chang Ho, "The Cycling of Partitions and
\\     Compositions under Repeated Shifts", Advances In Applied Mathematics,
\\     volume 21, 1998, pages 205-227, Am980597.
\\
\\ Compositions of n can be generated from bit runs, and conversely
\\ interpret the terms of a composition as bit runs.  This is a simpler
\\ ranking and unranking than for partitions.


\\ num = integer 1 to 2^(n-1) inclusive
\\ vec = composition summing to n
\\
\\ There are various ways to extract bit run lengths.  The form here uses
\\ num-1 so range 0 to 2^(n-1)-1, adds high bit 2^(n-1) so
\\
\\     high                low
\\      1  00 1 0000 1 000 1 0
\\         |----- num-1 -----|
\\
\\ Start at the high 1-bit, then composition term vec[1] says how many bit
\\ positions to go down for the first 1 within value num-1, vec[2] how many
\\ to go down to the next, etc.  b[] is binary high to low, with high 1
\\ included in num_to_composition() but excluded in composition_to_num().
\\ 
composition_to_num(vec) =
{
  my(b=vector(vecsum(vec)-1), pos=vec[1]);
  for(i=2,#vec, b[pos]=1; pos+=vec[i]);
  fromdigits(b,2) + 1;         \\ fromdigits() is new in gp 2.9
}
num_to_composition(num,n) =
{
  my(b=binary(num + (1<<(n-1)) - 1),
     vec=vector(hammingweight(b)), 
     pos=0);
  for(i=1,#b, vec[pos+=b[i]]++);
  vec;
}

Bulgarian_composition_step(p) =
{
  \\ like Bulgarian_solitaire_step(), but no vecsort()
  p=Vec(p);
  for(i=1,#p, p[i]--);
  select(x->x!=0, concat(p,[#p]));
}
make_Bulgarian_compositions(n) =
{
  vector(2^(n-1),v,
         my(p=composition_to_num
              (Bulgarian_composition_step(num_to_composition(v,n))));
         if(p==v, 0, p));
}

print();
print("Compositions:");

vpars=vector(12,n, make_Bulgarian_compositions(n));

\\ Hopkins and Jones show that the number of Garden of Eden compositions is
\\ 2^(n-1) - fibonacci(n+1) for n>=2.
\\
\\    Brian Hopkins, Michael A. Jones, "Shift-Induced Dynamical Systems on
\\    Partitions and Compositions", The Electronic Journal of Combinatorics,
\\    volume 13, 2006.
{
  print("Garden of Eden (childless), on all n, is 2^(n-1) - fibonacci(n+1)");
  print1("  ");
  for(n=2,#vpars,
     my(num_childless=vpar_num_childless(vpars[n]));
     print1(num_childless,",");
     num_childless == 2^(n-1) - fibonacci(n+1)  || error());
  print("   (A008466)");

  print("with child, fibonacci(n+1)");
  print1("  ");
  for(n=2,#vpars,
     my(num_childful = vpar_num_childful(vpars[n]));
     print1(num_childful,",");
     num_childful == fibonacci(n+1)  || error());
  print("   (A000045)");
}


\\-----------------------------------------------------------------------------
