\\ Copyright 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1)
read("../vpar.gp");


\\ Usage: gp <mean-unlabelled-singletons.gp
\\


\\-----------------------------------------------------------------------------
\\ Unlabelled Forests Total Singletons

\\ Over all rooted unlabelled forests of n vertices, the total number of
\\ singleton vertices equals the cumulative number of such forests.

\\ number of rooted unlabelled forests of n vertices
F(n) = vpar_count_unlabelled(n,1);

\\ total number of singletons in all the F(n) many forests of n vertices
S_by_sum_terms(n) = sum(i=0,n-1, F(i));

\\ vpar_count_unlabelled_vector() calculates the forest counts desired for
\\ this sum.  The vector is n=1 upwards, so F(0)=1 wanted in the sum is
\\ added separately.
S(n) = if(n==0,0, 1 + vecsum(vpar_count_unlabelled_vector(n-1,1)));

for(n=0,20, S(n) == S_by_sum_terms(n) || error());

\\ S as sum of F follows since if one vertex is a singleton then the rest
\\ can be any of the F(n-1) forests of the remaining n-1 vertices, and
\\ similarly two singletons etc,
\\
\\     >=1 singleton      rest F(n-1) forests of n-1 vertices
\\     >=2 singletons     rest F(n-2) forests of n-2 vertices
\\     ...
\\     >=n-1 singletons   rest F(1) forests of 1 vertex
\\      =n singletons     rest F(0) forests of 0 vertices
\\
\\ This can be thought of as ordering the components of the otherwise
\\ unordered forest so as to have the singletons first.  Then F(n-1) is how
\\ many ways the first position can be a singleton, F(n-2) many ways the
\\ second position can be a singleton, and so on.
\\
\\ When the last position n is a singleton, the rest is an empty forest of
\\ no vertices.  That should be reckoned as 1 empty forest so these
\\ singletone are counted.  So F(0)=1, which is what vpar_count_unlabelled()
\\ gives.

{
  print1("Number of forests      ");
  for(n=0,9, print1(F(n), ","));
  print("   (A000081)");
}
{
  print1("Their total singletons ");
  for(n=0,9, print1(S(n), ","));
  print("    (A087803)");
  print();
}

{
  \\ Check S(n) by going through forests and adding up their
  \\ vpar_num_singletons().
  for(n=0,10,
     my(vpar=vpar_unlabelled_first(n),
        got_F=0,
        got_S=0);
     until((vpar=vpar_unlabelled_next(vpar,1)) == 'none,  \\ forests
           got_F ++;
           got_S += vpar_num_singletons(vpar));
     got_F == F(n) || error();
     got_S == S(n) || error());
}


\\-----------------------------------------------------------------------------
\\ Unlabelled Forests Mean Singletons
\\
\\ The asymptotic for number of unlabelled trees is given in OEIS A000081.
\\ (Per Polya is it?  Or Otter as well or instead?)  Forests are similar,
\\ being trees of 1 fewer vertices.
\\
\\     T(n) ~= c * d^n / n^(3/2)        trees
\\     where c = 0.439924...            (A187770)
\\           d = 2.955765...            (A051491)
\\
\\ Cumulative F(n) in A087803 is then (with the constant factor c/(1-1/d)
\\ per the comment by Vladimir Reshetnikov)
\\
\\     S(n) ~= c*d/(d-1) * d^n / n^(3/2)
\\     where c*d/(d-1) = 0.664861...    (in A087803)
\\
\\ The mean is then
\\
\\     M(n) = S(n)/F(n) = S(n)/T(n+1)
\\         -> 1/(d-1) = 0.511308 ...    as n -> infinity
\\
\\ Convergence to this limit is not particularly fast.  For example at
\\ n=256 the 3rd decimal place is still well off.

M_by_ratio(n) = S(n) / F(n);

\\ Like S(n) above, vpar_count_unlabelled_vector() can calculate all terms
\\ together needed for the sum in S and then one further F.
M(n) =
{
  my(vec=vpar_count_unlabelled_vector(n,1),
     f = vec[n],
     s = 1 + if(n-1>0, vecsum(vec[1..n-1])));
  s / f;
}
for(n=1,20, M(n) == M_by_ratio(n) || error());

{
  print("  N   Mean singletons in a forest of n vertices");
  for(k=1,8,
     my(n=2^k);
     printf("%3d  %.8f\n", n, M(n)));
  print();
}

\\ round down x to n decimal places
floor_decimals(x,n) = floor(x*10^n)/10^n;

d = 2.9557652856519949747148175;
c = 0.4399240125710253040409033;
printf("d = %.8f                  (A051491)\n", floor_decimals(d,8));
printf("c = %.8f                  (A187770)\n", floor_decimals(c,8));
printf("c*d/(d-1) = %.8f          (in A087803)\n", floor_decimals(c*d/(d-1),8));

M_limit = 0.51130879934124059897199991;
printf("M limit 1/(d-1) = %.8f\n", floor_decimals(M_limit,8));
print();

abs(M_limit - 1/(d-1)) < 1e-12 || error();


\\-----------------------------------------------------------------------------
\\ Beyer and Hedetniemi's algorithm for iterating unlabelled trees is easily
\\ adapted to unlabelled forests by discarding the root of each tree.  This
\\ is Knuth's section 7.2.1.6 algorithm O and is done by vpar_unlabelled_next() 
\\ of forests.
\\
\\ Beyer and Hedetniemi's canonical form (per Scions) keeps all singletons
\\ as the endmost vertex numbers in the forest, so at positions ... n-2,
\\ n-1, n.  Their algorithm rewrites the depths of the singletons plus the
\\ last non-singleton.
\\
\\ The result is mean M+1 rewrites, being the singletons plus 1 non-singleton.
\\ (Is this per Kubica?)
\\
\\ The following code counts actual rewrites in the algorithm as implemented
\\ by vpar_unlabelled_next().  It is S(n) singletons plus F(n) for 1
\\ non-singleton each forest.  Except the last vpar is all singletons and
\\ there is nothing after to rewrite them to, so less those n and 1
\\ non-singleton there.
\\
\\ If the algorithm grew the last vpar of n vertices to the first vpar of
\\ n+1 vertices then that would be a rewrite of n of them, so getting a
\\ little bit closer to the limit.

{
  print(" N   Rewrites  Forests    Mean");
  for(n=0,8,
     my(vpar=vpar_unlabelled_first(n),
        count=0,
        rewrites=0);
     while(1,
           count++;
           my(new_vpar = vpar_unlabelled_next(vpar,1));  \\ next forest
           if(new_vpar == 'none, break);
           my(seen_diff = 0);
           for(i=1,#vpar,
              if(new_vpar[i] != vpar[i], seen_diff=1);
              rewrites += seen_diff);
           vpar=new_vpar);
     printf("%2d   %5d     %5d     %.6f\n", n, rewrites, count, rewrites/count);

     count    == F(n) || error();
     rewrites == S(n) + F(n) - (n+1) || error());
  print();
}


\\-----------------------------------------------------------------------------
\\ Unchanged Depths 0

\\ "Rewrite" described above of the singletons and last non-singleton may
\\ actually leaves some depth values unchanged.  This happens when the
\\ previous part of the forest copied is a whole component tree so that
\\ depths which had been 0 for the singletons remain 0 for the copies of the
\\ root of that component tree.  Likewise in the vpar what had been
\\ vpar[singleton]=0 stay vpar[new_root]=0.
\\
\\ The algorithm doesn't try to notice this.  Would expect more trouble than
\\ it's worth to notice a few times a 0 is about to be written to what is
\\ already a 0.
\\
\\ One or more 0 -> 0 writes occur when
\\
\\     * the last non-singleton vertex p is at depth 1, so its parent q
\\       (where the copying starts) is depth 0 (a root)
\\     * the q component tree omitting its last vertex (last non-singleton), 
\\       is size <= the number of singletons
\\
\\ For example component q of 4 vertices including p,
\\
\\     depth 0     q       s s s      q component, p child of q
\\                 | \ \               s singletons
\\     depth 1     * *  p
\\                     |--------|
\\                     change to copies of q .. p-1
\\
\\     becomes:    q    q    q
\\                 |\   |\ 
\\                 * *  * *
\\
\\ Excluding p is 3 vertices copied to p and s s s, so q at p and then q
\\ again at the last s.  But this last s was already depth 0 like q is.
\\
\\ The number of forests where this occurs, and the total 0->0 vertices in
\\ them, can be counted.
{
  print1("Depth values unchanged: ");
  my(l=List([]));
  for(n=0,10,
     my(vpar=vpar_unlabelled_first(n),
        depths=vpar_depths_vector(vpar),
        no_change_rewrites=0,
        count=0);
     while(1,
           my(new_vpar = vpar_unlabelled_next(vpar,1)); \\ next forest
           if(new_vpar == 'none, break);
           my(new_depths=vpar_depths_vector(new_vpar),
              pos = n - vpar_num_singletons(vpar));     \\ pos onwards

           \\ no changes is always depth 0 -> 0,
           \\ and likewise in vpar parent 0 -> 0 root
           depths[pos] >= 1 || error();
           depths[pos] != new_depths[pos] || error();
           for(i=pos,n,
              if(depths[i] == new_depths[i],
                 depths[i]   == 0 || error();
                 vpar[i]     == 0 || error();
                 new_vpar[i] == 0 || error()));

           my(this = sum(i=pos,n, depths[i] == new_depths[i]));
           no_change_rewrites += this;

           count += (this!=0);
           (this!=0) ==
           (depths[pos]==1 && pos+vpar_subtree_size(vpar,vpar[pos])-1<=n)
           || error();

           vpar=new_vpar;
           depths=new_depths);
     print1(no_change_rewrites, ",");
     listput(l,count));

  print();
  \\ not in OEIS: 1,2,5,9,21,43,100,229,559,1363,3449,8782,22789,59607,
  \\ (and nothing from the superseeker ...)

  print1("Num such forests:       ");
  for(i=1,#l,
     print1(l[i],","));
  print();
  \\ not in OEIS: 1,1,3,5,12,25,61,138,348,855,2187,5604

  print();
}


\\-----------------------------------------------------------------------------
\\ Knuth presents Beyer and Hedetniemi's algorithm in The Art of Computer
\\ Programming volume 4A section 7.2.1.6 algorithm O,
\\
\\     http://www-cs-faculty.stanford.edu/~knuth/fasc4a.ps.gz
\\     Pre-Fascicle 4A, Draft of Section 7.2.1.6.
\\     Algorithm O page 22, exercise 88 page 40, answer page 66.
\\
\\ Knuth adapts the algorithm to act on a vertex parent vector like vpar
\\ here, and iterates rooted forests.  Exercise 88 is to count "mems" in the
\\ algorithm.  A mem is a read or a write of memory.  Per Knuth's answer,
\\ the mean per forest is
\\
\\     2 + 3/(d-1) = 3.53392...
\\
\\ In terms of mean singletons M, it is
\\
\\     2 + 3/(d-1)  = 3*M + 2
\\
\\ This is M reads to look past the singletons to find the last
\\ non-singleton, then M+1 reads and M+1 writes making subtree copies to
\\ rewrite those singletons and one non-singleton.
\\
\\ Finding the last non-singleton examines M+1 vertices, but Knuth holds the
\\ end-most vpar[n] value both in a register and in memory, so no memory
\\ read for examining it.
\\
\\ The algorithm is expressed in a callback style so that the loop iterates
\\ forests and calls to a subroutine which processes them in whatever
\\ desired way.  How much the loop might keep in registers would depend on
\\ calling conventions.  Registers which the subroutine must preserve might
\\ as well be used in the caller to hold something useful.
\\
\\ Of course if iterator state is held in an "object" in memory then there
\\ is no loop and all information needed for "next()" will be in memory in
\\ the object.  The way vpar_unlabelled_next() does it has no additional
\\ information, it just takes the vpar to step.  One possibility for extra
\\ helper information would be to record how many singletons there are, or
\\ equivalently where rewriting should start for the next step (when that
\\ happens), instead of searching through the singletons.  But the mean
\\ number of singletons 0.511 is small so such a helper is unlikely to be a
\\ speed-up.
\\

mems = 2 + 3/(d-1);
abs(mems - (3*M_limit+2)) < 1e-12 || error();

printf("mems = %.8f\n", floor_decimals(mems,8));


\\-----------------------------------------------------------------------------
