\\ Copyright 2018, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");


\\ Usage: gp <perm-to-upwards.gp
\\
\\ The number of upwards labelled forests (vpar_count_updown()) is n!, and
\\ the number of permutations of n things is n!.  Various authors give
\\ correspondences between permutations and trees or forests (sometimes in
\\ the context of "parking functions").  One correspondence is
\\
\\     Dennis Walsh, "Counting Forests with Stirling and Bell Numbers"
\\     http://frank.mtsu.edu/~dwalsh/STIRFORT.pdf
\\
\\ The rule mapping between perm and tree is
\\ 
\\      parent vpar[v] = perm[v] if perm[v]>v,
\\      otherwise try perm[perm[v]] > v,
\\      otherwise try perm[perm[perm[v]]] > v,
\\      and so on until some perm^k[v] > v
\\      
\\ so as to set vpar[v] = perm^k[v].  If v is the biggest in its cycle in
\\ perm, so nothing is >v, then vpar[v]=0 for v a root.
\\
\\ An attraction of this form is that vpar[v]=perm[v] is like taking the
\\ perm to be vpar, then massage it a bit to void cycles.  But in practice
\\ the perm[v]>v requirement is often not met, so go to perm[perm[v]] etc,
\\ and so vpar doesn't look much like perm.
\\

perm_to_forest(perm) =
{
  vector(#perm,v,
         my(p=perm[v]);
         while(1,
               if(p==v, p=0; break());  \\ nothing bigger, v is a root
               if(p>v, break());  \\ found perm bigger than v
               p=perm[p]);        \\ try next in cycle
         p); \\ parent of v for return
}

\\ Dennis Walsh's example 5 is
\\
\\         8        9
\\        / \      /|\
\\       2   6    4 5 7
\\       |          |
\\       1          3
\\
print("Example 5");
perm = [6,8,5,9,4,2,3,1,7];
vpar = [6,8,5,9,9,8,9,0,0];
print(" v = "vector(#vpar,i,i));
print("perm "perm);
print("vpar "vpar);
vpar_validate(vpar);
vpar_is_upwards(vpar)           || error();
perm_to_forest(perm) == vpar    || error();
print();

\\ And example 6, where the permutation is given as a cycle.
\\ One cycle is one component tree.
\\
\\          10
\\        / | \
\\       7  8  9
\\      /      |\
\\     4       3 6
\\               |\
\\               2 5
\\               |
\\               1
\\
print("Example 6");
install(cyc_pow_perm, "GL", "cyc_pow_perm"); \\ Pari library
cycles = [ Vecsmall([5,1,2,6,3,9,8,4,7,10]) ];
perm = cyc_pow_perm(cycles,1);               \\ cycles to perm
perm == Vecsmall([2, 6, 9, 7, 1, 3, 10, 4, 8, 5]) || error();
vpar = [2,6,9,7, 6,9,10,10,10, 0];
print(" v = "vector(#vpar,i,i));
print("perm "perm);
print("vpar "vpar);
vpar_validate(vpar);
vpar_is_upwards(vpar)           || error();
perm_to_forest(perm) == vpar    || error();

\\ It can be noted that the cycle form is a post-order traversal of the
\\ tree, with subtrees traversed from highest to lowest.
\\ vpar_seq_postorder() is post-order but goes subtrees low to high.
\\ vpar_seq_preorder() reversed is postorder high to low.
\\
\\ Post-order is since following the cycle high to low has 7 child of 10,
\\ then the subtree is further vertices <7 (which is the 4), then bigger
\\ than 7 but smaller than 10 are further children of 10, namely 8 and 9.
\\
Vecrev(vpar_seq_preorder(vpar)) == Vec(cycles[1])  || error();

\\-----------------------------------------------------------------------------
